searchData={"items":[{"type":"task","title":"mix nbx","doc":"Prints Nebulex tasks and their information.\n\n    mix nbx","ref":"Mix.Tasks.Nbx.html"},{"type":"task","title":"mix nbx.gen.cache","doc":"Generates a new cache.\n\nThe cache will be placed in the `lib` directory.\n\nFor generating a cache with the config, you can run:\n\n    mix nbx.gen.cache -c MyApp.Cache\n\nBesides, you can also specify the adapter you want to use, like so:\n\n    mix nbx.gen.cache -c MyApp.LocalCache -a Nebulex.Adapters.Local\n    mix nbx.gen.cache -c MyApp.PartitionedCache -a Nebulex.Adapters.Partitioned\n    mix nbx.gen.cache -c MyApp.ReplicatedCache -a Nebulex.Adapters.Replicated\n    mix nbx.gen.cache -c MyApp.MultilevelCache -a Nebulex.Adapters.Multilevel","ref":"Mix.Tasks.Nbx.Gen.Cache.html"},{"type":"task","title":"Command line options - mix nbx.gen.cache","doc":"* `-c`, `--cache` - The cache to generate.\n\n  * `-a`, `--adapter` - The cache adapter to use (optional).\n    Defaults to `Nebulex.Adapters.Local`.","ref":"Mix.Tasks.Nbx.Gen.Cache.html#module-command-line-options"},{"type":"module","title":"Nebulex","doc":"Nebulex is split into 2 main components:\n\n  * `Nebulex.Cache` - caches are wrappers around the in-memory data store.\n    Via the cache, we can put, get, update, delete and query existing entries.\n    A cache needs an adapter to communicate to the in-memory data store.\n\n  * `Nebulex.Caching` - Declarative annotation-based caching via\n    **`Nebulex.Caching.Decorators`**. Decorators provide n elegant way of\n    annotating functions to be cached or evicted. Caching decorators also\n    enable the usage and/or implementation of cache usage patterns like\n    **Read-through**, **Write-through**, **Cache-as-SoR**, etc.\n    See [Cache Usage Patterns Guide](http://hexdocs.pm/nebulex/cache-usage-patterns.html).\n\nIn the following sections, we will provide an overview of those components and\nhow they interact with each other. Feel free to access their respective module\ndocumentation for more specific examples, options and configuration.\n\nIf you want to quickly check a sample application using Nebulex, please check\nthe [getting started guide](http://hexdocs.pm/nebulex/getting-started.html).","ref":"Nebulex.html"},{"type":"module","title":"Caches - Nebulex","doc":"`Nebulex.Cache` is the wrapper around the Cache. We can define a\ncache as follows:\n\n    defmodule MyApp.MyCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local\n    end\n\nWhere the configuration for the Cache must be in your application\nenvironment, usually defined in your `config/config.exs`:\n\n    config :my_app, MyApp.MyCache,\n      gc_interval: 3_600_000, #=> 1 hr\n      backend: :shards,\n      partitions: 2\n\nEach cache in Nebulex defines a `start_link/1` function that needs to be\ninvoked before using the cache. In general, this function is not called\ndirectly, but used as part of your application supervision tree.\n\nIf your application was generated with a supervisor (by passing `--sup`\nto `mix new`) you will have a `lib/my_app/application.ex` file containing\nthe application start callback that defines and starts your supervisor.\nYou just need to edit the `start/2` function to start the cache as a\nsupervisor on your application's supervisor:\n\n    def start(_type, _args) do\n      children = [\n        {MyApp.Cache, []}\n      ]\n\n      opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n      Supervisor.start_link(children, opts)\n    end\n\nOtherwise, you can start and stop the cache directly at any time by calling\n`MyApp.Cache.start_link/1` and `MyApp.Cache.stop/1`.","ref":"Nebulex.html#module-caches"},{"type":"module","title":"Declarative annotation-based caching - Nebulex","doc":"See [Nebulex.Caching](http://hexdocs.pm/nebulex/Nebulex.Caching.html).","ref":"Nebulex.html#module-declarative-annotation-based-caching"},{"type":"behaviour","title":"Nebulex.Adapter","doc":"Specifies the minimal API required from adapters.","ref":"Nebulex.Adapter.html"},{"type":"macrocallback","title":"Nebulex.Adapter.__before_compile__/1","doc":"The callback invoked in case the adapter needs to inject code.","ref":"Nebulex.Adapter.html#c:__before_compile__/1"},{"type":"macro","title":"Nebulex.Adapter.defspan/3","doc":"Helper macro for the adapters so they can add the logic for emitting the\nrecommended Telemetry events.\n\nSee the built-in adapters for more information on how to use this macro.","ref":"Nebulex.Adapter.html#defspan/3"},{"type":"callback","title":"Nebulex.Adapter.init/1","doc":"Initializes the adapter supervision tree by returning the children.","ref":"Nebulex.Adapter.html#c:init/1"},{"type":"function","title":"Nebulex.Adapter.with_meta/2","doc":"Executes the function `fun` passing as parameters the adapter and metadata\n(from the `c:init/1` callback) associated with the given cache `name_or_pid`.\n\nIt expects a name or a PID representing the cache.","ref":"Nebulex.Adapter.html#with_meta/2"},{"type":"type","title":"Nebulex.Adapter.adapter_meta/0","doc":"The metadata returned by the adapter `c:init/1`.\n\nIt must be a map and Nebulex itself will always inject two keys into\nthe meta:\n\n  * `:cache` - The cache module.\n  * `:pid` - The PID returned by the child spec returned in `c:init/1`","ref":"Nebulex.Adapter.html#t:adapter_meta/0"},{"type":"type","title":"Nebulex.Adapter.metadata/0","doc":"Metadata type","ref":"Nebulex.Adapter.html#t:metadata/0"},{"type":"type","title":"Nebulex.Adapter.t/0","doc":"Adapter","ref":"Nebulex.Adapter.html#t:t/0"},{"type":"behaviour","title":"Nebulex.Adapter.Entry","doc":"Specifies the entry API required from adapters.\n\nThis behaviour specifies all read/write key-based functions,\nthe ones applied to a specific cache entry.","ref":"Nebulex.Adapter.Entry.html"},{"type":"callback","title":"Nebulex.Adapter.Entry.delete/3","doc":"Deletes a single entry from cache.\n\nSee `c:Nebulex.Cache.delete/2`.","ref":"Nebulex.Adapter.Entry.html#c:delete/3"},{"type":"callback","title":"Nebulex.Adapter.Entry.expire/3","doc":"Returns `true` if the given `key` exists and the new `ttl` was successfully\nupdated, otherwise, `false` is returned.\n\nSee `c:Nebulex.Cache.expire/2`.","ref":"Nebulex.Adapter.Entry.html#c:expire/3"},{"type":"callback","title":"Nebulex.Adapter.Entry.get/3","doc":"Gets the value for a specific `key` in `cache`.\n\nSee `c:Nebulex.Cache.get/2`.","ref":"Nebulex.Adapter.Entry.html#c:get/3"},{"type":"callback","title":"Nebulex.Adapter.Entry.get_all/3","doc":"Gets a collection of entries from the Cache, returning them as `Map.t()` of\nthe values associated with the set of keys requested.\n\nFor every key that does not hold a value or does not exist, that key is\nsimply ignored. Because of this, the operation never fails.\n\nSee `c:Nebulex.Cache.get_all/2`.","ref":"Nebulex.Adapter.Entry.html#c:get_all/3"},{"type":"callback","title":"Nebulex.Adapter.Entry.has_key?/2","doc":"Returns whether the given `key` exists in cache.\n\nSee `c:Nebulex.Cache.has_key?/1`.","ref":"Nebulex.Adapter.Entry.html#c:has_key?/2"},{"type":"callback","title":"Nebulex.Adapter.Entry.put/6","doc":"Puts the given `value` under `key` into the `cache`.\n\nReturns `true` if the `value` with key `key` is successfully inserted;\notherwise `false` is returned.\n\nThe `ttl` argument sets the time-to-live for the stored entry. If it is not\nset, it means the entry hasn't a time-to-live, then it shouldn't expire.","ref":"Nebulex.Adapter.Entry.html#c:put/6"},{"type":"callback","title":"OnWrite - Nebulex.Adapter.Entry.put/6","doc":"The `on_write` argument supports the following values:\n\n  * `:put` - If the `key` already exists, it is overwritten. Any previous\n    time-to-live associated with the key is discarded on successful `write`\n    operation.\n\n  * `:put_new` - It only stores the entry if the `key` does not already exist,\n    otherwise, `false` is returned.\n\n  * `:replace` - Alters the value stored under the given `key`, but only\n    if the key already exists into the cache, otherwise, `false` is\n    returned.\n\nSee `c:Nebulex.Cache.put/3`, `c:Nebulex.Cache.put_new/3`,\n`c:Nebulex.Cache.replace/3`.","ref":"Nebulex.Adapter.Entry.html#c:put/6-onwrite"},{"type":"callback","title":"Nebulex.Adapter.Entry.put_all/5","doc":"Puts the given `entries` (key/value pairs) into the `cache`.\n\nReturns `true` if all the keys were inserted. If no key was inserted\n(at least one key already existed), `false` is returned.\n\nThe `ttl` argument sets the time-to-live for the stored entry. If it is not\nset, it means the entry hasn't a time-to-live, then it shouldn't expire.\nThe given `ttl` is applied to all keys.","ref":"Nebulex.Adapter.Entry.html#c:put_all/5"},{"type":"callback","title":"OnWrite - Nebulex.Adapter.Entry.put_all/5","doc":"The `on_write` argument supports the following values:\n\n  * `:put` - If the `key` already exists, it is overwritten. Any previous\n    time-to-live associated with the key is discarded on successful `write`\n    operation.\n\n  * `:put_new` - It only stores the entry if the `key` does not already exist,\n    otherwise, `false` is returned.\n\nIdeally, this operation should be atomic, so all given keys are set at once.\nBut it depends purely on the adapter's implementation and the backend used\ninternally by the adapter. Hence, it is recommended to checkout the\nadapter's documentation.\n\nSee `c:Nebulex.Cache.put_all/2`.","ref":"Nebulex.Adapter.Entry.html#c:put_all/5-onwrite"},{"type":"callback","title":"Nebulex.Adapter.Entry.take/3","doc":"Returns and removes the entry with key `key` in the cache.\n\nSee `c:Nebulex.Cache.take/2`.","ref":"Nebulex.Adapter.Entry.html#c:take/3"},{"type":"callback","title":"Nebulex.Adapter.Entry.touch/2","doc":"Returns `true` if the given `key` exists and the last access time was\nsuccessfully updated, otherwise, `false` is returned.\n\nSee `c:Nebulex.Cache.touch/1`.","ref":"Nebulex.Adapter.Entry.html#c:touch/2"},{"type":"callback","title":"Nebulex.Adapter.Entry.ttl/2","doc":"Returns the TTL (time-to-live) for the given `key`. If the `key` does not\nexist, then `nil` is returned.\n\nSee `c:Nebulex.Cache.ttl/1`.","ref":"Nebulex.Adapter.Entry.html#c:ttl/2"},{"type":"callback","title":"Nebulex.Adapter.Entry.update_counter/6","doc":"Updates the counter mapped to the given `key`.\n\nIf `amount` > 0, the counter is incremented by the given `amount`.\nIf `amount` < 0, the counter is decremented by the given `amount`.\nIf `amount` == 0, the counter is not updated.\n\nSee `c:Nebulex.Cache.incr/3`.\nSee `c:Nebulex.Cache.decr/3`.","ref":"Nebulex.Adapter.Entry.html#c:update_counter/6"},{"type":"type","title":"Nebulex.Adapter.Entry.adapter_meta/0","doc":"Proxy type to the adapter meta","ref":"Nebulex.Adapter.Entry.html#t:adapter_meta/0"},{"type":"type","title":"Nebulex.Adapter.Entry.entries/0","doc":"Proxy type to the cache entries","ref":"Nebulex.Adapter.Entry.html#t:entries/0"},{"type":"type","title":"Nebulex.Adapter.Entry.key/0","doc":"Proxy type to the cache key","ref":"Nebulex.Adapter.Entry.html#t:key/0"},{"type":"type","title":"Nebulex.Adapter.Entry.on_write/0","doc":"Write command","ref":"Nebulex.Adapter.Entry.html#t:on_write/0"},{"type":"type","title":"Nebulex.Adapter.Entry.opts/0","doc":"Proxy type to the cache options","ref":"Nebulex.Adapter.Entry.html#t:opts/0"},{"type":"type","title":"Nebulex.Adapter.Entry.ttl/0","doc":"TTL for a cache entry","ref":"Nebulex.Adapter.Entry.html#t:ttl/0"},{"type":"type","title":"Nebulex.Adapter.Entry.value/0","doc":"Proxy type to the cache value","ref":"Nebulex.Adapter.Entry.html#t:value/0"},{"type":"behaviour","title":"Nebulex.Adapter.Keyslot","doc":"This behaviour provides a callback to compute the hash slot for a specific\nkey based on the number of slots (partitions, nodes, ...).\n\nThe purpose of this module is to allow users to implement a custom\nhash-slot function to distribute the keys. It can be used to select\nthe node/slot where a specific key is supposed to be.\n\n> It is highly recommended to use a **Consistent Hashing** algorithm.","ref":"Nebulex.Adapter.Keyslot.html"},{"type":"behaviour","title":"Example - Nebulex.Adapter.Keyslot","doc":"defmodule MyApp.Keyslot do\n      use Nebulex.Adapter.Keyslot\n\n      @impl true\n      def hash_slot(key, range) do\n        key\n        |> :erlang.phash2()\n        |> :jchash.compute(range)\n      end\n    end\n\nThis example uses [Jumping Consistent Hash](https://github.com/cabol/jchash).","ref":"Nebulex.Adapter.Keyslot.html#module-example"},{"type":"callback","title":"Nebulex.Adapter.Keyslot.hash_slot/2","doc":"Returns an integer within the range `0..range-1` identifying the hash slot\nthe specified `key` hashes to.","ref":"Nebulex.Adapter.Keyslot.html#c:hash_slot/2"},{"type":"callback","title":"Example - Nebulex.Adapter.Keyslot.hash_slot/2","doc":"iex> MyKeyslot.hash_slot(\"mykey\", 10)\n    2","ref":"Nebulex.Adapter.Keyslot.html#c:hash_slot/2-example"},{"type":"behaviour","title":"Nebulex.Adapter.Persistence","doc":"Specifies the adapter persistence API.","ref":"Nebulex.Adapter.Persistence.html"},{"type":"behaviour","title":"Default implementation - Nebulex.Adapter.Persistence","doc":"This module provides a default implementation that uses `File` and `Stream`\nunder-the-hood. For dumping a cache to a file, the entries are streamed from\nthe cache and written in chunks (one chunk per line), and each chunk contains\nN number of entries. For loading the entries from a file, the file is read\nand streamed line-by-line, so that the entries collected on each line are\ninserted in streaming fashion as well.\n\nThe default implementation accepts the following options only for `dump`\noperation (there are not options for `load`):\n\n  * `entries_per_line` - The number of entries to be written per line in the\n    file. Defaults to `10`.\n\n  * `compression` - The compression level. The values are the same as\n    `:erlang.term_to_binary /2`. Defaults to `6`.\n\nSee `c:Nebulex.Cache.dump/2` and `c:Nebulex.Cache.load/2` for more\ninformation.","ref":"Nebulex.Adapter.Persistence.html#module-default-implementation"},{"type":"callback","title":"Nebulex.Adapter.Persistence.dump/3","doc":"Dumps a cache to the given file `path`.\n\nReturns `:ok` if successful, or `{:error, reason}` if an error occurs.\n\nSee `c:Nebulex.Cache.dump/2`.","ref":"Nebulex.Adapter.Persistence.html#c:dump/3"},{"type":"callback","title":"Nebulex.Adapter.Persistence.load/3","doc":"Loads a dumped cache from the given `path`.\n\nReturns `:ok` if successful, or `{:error, reason}` if an error occurs.\n\nSee `c:Nebulex.Cache.load/2`.","ref":"Nebulex.Adapter.Persistence.html#c:load/3"},{"type":"behaviour","title":"Nebulex.Adapter.Queryable","doc":"Specifies the query API required from adapters.","ref":"Nebulex.Adapter.Queryable.html"},{"type":"behaviour","title":"Query values - Nebulex.Adapter.Queryable","doc":"There are two types of query values. The ones shared and implemented\nby all adapters and the ones that are adapter specific.","ref":"Nebulex.Adapter.Queryable.html#module-query-values"},{"type":"behaviour","title":"Common queries - Nebulex.Adapter.Queryable","doc":"The following query values are shared and/or supported for all adapters:\n\n  * `nil` - Matches all cached entries.","ref":"Nebulex.Adapter.Queryable.html#module-common-queries"},{"type":"behaviour","title":"Adapter-specific queries - Nebulex.Adapter.Queryable","doc":"The `query` value depends entirely on the adapter implementation; it could\nany term. Therefore, it is highly recommended to see adapters' documentation\nfor more information about building queries. For example, the built-in\n`Nebulex.Adapters.Local` adapter uses `:ets.match_spec()` for queries,\nas well as other pre-defined ones like `:unexpired` and `:expired`.","ref":"Nebulex.Adapter.Queryable.html#module-adapter-specific-queries"},{"type":"callback","title":"Nebulex.Adapter.Queryable.execute/4","doc":"Executes the `query` according to the given `operation`.\n\nRaises `Nebulex.QueryError` if query is invalid.\n\nIn the the adapter does not support the given `operation`, an `ArgumentError`\nexception should be raised.","ref":"Nebulex.Adapter.Queryable.html#c:execute/4"},{"type":"callback","title":"Operations - Nebulex.Adapter.Queryable.execute/4","doc":"* `:all` - Returns a list with all entries from cache matching the given\n    `query`.\n  * `:count_all` - Returns the number of matched entries with the given\n    `query`.\n  * `:delete_all` - Deletes all entries matching the given `query`.\n    It returns the number of deleted entries.\n\nIt is used on `c:Nebulex.Cache.all/2`, `c:Nebulex.Cache.count_all/2`,\nand `c:Nebulex.Cache.delete_all/2`.","ref":"Nebulex.Adapter.Queryable.html#c:execute/4-operations"},{"type":"callback","title":"Nebulex.Adapter.Queryable.stream/3","doc":"Streams the given `query`.\n\nRaises `Nebulex.QueryError` if query is invalid.\n\nSee `c:Nebulex.Cache.stream/2`.","ref":"Nebulex.Adapter.Queryable.html#c:stream/3"},{"type":"type","title":"Nebulex.Adapter.Queryable.adapter_meta/0","doc":"Proxy type to the adapter meta","ref":"Nebulex.Adapter.Queryable.html#t:adapter_meta/0"},{"type":"type","title":"Nebulex.Adapter.Queryable.opts/0","doc":"Proxy type to the cache options","ref":"Nebulex.Adapter.Queryable.html#t:opts/0"},{"type":"behaviour","title":"Nebulex.Adapter.Stats","doc":"Specifies the stats API required from adapters.\n\nEach adapter is responsible for providing support for stats by implementing\nthis behaviour. However, this module brings with a default implementation\nusing [Erlang counters][https://erlang.org/doc/man/counters.html], with all\ncallbacks overridable, which is supported by the built-in adapters.\n\nSee `Nebulex.Adapters.Local` for more information about how this can be used\nfrom the adapter, and also [Nebulex Telemetry Guide][telemetry_guide] to learn\nhow to use the Cache with Telemetry.\n\n[telemetry_guide]: http://hexdocs.pm/nebulex/telemetry.html","ref":"Nebulex.Adapter.Stats.html"},{"type":"function","title":"Nebulex.Adapter.Stats.incr/3","doc":"Increments the `counter`'s `stat_name` by the given `incr` value.","ref":"Nebulex.Adapter.Stats.html#incr/3"},{"type":"function","title":"Examples - Nebulex.Adapter.Stats.incr/3","doc":"Nebulex.Adapter.Stats.incr(stats_counter, :hits)\n\n    Nebulex.Adapter.Stats.incr(stats_counter, :writes, 10)\n\n> **NOTE:** This function is usually called by the adapter in case it uses\n  the default implementation; the adapter should feed `Nebulex.Stats.t()`\n  counters.\n\nSee adapters documentation for more information about stats implementation.","ref":"Nebulex.Adapter.Stats.html#incr/3-examples"},{"type":"function","title":"Nebulex.Adapter.Stats.init/1","doc":"Initializes the Erlang's counter to be used by the adapter. See the module\ndocumentation for more information about the stats default implementation.\n\nReturns `nil` is the option `:stats` is set to `false` or it is not set at\nall; the stats will be skipped.","ref":"Nebulex.Adapter.Stats.html#init/1"},{"type":"function","title":"Example - Nebulex.Adapter.Stats.init/1","doc":"Nebulex.Adapter.Stats.init(opts)\n\n> **NOTE:** This function is usually called by the adapter in case it uses\n  the default implementation; the adapter should feed `Nebulex.Stats.t()`\n  counters.\n\nSee adapters documentation for more information about stats implementation.","ref":"Nebulex.Adapter.Stats.html#init/1-example"},{"type":"callback","title":"Nebulex.Adapter.Stats.stats/1","doc":"Returns `Nebulex.Stats.t()` with the current stats values.\n\nIf the stats are disabled for the cache, then `nil` is returned.\n\nThe adapter may also include additional custom measurements,\nas well as metadata.\n\nSee `c:Nebulex.Cache.stats/0`.","ref":"Nebulex.Adapter.Stats.html#c:stats/1"},{"type":"behaviour","title":"Nebulex.Adapter.Transaction","doc":"Specifies the adapter transactions API.","ref":"Nebulex.Adapter.Transaction.html"},{"type":"behaviour","title":"Default implementation - Nebulex.Adapter.Transaction","doc":"This module also provides a default implementation which uses the Erlang\nlibrary `:global`.\n\nThis implementation accepts the following options:\n\n  * `:keys` - The list of the keys that will be locked. Since the lock id is\n    generated based on the key, if this option is not set, a fixed/constant\n    lock id is used to perform the transaction, then all further transactions\n    (without this option set) are serialized and the performance is affected\n    significantly. For that reason it is recommended to pass the list of keys\n    involved in the transaction.\n\n  * `:nodes` - The list of the nodes where the lock will be set, or on\n    all nodes if none are specified.\n\n  * `:retries` - If the key has been locked by other process already, and\n    `:retries` is not equal to 0, the process sleeps for a while and tries\n    to execute the action later. When `:retries` attempts have been made,\n    an exception is raised. If `:retries` is `:infinity` (the default),\n    the function will eventually be executed (unless the lock is never\n    released).\n\nLet's see an example:\n\n    MyCache.transaction fn ->\n      counter = MyCache.get(:counter)\n      MyCache.set(:counter, counter + 1)\n    end\n\nLocking only the involved key (recommended):\n\n    MyCache.transaction [keys: [:counter]], fn ->\n      counter = MyCache.get(:counter)\n      MyCache.set(:counter, counter + 1)\n    end\n\n    MyCache.transaction [keys: [:alice, :bob]], fn ->\n      alice = MyCache.get(:alice)\n      bob = MyCache.get(:bob)\n      MyCache.set(:alice, %{alice | balance: alice.balance + 100})\n      MyCache.set(:bob, %{bob | balance: bob.balance + 100})\n    end","ref":"Nebulex.Adapter.Transaction.html#module-default-implementation"},{"type":"callback","title":"Nebulex.Adapter.Transaction.in_transaction?/1","doc":"Returns `true` if the given process is inside a transaction.\n\nSee `c:Nebulex.Cache.in_transaction?/0`.","ref":"Nebulex.Adapter.Transaction.html#c:in_transaction?/1"},{"type":"callback","title":"Nebulex.Adapter.Transaction.transaction/3","doc":"Runs the given function inside a transaction.\n\nA successful transaction returns the value returned by the function.\n\nSee `c:Nebulex.Cache.transaction/2`.","ref":"Nebulex.Adapter.Transaction.html#c:transaction/3"},{"type":"module","title":"Nebulex.Adapters.Local","doc":"Adapter module for Local Generational Cache; inspired by\n[epocxy](https://github.com/duomark/epocxy).\n\nGenerational caching using an ets table (or multiple ones when used with\n`:shards`) for each generation of cached data. Accesses hit the newer\ngeneration first, and migrate from the older generation to the newer\ngeneration when retrieved from the stale table. When a new generation\nis started, the oldest one is deleted. This is a form of mass garbage\ncollection which avoids using timers and expiration of individual\ncached elements.\n\nThis implementation of generation cache uses only two generations\n(which is more than enough) also referred like the `newer` and\nthe `older`.","ref":"Nebulex.Adapters.Local.html"},{"type":"module","title":"Overall features - Nebulex.Adapters.Local","doc":"* Configurable backend (`ets` or `:shards`).\n  * Expiration – A status based on TTL (Time To Live) option. To maintain\n    cache performance, expired entries may not be immediately removed or\n    evicted, they are expired or evicted on-demand, when the key is read.\n  * Eviction – [Generational Garbage Collection][gc].\n  * Sharding – For intensive workloads, the Cache may also be partitioned\n    (by using `:shards` backend and specifying the `:partitions` option).\n  * Support for transactions via Erlang global name registration facility.\n  * Support for stats.\n\n[gc]: http://hexdocs.pm/nebulex/Nebulex.Adapters.Local.Generation.html","ref":"Nebulex.Adapters.Local.html#module-overall-features"},{"type":"module","title":"Options - Nebulex.Adapters.Local","doc":"This adapter supports the following options and all of them can be given via\nthe cache configuration:\n\n  * `:backend` - Defines the backend or storage to be used for the adapter.\n    Supported backends are: `:ets` and `:shards`. Defaults to `:ets`.\n\n  * `:read_concurrency` - (boolean) Since this adapter uses ETS tables\n    internally, this option is used when a new table is created; see\n    `:ets.new/2`. Defaults to `true`.\n\n  * `:write_concurrency` - (boolean) Since this adapter uses ETS tables\n    internally, this option is used when a new table is created; see\n    `:ets.new/2`. Defaults to `true`.\n\n  * `:compressed` - (boolean) This option is used when a new ETS table is\n    created and it defines whether or not it includes X as an option; see\n    `:ets.new/2`. Defaults to `false`.\n\n  * `:backend_type` - This option defines the type of ETS to be used\n    (Defaults to `:set`). However, it is highly recommended to keep the\n    default value, since there are commands not supported (unexpected\n    exception may be raised) for types like `:bag` or `: duplicate_bag`.\n    Please see the [ETS](https://erlang.org/doc/man/ets.html) docs\n    for more information.\n\n  * `:partitions` - If it is set, an integer > 0 is expected, otherwise,\n    it defaults to `System.schedulers_online()`. This option is only\n    available for `:shards` backend.\n\n  * `:gc_interval` - If it is set, an integer > 0 is expected defining the\n    interval time in milliseconds to garbage collection to run, delete the\n    oldest generation and create a new one. If this option is not set,\n    garbage collection is never executed, so new generations must be\n    created explicitly, e.g.: `MyCache.new_generation(opts)`.\n\n  * `:max_size` - If it is set, an integer > 0 is expected defining the\n    max number of cached entries (cache limit). If it is not set (`nil`),\n    the check to release memory is not performed (the default).\n\n  * `:allocated_memory` - If it is set, an integer > 0 is expected defining\n    the max size in bytes allocated for a cache generation. When this option\n    is set and the configured value is reached, a new cache generation is\n    created so the oldest is deleted and force releasing memory space.\n    If it is not set (`nil`), the cleanup check to release memory is\n    not performed (the default).\n\n  * `:gc_cleanup_min_timeout` - An integer > 0 defining the min timeout in\n    milliseconds for triggering the next cleanup and memory check. This will\n    be the timeout to use when either the max size or max allocated memory\n    is reached. Defaults to `10_000` (10 seconds).\n\n  * `:gc_cleanup_max_timeout` - An integer > 0 defining the max timeout in\n    milliseconds for triggering the next cleanup and memory check. This is\n    the timeout used when the cache starts and there are few entries or the\n    consumed memory is near to `0`. Defaults to `600_000` (10 minutes).\n\n  * `:gc_flush_delay` - If it is set, an integer > 0 is expected defining the\n    delay in milliseconds before objects from the oldest generation are\n    flushed. Defaults to `10_000` (10 seconds).","ref":"Nebulex.Adapters.Local.html#module-options"},{"type":"module","title":"Usage - Nebulex.Adapters.Local","doc":"`Nebulex.Cache` is the wrapper around the cache. We can define a\nlocal cache as follows:\n\n    defmodule MyApp.LocalCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local\n    end\n\nWhere the configuration for the cache must be in your application\nenvironment, usually defined in your `config/config.exs`:\n\n    config :my_app, MyApp.LocalCache,\n      gc_interval: :timer.hours(12),\n      max_size: 1_000_000,\n      allocated_memory: 2_000_000_000,\n      gc_cleanup_min_timeout: :timer.seconds(10),\n      gc_cleanup_max_timeout: :timer.minutes(10)\n\nFor intensive workloads, the Cache may also be partitioned using `:shards`\nas cache backend (`backend: :shards`) and configuring the desired number of\npartitions via the `:partitions` option. Defaults to\n`System.schedulers_online()`.\n\n    config :my_app, MyApp.LocalCache,\n      gc_interval: :timer.hours(12),\n      max_size: 1_000_000,\n      allocated_memory: 2_000_000_000,\n      gc_cleanup_min_timeout: :timer.seconds(10),\n      gc_cleanup_max_timeout: :timer.minutes(10),\n      backend: :shards,\n      partitions: System.schedulers_online() * 2\n\nIf your application was generated with a supervisor (by passing `--sup`\nto `mix new`) you will have a `lib/my_app/application.ex` file containing\nthe application start callback that defines and starts your supervisor.\nYou just need to edit the `start/2` function to start the cache as a\nsupervisor on your application's supervisor:\n\n    def start(_type, _args) do\n      children = [\n        {MyApp.LocalCache, []},\n        ...\n      ]\n\nSee `Nebulex.Cache` for more information.","ref":"Nebulex.Adapters.Local.html#module-usage"},{"type":"module","title":"Eviction configuration - Nebulex.Adapters.Local","doc":"This section is to understand a bit better how the different configuration\noptions work and have an idea what values to set; especially if it is the\nfirst time using Nebulex.\n\n### `:ttl` option\n\nThe `:ttl` option that is used to set the expiration time for a key, it\ndoesn't work as eviction mechanism, since the local adapter implements a\ngenerational cache, the options that control the eviction process are:\n`:gc_interval`, `:gc_cleanup_min_timeout`, `:gc_cleanup_max_timeout`,\n`:max_size` and `:allocated_memory`. The `:ttl` is evaluated on-demand\nwhen a key is retrieved, and at that moment if it s expired, then remove\nit from the cache, hence, it can not be used as eviction method, it is\nmore for keep the integrity and consistency in the cache. For this reason,\nit is highly recommended to configure always the eviction options mentioned\nbefore.","ref":"Nebulex.Adapters.Local.html#module-eviction-configuration"},{"type":"module","title":"Caveats when using `:ttl` option: - Nebulex.Adapters.Local","doc":"* When using the `:ttl` option, ensure it is less than `:gc_interval`,\n    otherwise, there may be a situation where the key is evicted and the\n    `:ttl` hasn't happened yet (maybe because the garbage collector ran\n    before the key had been fetched).\n  * Assuming you have `:gc_interval` set to 2 hrs, then you put a new key\n    with `:ttl` set to 1 hr, and 1 minute later the GC runs, that key will\n    be moved to the older generation so it can be yet retrieved. On the other\n    hand, if the key is never fetched till the next GC cycle (causing moving\n    it to the newer generation), since the key is already in the oldest\n    generation it will be evicted from the cache so it won't be retrievable\n    anymore.","ref":"Nebulex.Adapters.Local.html#module-caveats-when-using-ttl-option"},{"type":"module","title":"Garbage collection or eviction options - Nebulex.Adapters.Local","doc":"This adapter implements a generational cache, which means its main eviction\nmechanism is pushing a new cache generation and remove the oldest one. In\nthis way, we ensure only the most frequently used keys are always available\nin the newer generation and the the least frequently used are evicted when\nthe garbage collector runs, and the garbage collector is triggered upon\nthese conditions:\n\n  * When the time interval defined by `:gc_interval` is completed.\n    This makes the garbage-collector process to run creating a new\n    generation and forcing to delete the oldest one.\n  * When the \"cleanup\" timeout expires, and then the limits `:max_size`\n    and `:allocated_memory` are checked, if one of those is reached,\n    then the garbage collector runs (a new generation is created and\n    the oldest one is deleted). The cleanup timeout is controlled by\n    `:gc_cleanup_min_timeout` and `:gc_cleanup_max_timeout`, it works\n    with an inverse linear backoff, which means the timeout is inverse\n    proportional to the memory growth; the bigger the cache size is,\n    the shorter the cleanup timeout will be.","ref":"Nebulex.Adapters.Local.html#module-garbage-collection-or-eviction-options"},{"type":"module","title":"First-time configuration - Nebulex.Adapters.Local","doc":"For configuring the cache with accurate and/or good values it is important\nto know several things in advance, like for example the size of an entry\nin average so we can calculate a good value for max size and/or allocated\nmemory, how intensive will be the load in terms of reads and writes, etc.\nThe problem is most of these aspects are unknown when it is a new app or\nwe are using the cache for the first time. Therefore, the following\nrecommendations will help you to configure the cache for the first time:\n\n  * When configuring the `:gc_interval`, think about how that often the\n    least frequently used entries should be evicted, or what is the desired\n    retention period for the cached entries. For example, if `:gc_interval`\n    is set to 1 hr, it means you will keep in cache only those entries that\n    are retrieved periodically within a 2 hr period; `gc_interval * 2`,\n    being 2 the number of generations. Longer than that, the GC will\n    ensure is always evicted (the oldest generation is always deleted).\n    If it is the first time using Nebulex, perhaps you can start with\n    `gc_interval: :timer.hours(12)` (12 hrs), so the max retention\n    period for the keys will be 1 day; but ensure you also set either the\n    `:max_size` or `:allocated_memory`.\n  * It is highly recommended to set either `:max_size` or `:allocated_memory`\n    to ensure the oldest generation is deleted (least frequently used keys\n    are evicted) when one of these limits is reached and also to avoid\n    running out of memory. For example, for the `:allocated_memory` we can\n    set 25% of the total memory, and for the `:max_size` something between\n    `100_000` and `1_000_000`.\n  * For `:gc_cleanup_min_timeout` we can set `10_000`, which means when the\n    cache is reaching the size or memory limit, the polling period for the\n    cleanup process will be 10 seconds. And for `:gc_cleanup_max_timeout`\n    we can set `600_000`, which means when the cache is almost empty the\n    polling period will be close to 10 minutes.","ref":"Nebulex.Adapters.Local.html#module-first-time-configuration"},{"type":"module","title":"Stats - Nebulex.Adapters.Local","doc":"This adapter does support stats by using the default implementation\nprovided by `Nebulex.Adapter.Stats`. The adapter also uses the\n`Nebulex.Telemetry.StatsHandler` to aggregate the stats and keep\nthem updated. Therefore, it requires the Telemetry events are emitted\nby the adapter (the `:telemetry` option should not be set to `false`\nso the Telemetry events can be dispatched), otherwise, stats won't\nwork properly.","ref":"Nebulex.Adapters.Local.html#module-stats"},{"type":"module","title":"Queryable API - Nebulex.Adapters.Local","doc":"Since this adapter is implemented on top of ETS tables, the query must be\na valid match spec given by `:ets.match_spec()`. However, there are some\npredefined and/or shorthand queries you can use. See the section\n\"Predefined queries\" below for for information.\n\nInternally, an entry is represented by the tuple\n`{:entry, key, value, touched, ttl}`, which means the match pattern within\nthe `:ets.match_spec()` must be something like:\n`{:entry, :\"$1\", :\"$2\", :\"$3\", :\"$4\"}`.\nIn order to make query building easier, you can use `Ex2ms` library.","ref":"Nebulex.Adapters.Local.html#module-queryable-api"},{"type":"module","title":"Predefined queries - Nebulex.Adapters.Local","doc":"* `nil` - All keys are returned.\n\n  * `:unexpired` -  All unexpired keys/entries.\n\n  * `:expired` -  All expired keys/entries.\n\n  * `{:in, [term]}` - Only the keys in the given key list (`[term]`)\n    are returned. This predefined query is only supported for\n    `c:Nebulex.Cache.delete_all/2`. This is the recommended\n    way of doing bulk delete of keys.","ref":"Nebulex.Adapters.Local.html#module-predefined-queries"},{"type":"module","title":"Examples - Nebulex.Adapters.Local","doc":"# built-in queries\n    MyCache.all()\n    MyCache.all(:unexpired)\n    MyCache.all(:expired)\n    MyCache.all({:in, [\"foo\", \"bar\"]})\n\n    # using a custom match spec (all values > 10)\n    spec = [{{:_, :\"$1\", :\"$2\", :_, :_}, [{:>, :\"$2\", 10}], [{{:\"$1\", :\"$2\"}}]}]\n    MyCache.all(spec)\n\n    # using Ex2ms\n    import Ex2ms\n\n    spec =\n      fun do\n        {_, key, value, _, _} when value > 10 -> {key, value}\n      end\n\n    MyCache.all(spec)\n\nThe `:return` option applies only for built-in queries, such as:\n`nil | :unexpired | :expired`, if you are using a custom `:ets.match_spec()`,\nthe return value depends on it.\n\nThe same applies to the `stream` function.","ref":"Nebulex.Adapters.Local.html#module-examples"},{"type":"module","title":"Extended API (convenience functions) - Nebulex.Adapters.Local","doc":"This adapter provides some additional convenience functions to the\n`Nebulex.Cache` API.\n\nCreating new generations:\n\n    MyCache.new_generation()\n    MyCache.new_generation(reset_timer: false)\n\nRetrieving the current generations:\n\n    MyCache.generations()\n\nRetrieving the newer generation:\n\n    MyCache.newer_generation()","ref":"Nebulex.Adapters.Local.html#module-extended-api-convenience-functions"},{"type":"macro","title":"Nebulex.Adapters.Local.entry/1","doc":"","ref":"Nebulex.Adapters.Local.html#entry/1"},{"type":"macro","title":"Nebulex.Adapters.Local.entry/2","doc":"","ref":"Nebulex.Adapters.Local.html#entry/2"},{"type":"function","title":"Nebulex.Adapters.Local.get_/2","doc":"","ref":"Nebulex.Adapters.Local.html#get_/2"},{"type":"function","title":"Nebulex.Adapters.Local.take_/2","doc":"","ref":"Nebulex.Adapters.Local.html#take_/2"},{"type":"module","title":"Nebulex.Adapters.Local.Generation","doc":"Generational garbage collection process.\n\nThe generational garbage collector manage the heap as several sub-heaps,\nknown as generations, based on age of the objects. An object is allocated\nin the youngest generation, sometimes called the nursery, and is promoted\nto an older generation if its lifetime exceeds the threshold of its current\ngeneration (defined by option `:gc_interval`). Every time the GC runs\n(triggered by `:gc_interval` timeout), a new cache generation is created\nand the oldest one is deleted.\n\nThe deletion of the oldest generation happens in two steps. First, the\nunderlying ets table is flushed to release space and only marked for deletion\nas there may still be processes referencing it. The actual deletion of the\nets table happens at next GC run.\n\nHowever, flushing is a blocking operation, once started, processes wanting\nto access the table will need to wait until it finishes. To circumvent this,\nflushing can be delayed by configuring `:gc_flush_delay` to allow time for\nthese processes to finish their work without being accidentally blocked.\n\nThe only way to create new generations is through this module (this server\nis the metadata owner) calling `new/2` function. When a Cache is created,\na generational garbage collector is attached to it automatically,\ntherefore, this server MUST NOT be started directly.","ref":"Nebulex.Adapters.Local.Generation.html"},{"type":"module","title":"Options - Nebulex.Adapters.Local.Generation","doc":"These options are configured through the `Nebulex.Adapters.Local` adapter:\n\n  * `:gc_interval` - If it is set, an integer > 0 is expected defining the\n    interval time in milliseconds to garbage collection to run, delete the\n    oldest generation and create a new one. If this option is not set,\n    garbage collection is never executed, so new generations must be\n    created explicitly, e.g.: `MyCache.new_generation(opts)`.\n\n  * `:max_size` - If it is set, an integer > 0 is expected defining the\n    max number of cached entries (cache limit). If it is not set (`nil`),\n    the check to release memory is not performed (the default).\n\n  * `:allocated_memory` - If it is set, an integer > 0 is expected defining\n    the max size in bytes allocated for a cache generation. When this option\n    is set and the configured value is reached, a new cache generation is\n    created so the oldest is deleted and force releasing memory space.\n    If it is not set (`nil`), the cleanup check to release memory is\n    not performed (the default).\n\n  * `:gc_cleanup_min_timeout` - An integer > 0 defining the min timeout in\n    milliseconds for triggering the next cleanup and memory check. This will\n    be the timeout to use when either the max size or max allocated memory\n    is reached. Defaults to `10_000` (10 seconds).\n\n  * `:gc_cleanup_max_timeout` - An integer > 0 defining the max timeout in\n    milliseconds for triggering the next cleanup and memory check. This is\n    the timeout used when the cache starts and there are few entries or the\n    consumed memory is near to `0`. Defaults to `600_000` (10 minutes).\n\n  * `:gc_flush_delay` - If it is set, an integer > 0 is expected defining the\n    delay in milliseconds before objects from the oldest generation are\n    flushed. Defaults to `10_000` (10 seconds).","ref":"Nebulex.Adapters.Local.Generation.html#module-options"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Nebulex.Adapters.Local.Generation.html#child_spec/1"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.delete_all/1","doc":"Removes or flushes all entries from the cache (including all its generations).","ref":"Nebulex.Adapters.Local.Generation.html#delete_all/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.delete_all/1","doc":"Nebulex.Adapters.Local.Generation.delete_all(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#delete_all/1-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.get_state/1","doc":"A convenience function for retrieving the state.","ref":"Nebulex.Adapters.Local.Generation.html#get_state/1"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.list/1","doc":"Returns the list of the generations in the form `[newer, older]`.","ref":"Nebulex.Adapters.Local.Generation.html#list/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.list/1","doc":"Nebulex.Adapters.Local.Generation.list(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#list/1-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.memory_info/1","doc":"Returns the memory info in a tuple form `{used_mem, total_mem}`.","ref":"Nebulex.Adapters.Local.Generation.html#memory_info/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.memory_info/1","doc":"Nebulex.Adapters.Local.Generation.memory_info(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#memory_info/1-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.new/2","doc":"Creates a new cache generation. Once the max number of generations\nis reached, when a new generation is created, the oldest one is\ndeleted.","ref":"Nebulex.Adapters.Local.Generation.html#new/2"},{"type":"function","title":"Options - Nebulex.Adapters.Local.Generation.new/2","doc":"* `:reset_timer` - Indicates if the poll frequency time-out should\n    be reset or not (default: true).","ref":"Nebulex.Adapters.Local.Generation.html#new/2-options"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.new/2","doc":"Nebulex.Adapters.Local.Generation.new(MyCache)\n\n    Nebulex.Adapters.Local.Generation.new(MyCache, reset_timer: false)","ref":"Nebulex.Adapters.Local.Generation.html#new/2-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.newer/1","doc":"Returns the newer generation.","ref":"Nebulex.Adapters.Local.Generation.html#newer/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.newer/1","doc":"Nebulex.Adapters.Local.Generation.newer(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#newer/1-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.realloc/2","doc":"Reallocates the block of memory that was previously allocated for the given\n`server_ref` with the new `size`. In other words, reallocates the max memory\nsize for a cache generation.","ref":"Nebulex.Adapters.Local.Generation.html#realloc/2"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.realloc/2","doc":"Nebulex.Adapters.Local.Generation.realloc(MyCache, 1_000_000)","ref":"Nebulex.Adapters.Local.Generation.html#realloc/2-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.reset_timer/1","doc":"Resets the timer for pushing new cache generations.","ref":"Nebulex.Adapters.Local.Generation.html#reset_timer/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.reset_timer/1","doc":"Nebulex.Adapters.Local.Generation.reset_timer(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#reset_timer/1-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.server/1","doc":"Returns the PID of the GC server for the given `server_ref`.","ref":"Nebulex.Adapters.Local.Generation.html#server/1"},{"type":"function","title":"Example - Nebulex.Adapters.Local.Generation.server/1","doc":"Nebulex.Adapters.Local.Generation.server(MyCache)","ref":"Nebulex.Adapters.Local.Generation.html#server/1-example"},{"type":"function","title":"Nebulex.Adapters.Local.Generation.start_link/1","doc":"Starts the garbage collector for the built-in local cache adapter.","ref":"Nebulex.Adapters.Local.Generation.html#start_link/1"},{"type":"type","title":"Nebulex.Adapters.Local.Generation.opts/0","doc":"","ref":"Nebulex.Adapters.Local.Generation.html#t:opts/0"},{"type":"type","title":"Nebulex.Adapters.Local.Generation.server_ref/0","doc":"","ref":"Nebulex.Adapters.Local.Generation.html#t:server_ref/0"},{"type":"type","title":"Nebulex.Adapters.Local.Generation.t/0","doc":"","ref":"Nebulex.Adapters.Local.Generation.html#t:t/0"},{"type":"module","title":"Nebulex.Adapters.Multilevel","doc":"Adapter module for Multi-level Cache.\n\nThis is just a simple layer on top of local or distributed cache\nimplementations that enables to have a cache hierarchy by levels.\nMulti-level caches generally operate by checking the fastest,\nlevel 1 (L1) cache first; if it hits, the adapter proceeds at\nhigh speed. If that first cache misses, the next fastest cache\n(level 2, L2) is checked, and so on, before accessing external\nmemory (that can be handled by a `cacheable` decorator).\n\nFor write functions, the \"Write Through\" policy is applied by default;\nthis policy ensures that the data is stored safely as it is written\nthroughout the hierarchy. However, it is possible to force the write\noperation in a specific level (although it is not recommended) via\n`level` option, where the value is a positive integer greater than 0.\n\nWe can define a multi-level cache as follows:\n\n    defmodule MyApp.Multilevel do\n      use Nebulex.Cache,\n        otp_app: :nebulex,\n        adapter: Nebulex.Adapters.Multilevel\n\n      defmodule L1 do\n        use Nebulex.Cache,\n          otp_app: :nebulex,\n          adapter: Nebulex.Adapters.Local\n      end\n\n      defmodule L2 do\n        use Nebulex.Cache,\n          otp_app: :nebulex,\n          adapter: Nebulex.Adapters.Partitioned\n      end\n    end\n\nWhere the configuration for the cache and its levels must be in your\napplication environment, usually defined in your `config/config.exs`:\n\n    config :my_app, MyApp.Multilevel,\n      model: :inclusive,\n      levels: [\n        {\n          MyApp.Multilevel.L1,\n          gc_interval: :timer.hours(12),\n          backend: :shards\n        },\n        {\n          MyApp.Multilevel.L2,\n          primary: [\n            gc_interval: :timer.hours(12),\n            backend: :shards\n          ]\n        }\n      ]\n\nIf your application was generated with a supervisor (by passing `--sup`\nto `mix new`) you will have a `lib/my_app/application.ex` file containing\nthe application start callback that defines and starts your supervisor.\nYou just need to edit the `start/2` function to start the cache as a\nsupervisor on your application's supervisor:\n\n    def start(_type, _args) do\n      children = [\n        {MyApp.Multilevel, []},\n        ...\n      ]\n\nSee `Nebulex.Cache` for more information.","ref":"Nebulex.Adapters.Multilevel.html"},{"type":"module","title":"Options - Nebulex.Adapters.Multilevel","doc":"This adapter supports the following options and all of them can be given via\nthe cache configuration:\n\n  * `:levels` - This option is to define the levels, a list of tuples\n    `{cache_level :: Nebulex.Cache.t(), opts :: Keyword.t()}`, where\n    the first element is the module that defines the cache for that\n    level, and the second one is the options that will be passed to\n    that level in the `start/link/1` (which depends on the adapter\n    this level is using). The order in which the levels are defined\n    is the same the multi-level cache will use. For example, the first\n    cache in the list will be the L1 cache (level 1) and so on;\n    the Nth element will be the LN cache. This option is mandatory,\n    if it is not set or empty, an exception will be raised.\n\n  * `:model` - Specifies the cache model: `:inclusive` or `:exclusive`;\n    defaults to `:inclusive`. In an inclusive cache, the same data can be\n    present in all caches/levels. In an exclusive cache, data can be present\n    in only one cache/level and a key cannot be found in the rest of caches\n    at the same time. This option applies to the `get` callback only; if the\n    cache `:model` is `:inclusive`, when the key is found in a level N,\n    that entry is duplicated backwards (to all previous levels: 1..N-1).\n    However, when the mode is set to `:inclusive`, the `get_all` operation\n    is translated into multiple `get` calls underneath (which may be a\n    significant performance penalty) since is required to replicate the\n    entries properly with their current TTLs. It is possible to skip the\n    replication when calling `get_all` using the option `:replicate`.\n\n  * `:replicate` - This option applies only to the `get_all` callback.\n    Determines whether the entries should be replicated to the backward\n    levels or not. Defaults to `true`.","ref":"Nebulex.Adapters.Multilevel.html#module-options"},{"type":"module","title":"Shared options - Nebulex.Adapters.Multilevel","doc":"Almost all of the cache functions outlined in `Nebulex.Cache` module\naccept the following options:\n\n  * `:level` - It may be an integer greater than 0 that specifies the cache\n    level where the operation will take place. By default, the evaluation\n    is performed throughout the whole cache hierarchy (all levels).","ref":"Nebulex.Adapters.Multilevel.html#module-shared-options"},{"type":"module","title":"Telemetry events - Nebulex.Adapters.Multilevel","doc":"This adapter emits all recommended Telemetry events, and documented\nin `Nebulex.Cache` module (see **\"Adapter-specific events\"** section).\n\nSince the multi-level adapter is a layer/wrapper on top of other existing\nadapters, each cache level may Telemetry emit events independently.\nFor example, for the cache defined before `MyApp.Multilevel`, the next\nevents will be emitted for the main multi-level cache:\n\n  * `[:my_app, :multilevel, :command, :start]`\n  * `[:my_app, :multilevel, :command, :stop]`\n  * `[:my_app, :multilevel, :command, :exception]`\n\nFor the L1 (configured with the local adapter):\n\n  * `[:my_app, :multilevel, :l1, :command, :start]`\n  * `[:my_app, :multilevel, :l1, :command, :stop]`\n  * `[:my_app, :multilevel, :l1, :command, :exception]`\n\nFor the L2 (configured with the partitioned adapter):\n\n  * `[:my_app, :multilevel, :l2, :command, :start]`\n  * `[:my_app, :multilevel, :l2, :primary, :command, :start]`\n  * `[:my_app, :multilevel, :l2, :command, :stop]`\n  * `[:my_app, :multilevel, :l2, :primary, :command, :stop]`\n  * `[:my_app, :multilevel, :l2, :command, :exception]`\n  * `[:my_app, :multilevel, :l2, :primary, :command, :exception]`\n\nSee also the [Telemetry guide](http://hexdocs.pm/nebulex/telemetry.html)\nfor more information and examples.","ref":"Nebulex.Adapters.Multilevel.html#module-telemetry-events"},{"type":"module","title":"Stats - Nebulex.Adapters.Multilevel","doc":"Since the multi-level adapter works as a wrapper for the configured cache\nlevels, the support for stats depends on the underlying levels. Also, the\nmeasurements are consolidated per level, they are not aggregated. For example,\nif we enable the stats for the multi-level cache defined previously and run:\n\n    MyApp.Multilevel.stats()\n\nThe returned stats will look like:\n\n    %Nebulex.Stats{\n      measurements: %{\n        l1: %{evictions: 0, expirations: 0, hits: 0, misses: 0, writes: 0},\n        l2: %{evictions: 0, expirations: 0, hits: 0, misses: 0, writes: 0}\n      },\n      metadata: %{\n        l1: %{\n          cache: NMyApp.Multilevel.L1,\n          started_at: ~U[2021-01-10 13:06:04.075084Z]\n        },\n        l2: %{\n          cache: MyApp.Multilevel.L2.Primary,\n          started_at: ~U[2021-01-10 13:06:04.089888Z]\n        },\n        cache: MyApp.Multilevel,\n        started_at: ~U[2021-01-10 13:06:04.066750Z]\n      }\n    }\n\n**IMPORTANT:** Those cache levels with stats disabled won't be included\ninto the returned stats (they are skipped). If a cache level is using\nan adapter that does not support stats, you may get unexpected errors.\nTherefore, and as overall recommendation, check out the documentation\nfor adapters used by the underlying cache levels and ensure they\nimplement the `Nebulex.Adapter.Stats` behaviour.","ref":"Nebulex.Adapters.Multilevel.html#module-stats"},{"type":"module","title":"Stats with Telemetry - Nebulex.Adapters.Multilevel","doc":"In case you are using Telemetry metrics, you can define the metrics per\nlevel, for example:\n\n    last_value(\"nebulex.cache.stats.l1.hits\",\n      event_name: \"nebulex.cache.stats\",\n      measurement: &get_in(&1, [:l1, :hits]),\n      tags: [:cache]\n    )\n    last_value(\"nebulex.cache.stats.l1.misses\",\n      event_name: \"nebulex.cache.stats\",\n      measurement: &get_in(&1, [:l1, :misses]),\n      tags: [:cache]\n    )\n\n> See the section **\"Instrumenting Multi-level caches\"** in the\n  [Telemetry guide](http://hexdocs.pm/nebulex/telemetry.html)\n  for more information.","ref":"Nebulex.Adapters.Multilevel.html#module-stats-with-telemetry"},{"type":"module","title":"Extended API - Nebulex.Adapters.Multilevel","doc":"This adapter provides one additional convenience function for retrieving\nthe cache model for the given cache `name`:\n\n    MyCache.model()\n    MyCache.model(:cache_name)","ref":"Nebulex.Adapters.Multilevel.html#module-extended-api"},{"type":"module","title":"Caveats of multi-level adapter - Nebulex.Adapters.Multilevel","doc":"Because this adapter reuses other existing/configured adapters, it inherits\nall their limitations too. Therefore, it is highly recommended to check the\ndocumentation of the adapters to use.","ref":"Nebulex.Adapters.Multilevel.html#module-caveats-of-multi-level-adapter"},{"type":"module","title":"Nebulex.Adapters.Nil","doc":"The **Nil adapter** is a special cache adapter that disables the cache;\nit loses all the items saved on it and it returns `nil` for all the read\nand `true` for all save operations. This adapter is mostly useful for tests.","ref":"Nebulex.Adapters.Nil.html"},{"type":"module","title":"Example - Nebulex.Adapters.Nil","doc":"Suppose you have an application using Ecto for database access and Nebulex\nfor caching. Then, you have defined a cache and a repo within it. Since you\nare using a database, there might be some cases you may want to disable the\ncache to avoid issues when running the test, for example, in some test cases,\nwhen accessing the database you expect no data at all, but you could retrieve\nthe data from cache anyway because maybe it was cached in a previous test.\nTherefore, you have to delete all entries from the cache before to run each\ntest to make sure the cache is always empty. This is where the Nil adapter\ncomes in, instead of adding code to flush the cache before each test, you\ncould define a test cache using the Nil adapter for the tests.\n\nOne one hand, you have defined the cache in your application within\n`lib/my_app/cache.ex`:\n\n    defmodule MyApp.Cache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local\n    end\n\nAnd on the other hand, in the tests you have defined the test cache within\n`test/support/test_cache.ex`:\n\n    defmodule MyApp.TestCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Nil\n    end\n\nNow, we have to tell the app what cache to use depending on the environment,\nfor tests we want `MyApp.TestCache`, otherwise it is always `MyApp.Cache`.\nWe can do this very easy by introducing a new config parameter to decide\nwhat cache module to use. For tests you can define the config\n`config/test.exs`:\n\n    config :my_app,\n      nebulex_cache: MyApp.TestCache,\n      ...\n\nThe final piece is to read the config parameter and start the cache properly.\nWithin `lib/my_app/application.ex` you could have:\n\n    def start(_type, _args) do\n      children = [\n        {Application.get_env(:my_app, :nebulex_cache, MyApp.Cache), []},\n      ]\n\n      ...\n\nAs you can see, by default `MyApp.Cache` is always used, unless the\n`:nebulex_cache` option points to a different module, which will be\nwhen tests are executed (`:test` env).","ref":"Nebulex.Adapters.Nil.html#module-example"},{"type":"module","title":"Nebulex.Adapters.Partitioned","doc":"Built-in adapter for partitioned cache topology.","ref":"Nebulex.Adapters.Partitioned.html"},{"type":"module","title":"Overall features - Nebulex.Adapters.Partitioned","doc":"* Partitioned cache topology (Sharding Distribution Model).\n  * Configurable primary storage adapter.\n  * Configurable Keyslot to distributed the keys across the cluster members.\n  * Support for transactions via Erlang global name registration facility.\n  * Stats support rely on the primary storage adapter.","ref":"Nebulex.Adapters.Partitioned.html#module-overall-features"},{"type":"module","title":"Partitioned Cache Topology - Nebulex.Adapters.Partitioned","doc":"There are several key points to consider about a partitioned cache:\n\n  * _**Partitioned**_: The data in a distributed cache is spread out over\n    all the servers in such a way that no two servers are responsible for\n    the same piece of cached data. This means that the size of the cache\n    and the processing power associated with the management of the cache\n    can grow linearly with the size of the cluster. Also, it means that\n    operations against data in the cache can be accomplished with a\n    \"single hop,\" in other words, involving at most one other server.\n\n  * _**Load-Balanced**_:  Since the data is spread out evenly over the\n    servers, the responsibility for managing the data is automatically\n    load-balanced across the cluster.\n\n  * _**Ownership**_: Exactly one node in the cluster is responsible for each\n    piece of data in the cache.\n\n  * _**Point-To-Point**_: The communication for the partitioned cache is all\n    point-to-point, enabling linear scalability.\n\n  * _**Location Transparency**_: Although the data is spread out across\n    cluster nodes, the exact same API is used to access the data, and the\n    same behavior is provided by each of the API methods. This is called\n    location transparency, which means that the developer does not have to\n    code based on the topology of the cache, since the API and its behavior\n    will be the same with a local cache, a replicated cache, or a distributed\n    cache.\n\n  * _**Failover**_: Failover of a distributed cache involves promoting backup\n    data to be primary storage. When a cluster node fails, all remaining\n    cluster nodes determine what data each holds in backup that the failed\n    cluster node had primary responsible for when it died. Those data becomes\n    the responsibility of whatever cluster node was the backup for the data.\n    However, this adapter does not provide fault-tolerance implementation,\n    each piece of data is kept in a single node/machine (via sharding), then,\n    if a node fails, the data kept by this node won't be available for the\n    rest of the cluster members.\n\n> Based on **\"Distributed Caching Essential Lessons\"** by **Cameron Purdy**\n  and [Coherence Partitioned Cache Service][oracle-pcs].\n\n[oracle-pcs]: https://docs.oracle.com/cd/E13924_01/coh.340/e13819/partitionedcacheservice.htm","ref":"Nebulex.Adapters.Partitioned.html#module-partitioned-cache-topology"},{"type":"module","title":"Additional implementation notes - Nebulex.Adapters.Partitioned","doc":"`:pg2` or `:pg` (>= OTP 23) is used under-the-hood by the adapter to manage\nthe cluster nodes. When the partitioned cache is started in a node, it creates\na group and joins it (the cache supervisor PID is joined to the group). Then,\nwhen a function is invoked, the adapter picks a node from the group members,\nand then the function is executed on that specific node. In the same way,\nwhen a partitioned cache supervisor dies (the cache is stopped or killed for\nsome reason), the PID of that process is automatically removed from the PG\ngroup; this is why it's recommended to use consistent hashing for distributing\nthe keys across the cluster nodes.\n\n> **NOTE:** `pg2` will be replaced by `pg` in future, since the `pg2` module\n  is deprecated as of OTP 23 and scheduled for removal in OTP 24.\n\nThis adapter depends on a local cache adapter (primary storage), it adds\na thin layer on top of it in order to distribute requests across a group\nof nodes, where is supposed the local cache is running already. However,\nyou don't need to define any additional cache module for the primary\nstorage, instead, the adapter initializes it automatically (it adds the\nprimary storage as part of the supervision tree) based on the given\noptions within the `primary_storage_adapter:` argument.","ref":"Nebulex.Adapters.Partitioned.html#module-additional-implementation-notes"},{"type":"module","title":"Usage - Nebulex.Adapters.Partitioned","doc":"When used, the Cache expects the `:otp_app` and `:adapter` as options.\nThe `:otp_app` should point to an OTP application that has the cache\nconfiguration. For example:\n\n    defmodule MyApp.PartitionedCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Partitioned\n    end\n\nOptionally, you can configure the desired primary storage adapter with the\noption `:primary_storage_adapter`; defaults to `Nebulex.Adapters.Local`.\n\n    defmodule MyApp.PartitionedCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Partitioned,\n        primary_storage_adapter: Nebulex.Adapters.Local\n    end\n\nAlso, you can provide a custom keyslot function:\n\n    defmodule MyApp.PartitionedCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Partitioned,\n        primary_storage_adapter: Nebulex.Adapters.Local\n\n      @behaviour Nebulex.Adapter.Keyslot\n\n      @impl true\n      def hash_slot(key, range) do\n        key\n        |> :erlang.phash2()\n        |> :jchash.compute(range)\n      end\n    end\n\nWhere the configuration for the cache must be in your application environment,\nusually defined in your `config/config.exs`:\n\n    config :my_app, MyApp.PartitionedCache,\n      keyslot: MyApp.PartitionedCache,\n      primary: [\n        gc_interval: 3_600_000,\n        backend: :shards\n      ]\n\nIf your application was generated with a supervisor (by passing `--sup`\nto `mix new`) you will have a `lib/my_app/application.ex` file containing\nthe application start callback that defines and starts your supervisor.\nYou just need to edit the `start/2` function to start the cache as a\nsupervisor on your application's supervisor:\n\n    def start(_type, _args) do\n      children = [\n        {MyApp.PartitionedCache, []},\n        ...\n      ]\n\nSee `Nebulex.Cache` for more information.","ref":"Nebulex.Adapters.Partitioned.html#module-usage"},{"type":"module","title":"Options - Nebulex.Adapters.Partitioned","doc":"This adapter supports the following options and all of them can be given via\nthe cache configuration:\n\n  * `:primary` - The options that will be passed to the adapter associated\n    with the local primary storage. These options will depend on the local\n    adapter to use.\n\n  * `:keyslot` - Defines the module implementing `Nebulex.Adapter.Keyslot`\n    behaviour.\n\n  * `:task_supervisor_opts` - Start-time options passed to\n    `Task.Supervisor.start_link/1` when the adapter is initialized.\n\n  * `:join_timeout` - Interval time in milliseconds for joining the\n    running partitioned cache to the cluster. This is to ensure it is\n    always joined. Defaults to `:timer.seconds(180)`.","ref":"Nebulex.Adapters.Partitioned.html#module-options"},{"type":"module","title":"Shared options - Nebulex.Adapters.Partitioned","doc":"Almost all of the cache functions outlined in `Nebulex.Cache` module\naccept the following options:\n\n  * `:timeout` - The time-out value in milliseconds for the command that\n    will be executed. If the timeout is exceeded, then the current process\n    will exit. For executing a command on remote nodes, this adapter uses\n    `Task.await/2` internally for receiving the result, so this option tells\n    how much time the adapter should wait for it. If the timeout is exceeded,\n    the task is shut down but the current process doesn't exit, only the\n    result associated with that task is skipped in the reduce phase.","ref":"Nebulex.Adapters.Partitioned.html#module-shared-options"},{"type":"module","title":"Telemetry events - Nebulex.Adapters.Partitioned","doc":"This adapter emits all recommended Telemetry events, and documented\nin `Nebulex.Cache` module (see **\"Adapter-specific events\"** section).\n\nSince the partitioned adapter depends on the configured primary storage\nadapter (local cache adapter), this one may also emit Telemetry events.\nTherefore, there will be events emitted by the partitioned adapter as well\nas the primary storage adapter. For example, for the cache defined before\n`MyApp.PartitionedCache`, these would be the emitted events:\n\n  * `[:my_app, :partitioned_cache, :command, :start]`\n  * `[:my_app, :partitioned_cache, :primary, :command, :start]`\n  * `[:my_app, :partitioned_cache, :command, :stop]`\n  * `[:my_app, :partitioned_cache, :primary, :command, :stop]`\n  * `[:my_app, :partitioned_cache, :command, :exception]`\n  * `[:my_app, :partitioned_cache, :primary, :command, :exception]`\n\nAs you may notice, the telemetry prefix by default for the partitioned cache\nis `[:my_app, :partitioned_cache]`, and the prefix for its primary storage\n`[:my_app, :partitioned_cache, :primary]`.\n\nSee also the [Telemetry guide](http://hexdocs.pm/nebulex/telemetry.html)\nfor more information and examples.","ref":"Nebulex.Adapters.Partitioned.html#module-telemetry-events"},{"type":"module","title":"Adapter-specific telemetry events - Nebulex.Adapters.Partitioned","doc":"This adapter exposes following Telemetry events:\n\n  * `telemetry_prefix ++ [:bootstrap, :started]` - Dispatched by the adapter\n    when the bootstrap process is started.\n\n    * Measurements: `%{system_time: non_neg_integer}`\n    * Metadata:\n\n      ```\n      %{\n        adapter_meta: %{optional(atom) => term},\n        cluster_nodes: [node]\n      }\n      ```\n\n  * `telemetry_prefix ++ [:bootstrap, :stopped]` - Dispatched by the adapter\n    when the bootstrap process is stopped.\n\n    * Measurements: `%{system_time: non_neg_integer}`\n    * Metadata:\n\n      ```\n      %{\n        adapter_meta: %{optional(atom) => term},\n        cluster_nodes: [node],\n        reason: term\n      }\n      ```\n\n  * `telemetry_prefix ++ [:bootstrap, :exit]` - Dispatched by the adapter\n    when the bootstrap has received an exit signal.\n\n    * Measurements: `%{system_time: non_neg_integer}`\n    * Metadata:\n\n      ```\n      %{\n        adapter_meta: %{optional(atom) => term},\n        cluster_nodes: [node],\n        reason: term\n      }\n      ```\n\n  * `telemetry_prefix ++ [:bootstrap, :joined]` - Dispatched by the adapter\n    when the bootstrap has joined the cache to the cluster.\n\n    * Measurements: `%{system_time: non_neg_integer}`\n    * Metadata:\n\n      ```\n      %{\n        adapter_meta: %{optional(atom) => term},\n        cluster_nodes: [node]\n      }\n      ```","ref":"Nebulex.Adapters.Partitioned.html#module-adapter-specific-telemetry-events"},{"type":"module","title":"Stats - Nebulex.Adapters.Partitioned","doc":"This adapter depends on the primary storage adapter for the stats support.\nTherefore, it is important to ensure the underlying primary storage adapter\ndoes support stats, otherwise, you may get unexpected errors.","ref":"Nebulex.Adapters.Partitioned.html#module-stats"},{"type":"module","title":"Extended API - Nebulex.Adapters.Partitioned","doc":"This adapter provides some additional convenience functions to the\n`Nebulex.Cache` API.\n\nRetrieving the primary storage or local cache module:\n\n    MyCache.__primary__()\n\nRetrieving the cluster nodes associated with the given cache `name`:\n\n    MyCache.nodes()\n\nGet a cluster node based on the given `key`:\n\n    MyCache.get_node(\"mykey\")\n\nJoining the cache to the cluster:\n\n    MyCache.join_cluster()\n\nLeaving the cluster (removes the cache from the cluster):\n\n    MyCache.leave_cluster()","ref":"Nebulex.Adapters.Partitioned.html#module-extended-api"},{"type":"module","title":"Caveats of partitioned adapter - Nebulex.Adapters.Partitioned","doc":"For `c:Nebulex.Cache.get_and_update/3` and `c:Nebulex.Cache.update/4`,\nthey both have a parameter that is the anonymous function, and it is compiled\ninto the module where it is created, which means it necessarily doesn't exists\non remote nodes. To ensure they work as expected, you must provide functions\nfrom modules existing in all nodes of the group.","ref":"Nebulex.Adapters.Partitioned.html#module-caveats-of-partitioned-adapter"},{"type":"function","title":"Nebulex.Adapters.Partitioned.do_put_all/4","doc":"","ref":"Nebulex.Adapters.Partitioned.html#do_put_all/4"},{"type":"function","title":"Nebulex.Adapters.Partitioned.eval_stream/3","doc":"Helper to perform `stream/3` locally.","ref":"Nebulex.Adapters.Partitioned.html#eval_stream/3"},{"type":"function","title":"Nebulex.Adapters.Partitioned.with_dynamic_cache/3","doc":"Helper function to use dynamic cache for internal primary cache storage\nwhen needed.","ref":"Nebulex.Adapters.Partitioned.html#with_dynamic_cache/3"},{"type":"module","title":"Nebulex.Adapters.Replicated","doc":"Built-in adapter for replicated cache topology.","ref":"Nebulex.Adapters.Replicated.html"},{"type":"module","title":"Overall features - Nebulex.Adapters.Replicated","doc":"* Replicated cache topology.\n  * Configurable primary storage adapter.\n  * Cache-level locking when deleting all entries or adding new nodes.\n  * Key-level (or entry-level) locking for key-based write-like operations.\n  * Support for transactions via Erlang global name registration facility.\n  * Stats support rely on the primary storage adapter.","ref":"Nebulex.Adapters.Replicated.html#module-overall-features"},{"type":"module","title":"Replicated Cache Topology - Nebulex.Adapters.Replicated","doc":"A replicated cache is a clustered, fault tolerant cache where data is fully\nreplicated to every member in the cluster. This cache offers the fastest read\nperformance with linear performance scalability for reads but poor scalability\nfor writes (as writes must be processed by every member in the cluster).\nBecause data is replicated to all servers, adding servers does not increase\naggregate cache capacity.\n\nThere are several challenges to building a reliably replicated cache. The\nfirst is how to get it to scale and perform well. Updates to the cache have\nto be sent to all cluster nodes, and all cluster nodes have to end up with\nthe same data, even if multiple updates to the same piece of data occur at\nthe same time. Also, if a cluster node requests a lock, ideally it should\nnot have to get all cluster nodes to agree on the lock or at least do it in\na very efficient way (`:global` is used here), otherwise it will scale\nextremely poorly; yet in the case of a cluster node failure, all of the data\nand lock information must be kept safely.\n\nThe best part of a replicated cache is its access speed. Since the data is\nreplicated to each cluster node, it is available for use without any waiting.\nThis is referred to as \"zero latency access,\" and is perfect for situations\nin which an application requires the highest possible speed in its data\naccess.\n\nHowever, there are some limitations:\n\n  * _**Cost Per Update**_ - Updating a replicated cache requires pushing\n    the new version of the data to all other cluster members, which will\n    limit scalability if there is a high frequency of updates per member.\n\n  * _**Cost Per Entry**_ - The data is replicated to every cluster member,\n    so Memory Heap space is used on each member, which will impact\n    performance for large caches.\n\n> Based on **\"Distributed Caching Essential Lessons\"** by **Cameron Purdy**.","ref":"Nebulex.Adapters.Replicated.html#module-replicated-cache-topology"},{"type":"module","title":"Usage - Nebulex.Adapters.Replicated","doc":"When used, the Cache expects the `:otp_app` and `:adapter` as options.\nThe `:otp_app` should point to an OTP application that has the cache\nconfiguration. For example:\n\n    defmodule MyApp.ReplicatedCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Replicated\n    end\n\nOptionally, you can configure the desired primary storage adapter with the\noption `:primary_storage_adapter`; defaults to `Nebulex.Adapters.Local`.\n\n    defmodule MyApp.ReplicatedCache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Replicated,\n        primary_storage_adapter: Nebulex.Adapters.Local\n    end\n\nThe configuration for the cache must be in your application environment,\nusually defined in your `config/config.exs`:\n\n    config :my_app, MyApp.ReplicatedCache,\n      primary: [\n        gc_interval: 3_600_000,\n        backend: :shards\n      ]\n\nIf your application was generated with a supervisor (by passing `--sup`\nto `mix new`) you will have a `lib/my_app/application.ex` file containing\nthe application start callback that defines and starts your supervisor.\nYou just need to edit the `start/2` function to start the cache as a\nsupervisor on your application's supervisor:\n\n    def start(_type, _args) do\n      children = [\n        {MyApp.ReplicatedCache, []},\n        ...\n      ]\n\nSee `Nebulex.Cache` for more information.","ref":"Nebulex.Adapters.Replicated.html#module-usage"},{"type":"module","title":"Options - Nebulex.Adapters.Replicated","doc":"This adapter supports the following options and all of them can be given via\nthe cache configuration:\n\n  * `:primary` - The options that will be passed to the adapter associated\n    with the local primary storage. These options will depend on the local\n    adapter to use.\n\n  * `:task_supervisor_opts` - Start-time options passed to\n    `Task.Supervisor.start_link/1` when the adapter is initialized.","ref":"Nebulex.Adapters.Replicated.html#module-options"},{"type":"module","title":"Shared options - Nebulex.Adapters.Replicated","doc":"Almost all of the cache functions outlined in `Nebulex.Cache` module\naccept the following options:\n\n  * `:timeout` - The time-out value in milliseconds for the command that\n    will be executed. If the timeout is exceeded, then the current process\n    will exit. For executing a command on remote nodes, this adapter uses\n    `Task.await/2` internally for receiving the result, so this option tells\n    how much time the adapter should wait for it. If the timeout is exceeded,\n    the task is shut down but the current process doesn't exit, only the\n    result associated with that task is skipped in the reduce phase.","ref":"Nebulex.Adapters.Replicated.html#module-shared-options"},{"type":"module","title":"Telemetry events - Nebulex.Adapters.Replicated","doc":"This adapter emits all recommended Telemetry events, and documented\nin `Nebulex.Cache` module (see **\"Adapter-specific events\"** section).\n\nSince the replicated adapter depends on the configured primary storage\nadapter (local cache adapter), this one may also emit Telemetry events.\nTherefore, there will be events emitted by the replicated adapter as well\nas the primary storage adapter. For example, for the cache defined before\n`MyApp.ReplicatedCache`, these would be the emitted events:\n\n  * `[:my_app, :replicated_cache, :command, :start]`\n  * `[:my_app, :replicated_cache, :primary, :command, :start]`\n  * `[:my_app, :replicated_cache, :command, :stop]`\n  * `[:my_app, :replicated_cache, :primary, :command, :stop]`\n  * `[:my_app, :replicated_cache, :command, :exception]`\n  * `[:my_app, :replicated_cache, :primary, :command, :exception]`\n\nAs you may notice, the telemetry prefix by default for the replicated cache\nis `[:my_app, :replicated_cache]`, and the prefix for its primary storage\n`[:my_app, :replicated_cache, :primary]`.\n\nSee also the [Telemetry guide](http://hexdocs.pm/nebulex/telemetry.html)\nfor more information and examples.","ref":"Nebulex.Adapters.Replicated.html#module-telemetry-events"},{"type":"module","title":"Stats - Nebulex.Adapters.Replicated","doc":"This adapter depends on the primary storage adapter for the stats support.\nTherefore, it is important to ensure the underlying primary storage adapter\ndoes support stats, otherwise, you may get unexpected errors.","ref":"Nebulex.Adapters.Replicated.html#module-stats"},{"type":"module","title":"Extended API - Nebulex.Adapters.Replicated","doc":"This adapter provides some additional convenience functions to the\n`Nebulex.Cache` API.\n\nRetrieving the primary storage or local cache module:\n\n    MyCache.__primary__()\n\nRetrieving the cluster nodes associated with the given cache name:\n\n    MyCache.nodes()\n\nJoining the cache to the cluster:\n\n    MyCache.join_cluster()\n\nLeaving the cluster (removes the cache from the cluster):\n\n    MyCache.leave_cluster()","ref":"Nebulex.Adapters.Replicated.html#module-extended-api"},{"type":"module","title":"Adapter-specific telemetry events - Nebulex.Adapters.Replicated","doc":"This adapter exposes following Telemetry events:\n\n  * `telemetry_prefix ++ [:replication]` - Dispatched by the adapter\n    when a replication error occurs due to a write-like operation\n    under-the-hood.\n\n    * Measurements: `%{rpc_errors: non_neg_integer}`\n    * Metadata:\n\n      ```\n      %{\n        adapter_meta: %{optional(atom) => term},\n        rpc_errors: [{node, error :: term}]\n      }\n      ```\n\n  * `telemetry_prefix ++ [:bootstrap]` - Dispatched by the adapter at start\n    time when there are errors while syncing up with the cluster nodes.\n\n    * Measurements:\n\n      ```\n      %{\n        failed_nodes: non_neg_integer,\n        remote_errors: non_neg_integer\n      }\n      ```\n\n    * Metadata:\n\n      ```\n      %{\n        adapter_meta: %{optional(atom) => term},\n        failed_nodes: [node],\n        remote_errors: [term]\n      }\n      ```","ref":"Nebulex.Adapters.Replicated.html#module-adapter-specific-telemetry-events"},{"type":"module","title":"Caveats of replicated adapter - Nebulex.Adapters.Replicated","doc":"As it is explained in the beginning, a replicated topology not only brings\nwith advantages (mostly for reads) but also with some limitations and\nchallenges.\n\nThis adapter uses global locks (via `:global`) for all operation that modify\nor alter the cache somehow to ensure as much consistency as possible across\nall members of the cluster. These locks may be per key or for the entire cache\ndepending on the operation taking place. For that reason, it is very important\nto be aware about those operation that can potentially lead to performance and\nscalability issues, so that you can do a better usage of the replicated\nadapter. The following is with the operations and aspects you should pay\nattention to:\n\n  * Starting and joining a new replicated node to the cluster is the most\n    expensive action, because all write-like operations across all members of\n    the cluster are blocked until the new node completes the synchronization\n    process, which involves copying cached data from any of the existing\n    cluster nodes into the new node, and this could be very expensive\n    depending on the number of caches entries. For that reason, adding new\n    nodes is considered an expensive operation that should happen only from\n    time to time.\n\n  * Deleting all entries. When `c:Nebulex.Cache.delete_all/2` action is\n    executed, like in the previous case, all write-like operations in all\n    members of the cluster are blocked until the deletion action is completed\n    (this implies deleting all cached data from all cluster nodes). Therefore,\n    deleting all entries from cache is also considered an expensive operation\n    that should happen only from time to time.\n\n  * Write-like operations based on a key only block operations related to\n    that key across all members of the cluster. This is not as critical as\n    the previous two cases but it is something to keep in mind anyway because\n    if there is a highly demanded key in terms of writes, that could be also\n    a potential bottleneck.\n\nSumming up, the replicated cache topology along with this adapter should\nbe used mainly when the the reads clearly dominate over the writes (e.g.:\nReads 80% and Writes 20% or less). Besides, operations like deleting all\nentries from cache or adding new nodes must be executed only once in a while\nto avoid performance issues, since they are very expensive.","ref":"Nebulex.Adapters.Replicated.html#module-caveats-of-replicated-adapter"},{"type":"function","title":"Nebulex.Adapters.Replicated.with_dynamic_cache/3","doc":"Helper function to use dynamic cache for internal primary cache storage\nwhen needed.","ref":"Nebulex.Adapters.Replicated.html#with_dynamic_cache/3"},{"type":"behaviour","title":"Nebulex.Cache","doc":"Cache's main interface; defines the cache abstraction layer which is\nhighly inspired by [Ecto](https://github.com/elixir-ecto/ecto).\n\nA Cache maps to an underlying implementation, controlled by the\nadapter. For example, Nebulex ships with a default adapter that\nimplements a local generational cache.\n\nWhen used, the Cache expects the `:otp_app` and `:adapter` as options.\nThe `:otp_app` should point to an OTP application that has the cache\nconfiguration. For example, the Cache:\n\n    defmodule MyApp.Cache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local\n    end\n\nCould be configured with:\n\n    config :my_app, MyApp.Cache,\n      backend: :shards,\n      gc_interval: :timer.hours(12),\n      max_size: 1_000_000,\n      allocated_memory: 2_000_000_000,\n      gc_cleanup_min_timeout: :timer.seconds(10),\n      gc_cleanup_max_timeout: :timer.minutes(10)\n\nMost of the configuration that goes into the `config` is specific\nto the adapter. For this particular example, you can check\n[`Nebulex.Adapters.Local`](https://hexdocs.pm/nebulex/Nebulex.Adapters.Local.html)\nfor more information. In spite of this, the following configuration values\nare shared across all adapters:\n\n  * `:name` - The name of the Cache supervisor process.\n\n  * `:telemetry_prefix` - It is recommend for adapters to publish events\n    using the `Telemetry` library. By default, the telemetry prefix is based\n    on the module name, so if your module is called `MyApp.Cache`, the prefix\n    will be `[:my_app, :cache]`. See the \"Telemetry events\" section to see\n    what events recommended for the  adapters to publish.. Note that if you\n    have multiple caches, you should keep the `:telemetry_prefix` consistent\n    for each of them and use the `:cache` and/or `:name` (in case of a named\n    or dynamic cache) properties in the event metadata for distinguishing\n    between caches.\n\n  * `:telemetry` - An optional flag to tell the adapters whether Telemetry\n    events should be emitted or not. Defaults to `true`.\n\n  * `:stats` - Boolean to define whether or not the cache will provide stats.\n    Defaults to `false`. Each adapter is responsible for providing stats by\n    implementing `Nebulex.Adapter.Stats` behaviour. See the \"Stats\" section\n    below.","ref":"Nebulex.Cache.html"},{"type":"behaviour","title":"Telemetry events - Nebulex.Cache","doc":"Similar to Ecto or Phoenix, Nebulex also provides built-in Telemetry events\napplied to all caches, and cache adapter-specific events.","ref":"Nebulex.Cache.html#module-telemetry-events"},{"type":"behaviour","title":"Nebulex built-in events - Nebulex.Cache","doc":"The following events are emitted by all Nebulex caches:\n\n  * `[:nebulex, :cache, :init]` - it is dispatched whenever a cache starts.\n    The measurement is a single `system_time` entry in native unit. The\n    metadata is the `:cache` and all initialization options under `:opts`.","ref":"Nebulex.Cache.html#module-nebulex-built-in-events"},{"type":"behaviour","title":"Adapter-specific events - Nebulex.Cache","doc":"It is recommend the adapters to publish certain `Telemetry` events listed\nbelow. Those events will use the `:telemetry_prefix` outlined above which\ndefaults to `[:my_app, :cache]`.\n\nFor instance, to receive all events published by a cache called `MyApp.Cache`,\none could define a module:\n\n    defmodule MyApp.Telemetry do\n      def handle_event(\n            [:my_app, :cache, :command, event],\n            measurements,\n            metadata,\n            config\n          ) do\n        case event do\n          :start ->\n            # Handle start event ...\n\n          :stop ->\n            # Handle stop event ...\n\n          :exception ->\n            # Handle exception event ...\n        end\n      end\n    end\n\nThen, in the `Application.start/2` callback, attach the handler to this event\nusing a unique handler id:\n\n    :telemetry.attach(\n      \"my-app-handler-id\",\n      [:my_app, :cache, :command],\n      &MyApp.Telemetry.handle_event/4,\n      %{}\n    )\n\nSee [the telemetry documentation](https://hexdocs.pm/telemetry/)\nfor more information.\n\nThe following are the events you should expect from Nebulex. All examples\nbelow consider a cache named `MyApp.Cache`:\n\n#### `[:my_app, :cache, :command, :start]`\n\nThis event should be invoked on every cache call sent to the adapter before\nthe command logic is executed.\n\nThe `:measurements` map will include the following:\n\n  * `:system_time` - The current system time in native units from calling:\n    `System.system_time()`.\n\nA Telemetry `:metadata` map including the following fields. Each cache adapter\nmay emit different information here. For built-in adapters, it will contain:\n\n  * `:adapter_meta` - The adapter metadata.\n  * `:function_name` - The name of the invoked adapter function.\n  * `:args` - The arguments of the invoked adapter function, omitting the\n    first argument, since it is the adapter metadata already included into\n    the event's metadata.\n\n#### `[:my_app, :cache, :command, :stop]`\n\nThis event should be invoked on every cache call sent to the adapter after\nthe command logic is executed.\n\nThe `:measurements` map will include the following:\n\n  * `:duration` - The time spent executing the cache command. The measurement\n    is given in the `:native` time unit. You can read more about it in the\n    docs for `System.convert_time_unit/3`.\n\nA Telemetry `:metadata` map including the following fields. Each cache adapter\nmay emit different information here. For built-in adapters, it will contain:\n\n  * `:adapter_meta` - The adapter metadata.\n  * `:function_name` - The name of the invoked adapter function.\n  * `:args` - The arguments of the invoked adapter function, omitting the\n    first argument, since it is the adapter metadata already included into\n    the event's metadata.\n  * `:result` - The command result.\n\n#### `[:my_app, :cache, :command, :exception]`\n\nThis event should be invoked when an error or exception occurs while executing\nthe cache command.\n\nThe `:measurements` map will include the following:\n\n  * `:duration` - The time spent executing the cache command. The measurement\n    is given in the `:native` time unit. You can read more about it in the\n    docs for `System.convert_time_unit/3`.\n\nA Telemetry `:metadata` map including the following fields. Each cache adapter\nmay emit different information here. For built-in adapters, it will contain:\n\n  * `:adapter_meta` - The adapter metadata.\n  * `:function_name` - The name of the invoked adapter function.\n  * `:args` - The arguments of the invoked adapter function, omitting the\n    first argument, since it is the adapter metadata already included into\n    the event's metadata.\n  * `:kind` - The type of the error: `:error`, `:exit`, or `:throw`.\n  * `:reason` - The reason of the error.\n  * `:stacktrace` - The stacktrace.\n\n**NOTE:** The events outlined above are the recommended for the adapters\nto dispatch. However, it is highly recommended to review the used adapter\ndocumentation to ensure it is fully compatible with these events, perhaps\ndifferences, or perhaps also additional events.","ref":"Nebulex.Cache.html#module-adapter-specific-events"},{"type":"behaviour","title":"Stats - Nebulex.Cache","doc":"Stats are provided by the adapters by implementing the optional behaviour\n`Nebulex.Adapter.Stats`. This behaviour exposes a callback to return the\ncurrent cache stats.  Nevertheless, the behaviour brings with a default\nimplementation using [Erlang counters][counters], which is used by the\nlocal built-in adapter (`Nebulex.Adapters.Local`).\n\n[counters]: https://erlang.org/doc/man/counters.html\n\nOne can enable the stats by setting the option `:stats` to `true`.\nFor example, in the configuration file:\n\n    config :my_app, MyApp.Cache,\n      stats: true,\n      ...\n\n> Remember to check if the underlying adapter implements the\n  `Nebulex.Adapter.Stats` behaviour.\n\nSee `c:Nebulex.Cache.stats/0` for more information.","ref":"Nebulex.Cache.html#module-stats"},{"type":"behaviour","title":"Dispatching stats via Telemetry - Nebulex.Cache","doc":"It is possible to emit Telemetry events for the current stats via\n`c:Nebulex.Cache.dispatch_stats/1`, but it has to be invoked explicitly;\nNebulex does not emit this Telemetry event automatically. But it is very\neasy to emit this event using [`:telemetry_poller`][telemetry_poller].\n\n[telemetry_poller]: https://github.com/beam-telemetry/telemetry_poller\n\nFor example, one can define a custom pollable measurement:\n\n    :telemetry_poller.start_link(\n      measurements: [\n        {MyApp.Cache, :dispatch_stats, []},\n      ],\n      # configure sampling period - default is :timer.seconds(5)\n      period: :timer.seconds(30),\n      name: :my_cache_stats_poller\n    )\n\nOr you can also start the `:telemetry_poller` process along with your\napplication supervision tree:\n\n    def start(_type, _args) do\n      my_cache_stats_poller_opts = [\n        measurements: [\n          {MyApp.Cache, :dispatch_stats, []},\n        ],\n        period: :timer.seconds(30),\n        name: :my_cache_stats_poller\n      ]\n\n      children = [\n        {MyApp.Cache, []},\n        {:telemetry_poller, my_cache_stats_poller_opts}\n      ]\n\n      opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n      Supervisor.start_link(children, opts)\n    end\n\nSee [Nebulex Telemetry Guide](http://hexdocs.pm/nebulex/telemetry.html)\nfor more information.","ref":"Nebulex.Cache.html#module-dispatching-stats-via-telemetry"},{"type":"behaviour","title":"Distributed topologies - Nebulex.Cache","doc":"Nebulex provides the following adapters for distributed topologies:\n\n  * `Nebulex.Adapters.Partitioned` - Partitioned cache topology.\n  * `Nebulex.Adapters.Replicated` - Replicated cache topology.\n  * `Nebulex.Adapters.Multilevel` - Multi-level distributed cache topology.\n\nThese adapters work more as wrappers for an existing local adapter and provide\nthe distributed topology on top of it. Optionally, you can set the adapter for\nthe primary cache storage with the option `:primary_storage_adapter`. Defaults\nto `Nebulex.Adapters.Local`.","ref":"Nebulex.Cache.html#module-distributed-topologies"},{"type":"callback","title":"Nebulex.Cache.__adapter__/0","doc":"Returns the adapter tied to the cache.","ref":"Nebulex.Cache.html#c:__adapter__/0"},{"type":"callback","title":"Nebulex.Cache.__default_key_generator__/0","doc":"Returns the default key generator applied only when using\n**\"declarative annotation-based caching\"** via `Nebulex.Caching.Decorators`.\n\nSometimes you may want to set a different key generator when using\ndeclarative caching. By default, the key generator is set to\n`Nebulex.Caching.SimpleKeyGenerator`. You can change the default\nkey generator at compile time with:\n\n    use Nebulex.Cache, default_key_generator: MyKeyGenerator\n\nSee `Nebulex.Caching.Decorators` and `Nebulex.Caching.KeyGenerator`\nfor more information.","ref":"Nebulex.Cache.html#c:__default_key_generator__/0"},{"type":"callback","title":"Nebulex.Cache.all/2","doc":"Fetches all entries from cache matching the given `query`.\n\nMay raise `Nebulex.QueryError` if query validation fails.","ref":"Nebulex.Cache.html#c:all/2"},{"type":"callback","title":"Query values - Nebulex.Cache.all/2","doc":"There are two types of query values. The ones shared and implemented\nby all adapters and the ones that are adapter specific.","ref":"Nebulex.Cache.html#c:all/2-query-values"},{"type":"callback","title":"Common queries - Nebulex.Cache.all/2","doc":"The following query values are shared and/or supported for all adapters:\n\n  * `nil` - Returns a list with all cached entries based on the `:return`\n    option.","ref":"Nebulex.Cache.html#c:all/2-common-queries"},{"type":"callback","title":"Adapter-specific queries - Nebulex.Cache.all/2","doc":"The `query` value depends entirely on the adapter implementation; it could\nany term. Therefore, it is highly recommended to see adapters' documentation\nfor more information about building queries. For example, the built-in\n`Nebulex.Adapters.Local` adapter uses `:ets.match_spec()` for queries,\nas well as other pre-defined ones like `:unexpired` and `:expired`.","ref":"Nebulex.Cache.html#c:all/2-adapter-specific-queries"},{"type":"callback","title":"Options - Nebulex.Cache.all/2","doc":"* `:return` - Tells the query what to return from the matched entries.\n    See the possible values in the \"Query return option\" section below.\n    The default depends on the adapter, for example, the default for the\n    built-in adapters is `:key`. This option is supported by the built-in\n    adapters, but it is recommended to see the adapter's documentation\n    to confirm its compatibility with this option.\n\nSee the configured adapter documentation for more runtime options.","ref":"Nebulex.Cache.html#c:all/2-options"},{"type":"callback","title":"Query return option - Nebulex.Cache.all/2","doc":"The following are the possible values for the `:return` option:\n\n  * `:key` - Returns a list only with the keys.\n  * `:value` - Returns a list only with the values.\n  * `:entry` - Returns a list of `t:Nebulex.Entry.t/0`.\n  * `{:key, :value}` - Returns a list of tuples in the form `{key, value}`.\n\nSee adapters documentation to confirm what of these options are supported\nand what other added.","ref":"Nebulex.Cache.html#c:all/2-query-return-option"},{"type":"callback","title":"Example - Nebulex.Cache.all/2","doc":"Populate the cache with some entries:\n\n    iex> :ok = Enum.each(1..5, &MyCache.put(&1, &1 * 2))\n\nFetch all (with default params):\n\n    iex> MyCache.all()\n    [1, 2, 3, 4, 5]\n\nFetch all entries and return values:\n\n    iex> MyCache.all(nil, return: :value)\n    [2, 4, 6, 8, 10]\n\nFetch all entries and return them as key/value pairs:\n\n    iex> MyCache.all(nil, return: {:key, :value})\n    [{1, 2}, {2, 4}, {3, 6}, {4, 8}, {5, 10}]\n\nFetch all entries that match with the given query assuming we are using\n`Nebulex.Adapters.Local` adapter:\n\n    iex> query = [{{:_, :\"$1\", :\"$2\", :_, :_}, [{:>, :\"$2\", 5}], [:\"$1\"]}]\n    iex> MyCache.all(query)\n    [3, 4, 5]","ref":"Nebulex.Cache.html#c:all/2-example"},{"type":"callback","title":"Query - Nebulex.Cache.all/2","doc":"Query spec is defined by the adapter, hence, it is recommended to review\nadapters documentation. For instance, the built-in `Nebulex.Adapters.Local`\nadapter supports `nil | :unexpired | :expired | :ets.match_spec()` as query\nvalue.","ref":"Nebulex.Cache.html#c:all/2-query"},{"type":"callback","title":"Examples - Nebulex.Cache.all/2","doc":"Additional built-in queries for `Nebulex.Adapters.Local` adapter:\n\n    iex> unexpired = MyCache.all(:unexpired)\n    iex> expired = MyCache.all(:expired)\n\nIf we are using Nebulex.Adapters.Local adapter, the stored entry tuple\n`{:entry, key, value, touched, ttl}`, then the match spec could be\nsomething like:\n\n    iex> spec = [\n    ...>   {{:entry, :\"$1\", :\"$2\", :_, :_},\n    ...>   [{:>, :\"$2\", 5}], [{{:\"$1\", :\"$2\"}}]}\n    ...> ]\n    iex> MyCache.all(spec)\n    [{3, 6}, {4, 8}, {5, 10}]\n\nThe same previous query but using `Ex2ms`:\n\n    iex> import Ex2ms\n    Ex2ms\n\n    iex> spec =\n    ...>   fun do\n    ...>     {_. key, value, _, _} when value > 5 -> {key, value}\n    ...>   end\n\n    iex> MyCache.all(spec)\n    [{3, 6}, {4, 8}, {5, 10}]","ref":"Nebulex.Cache.html#c:all/2-examples"},{"type":"callback","title":"Nebulex.Cache.config/0","doc":"Returns the adapter configuration stored in the `:otp_app` environment.\n\nIf the `c:init/1` callback is implemented in the cache, it will be invoked.","ref":"Nebulex.Cache.html#c:config/0"},{"type":"callback","title":"Nebulex.Cache.count_all/2","doc":"Counts all entries in cache matching the given `query`.\n\nIt returns the count of the matched entries.\n\nIf `query` is `nil` (the default), then the total number of\ncached entries is returned.\n\nMay raise `Nebulex.QueryError` if query validation fails.","ref":"Nebulex.Cache.html#c:count_all/2"},{"type":"callback","title":"Query values - Nebulex.Cache.count_all/2","doc":"See `c:all/2` callback for more information about the query values.","ref":"Nebulex.Cache.html#c:count_all/2-query-values"},{"type":"callback","title":"Example - Nebulex.Cache.count_all/2","doc":"Populate the cache with some entries:\n\n    iex> :ok = Enum.each(1..5, &MyCache.put(&1, &1 * 2))\n\nCount all entries in cache:\n\n    iex> MyCache.count_all()\n    5\n\nCount all entries that match with the given query assuming we are using\n`Nebulex.Adapters.Local` adapter:\n\n    iex> query = [{{:_, :\"$1\", :\"$2\", :_, :_}, [{:>, :\"$2\", 5}], [true]}]\n    iex> MyCache.count_all(query)\n\n> For the local adapter you can use [Ex2ms](https://github.com/ericmj/ex2ms)\n  to build the match specs much easier.\n\nAdditional built-in queries for `Nebulex.Adapters.Local` adapter:\n\n    iex> unexpired = MyCache.count_all(:unexpired)\n    iex> expired = MyCache.count_all(:expired)","ref":"Nebulex.Cache.html#c:count_all/2-example"},{"type":"callback","title":"Nebulex.Cache.decr/3","doc":"Decrements the counter stored at `key` by the given `amount`.\n\nIf `amount < 0` (negative), the value is incremented by that `amount`\ninstead (opposite to `incr/3`).","ref":"Nebulex.Cache.html#c:decr/3"},{"type":"callback","title":"Options - Nebulex.Cache.decr/3","doc":"* `:ttl` - (positive integer or `:infinity`) Defines the time-to-live\n    (or expiry time) for the given key  in **milliseconds**. Defaults\n    to `:infinity`.\n\n  * `:default` - If `key` is not present in Cache, the default value is\n    inserted as initial value of key before the it is incremented.\n    Defaults to `0`.\n\nSee the configured adapter documentation for more runtime options.","ref":"Nebulex.Cache.html#c:decr/3-options"},{"type":"callback","title":"Examples - Nebulex.Cache.decr/3","doc":"iex> MyCache.decr(:a)\n    -1\n\n    iex> MyCache.decr(:a, 2)\n    -3\n\n    iex> MyCache.decr(:a, -1)\n    -2\n\n    iex> MyCache.decr(:missing_key, 2, default: 10)\n    8","ref":"Nebulex.Cache.html#c:decr/3-examples"},{"type":"callback","title":"Nebulex.Cache.delete/2","doc":"Deletes the entry in Cache for a specific `key`.\n\nSee the configured adapter documentation for runtime options.","ref":"Nebulex.Cache.html#c:delete/2"},{"type":"callback","title":"Example - Nebulex.Cache.delete/2","doc":"iex> MyCache.put(:a, 1)\n    :ok\n\n    iex> MyCache.delete(:a)\n    :ok\n\n    iex> MyCache.get(:a)\n    nil\n\n    iex> MyCache.delete(:non_existent_key)\n    :ok","ref":"Nebulex.Cache.html#c:delete/2-example"},{"type":"callback","title":"Nebulex.Cache.delete_all/2","doc":"Deletes all entries matching the given `query`. If `query` is `nil`,\nthen all entries in the cache are deleted.\n\nIt returns the number of deleted entries.\n\nMay raise `Nebulex.QueryError` if query validation fails.\n\nSee the configured adapter documentation for runtime options.","ref":"Nebulex.Cache.html#c:delete_all/2"},{"type":"callback","title":"Query values - Nebulex.Cache.delete_all/2","doc":"See `c:all/2` callback for more information about the query values.","ref":"Nebulex.Cache.html#c:delete_all/2-query-values"},{"type":"callback","title":"Example - Nebulex.Cache.delete_all/2","doc":"Populate the cache with some entries:\n\n    iex> :ok = Enum.each(1..5, &MyCache.put(&1, &1 * 2))\n\nDelete all (with default params):\n\n    iex> MyCache.delete_all()\n    5\n\nDelete all entries that match with the given query assuming we are using\n`Nebulex.Adapters.Local` adapter:\n\n    iex> query = [{{:_, :\"$1\", :\"$2\", :_, :_}, [{:>, :\"$2\", 5}], [true]}]\n    iex> MyCache.delete_all(query)\n\n> For the local adapter you can use [Ex2ms](https://github.com/ericmj/ex2ms)\n  to build the match specs much easier.\n\nAdditional built-in queries for `Nebulex.Adapters.Local` adapter:\n\n    iex> unexpired = MyCache.delete_all(:unexpired)\n    iex> expired = MyCache.delete_all(:expired)","ref":"Nebulex.Cache.html#c:delete_all/2-example"},{"type":"callback","title":"Nebulex.Cache.dispatch_stats/1","doc":"Emits a telemetry event when called with the current stats count.\n\nThe telemetry `:measurements` map will include the same as\n`Nebulex.Stats.t()`'s measurements. For example:\n\n  * `:evictions` - Current **evictions** count.\n  * `:expirations` - Current **expirations** count.\n  * `:hits` - Current **hits** count.\n  * `:misses` - Current **misses** count.\n  * `:updates` - Current **updates** count.\n  * `:writes` - Current **writes** count.\n\nThe telemetry `:metadata` map will include the same as `Nebulex.Stats.t()`'s\nmetadata by default. For example:\n\n  * `:cache` - The cache module, or the name (if an explicit name has been\n    given to the cache).\n\nAdditionally, you can add your own metadata fields by given the option\n`:metadata`.","ref":"Nebulex.Cache.html#c:dispatch_stats/1"},{"type":"callback","title":"Options - Nebulex.Cache.dispatch_stats/1","doc":"* `:event_prefix` – The prefix of the telemetry event.\n    Defaults to `[:nebulex, :cache]`.\n\n  * `:metadata` – A map with additional metadata fields. Defaults to `%{}`.","ref":"Nebulex.Cache.html#c:dispatch_stats/1-options"},{"type":"callback","title":"Examples - Nebulex.Cache.dispatch_stats/1","doc":"iex> MyCache.dispatch_stats()\n    :ok\n\n    iex> MyCache.Stats.dispatch_stats(\n    ...>   event_prefix: [:my_cache],\n    ...>   metadata: %{tag: \"tag1\"}\n    ...> )\n    :ok\n\n**NOTE:** Since `:telemetry` is an optional dependency, when it is not\ndefined, a default implementation is provided without any logic, just\nreturning `:ok`.","ref":"Nebulex.Cache.html#c:dispatch_stats/1-examples"},{"type":"callback","title":"Nebulex.Cache.dump/2","doc":"Dumps a cache to the given file `path`.\n\nReturns `:ok` if successful, or `{:error, reason}` if an error occurs.","ref":"Nebulex.Cache.html#c:dump/2"},{"type":"callback","title":"Options - Nebulex.Cache.dump/2","doc":"This operation relies entirely on the adapter implementation, which means the\noptions depend on each of them. For that reason, it is recommended to review\nthe documentation of the adapter to be used. The built-in adapters inherit\nthe default implementation from `Nebulex.Adapter.Persistence`, hence, review\nthe available options there.","ref":"Nebulex.Cache.html#c:dump/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.dump/2","doc":"Populate the cache with some entries:\n\n    iex> entries = for x <- 1..10, into: %{}, do: {x, x}\n    iex> MyCache.set_many(entries)\n    :ok\n\nDump cache to a file:\n\n    iex> MyCache.dump(\"my_cache\")\n    :ok","ref":"Nebulex.Cache.html#c:dump/2-examples"},{"type":"callback","title":"Nebulex.Cache.expire/2","doc":"Returns `true` if the given `key` exists and the new `ttl` was successfully\nupdated, otherwise, `false` is returned.","ref":"Nebulex.Cache.html#c:expire/2"},{"type":"callback","title":"Examples - Nebulex.Cache.expire/2","doc":"iex> MyCache.put(:a, 1)\n    :ok\n\n    iex> MyCache.expire(:a, 5)\n    true\n\n    iex> MyCache.expire(:a, :infinity)\n    true\n\n    iex> MyCache.ttl(:b, 5)\n    false","ref":"Nebulex.Cache.html#c:expire/2-examples"},{"type":"callback","title":"Nebulex.Cache.flush/0","doc":"Flushes the cache and returns the number of evicted keys.","ref":"Nebulex.Cache.html#c:flush/0"},{"type":"callback","title":"Examples - Nebulex.Cache.flush/0","doc":"iex> :ok = Enum.each(1..5, &MyCache.put(&1, &1))\n    iex> MyCache.flush()\n    5\n\n    iex> MyCache.size()\n    0","ref":"Nebulex.Cache.html#c:flush/0-examples"},{"type":"callback","title":"Nebulex.Cache.get/2","doc":"Gets a value from Cache where the key matches the given `key`.\n\nReturns `nil` if no result was found.\n\nSee the configured adapter documentation for runtime options.","ref":"Nebulex.Cache.html#c:get/2"},{"type":"callback","title":"Example - Nebulex.Cache.get/2","doc":"iex> MyCache.put(\"foo\", \"bar\")\n    :ok\n\n    iex>  MyCache.get(\"foo\")\n    \"bar\"\n\n    iex> MyCache.get(:non_existent_key)\n    nil","ref":"Nebulex.Cache.html#c:get/2-example"},{"type":"callback","title":"Nebulex.Cache.get!/2","doc":"Similar to `c:get/2` but raises `KeyError` if `key` is not found.\n\nSee the configured adapter documentation for runtime options.","ref":"Nebulex.Cache.html#c:get!/2"},{"type":"callback","title":"Example - Nebulex.Cache.get!/2","doc":"MyCache.get!(:a)","ref":"Nebulex.Cache.html#c:get!/2-example"},{"type":"callback","title":"Nebulex.Cache.get_all/2","doc":"Returns a `map` with all the key-value pairs in the Cache where the key\nis in `keys`.\n\nIf `keys` contains keys that are not in the Cache, they're simply ignored.\n\nSee the configured adapter documentation for runtime options.","ref":"Nebulex.Cache.html#c:get_all/2"},{"type":"callback","title":"Example - Nebulex.Cache.get_all/2","doc":"iex> MyCache.put_all([a: 1, c: 3])\n    :ok\n\n    iex> MyCache.get_all([:a, :b, :c])\n    %{a: 1, c: 3}","ref":"Nebulex.Cache.html#c:get_all/2-example"},{"type":"callback","title":"Nebulex.Cache.get_and_update/3","doc":"Gets the value from `key` and updates it, all in one pass.\n\n`fun` is called with the current cached value under `key` (or `nil` if `key`\nhasn't been cached) and must return a two-element tuple: the current value\n(the retrieved value, which can be operated on before being returned) and\nthe new value to be stored under `key`. `fun` may also return `:pop`, which\nmeans the current value shall be removed from Cache and returned.\n\nThe returned value is a tuple with the current value returned by `fun` and\nthe new updated value under `key`.","ref":"Nebulex.Cache.html#c:get_and_update/3"},{"type":"callback","title":"Options - Nebulex.Cache.get_and_update/3","doc":"* `:ttl` - (positive integer or `:infinity`) Defines the time-to-live\n    (or expiry time) for the given key  in **milliseconds**. Defaults\n    to `:infinity`.\n\nSee the configured adapter documentation for more runtime options.","ref":"Nebulex.Cache.html#c:get_and_update/3-options"},{"type":"callback","title":"Examples - Nebulex.Cache.get_and_update/3","doc":"Update nonexistent key:\n\n    iex> MyCache.get_and_update(:a, fn current_value ->\n    ...>   {current_value, \"value!\"}\n    ...> end)\n    {nil, \"value!\"}\n\nUpdate existing key:\n\n    iex> MyCache.get_and_update(:a, fn current_value ->\n    ...>   {current_value, \"new value!\"}\n    ...> end)\n    {\"value!\", \"new value!\"}\n\nPop/remove value if exist:\n\n    iex> MyCache.get_and_update(:a, fn _ -> :pop end)\n    {\"new value!\", nil}\n\nPop/remove nonexistent key:\n\n    iex> MyCache.get_and_update(:b, fn _ -> :pop end)\n    {nil, nil}","ref":"Nebulex.Cache.html#c:get_and_update/3-examples"},{"type":"callback","title":"Nebulex.Cache.get_dynamic_cache/0","doc":"Returns the atom name or pid of the current cache\n(based on Ecto dynamic repo).\n\nSee also `c:put_dynamic_cache/1`.","ref":"Nebulex.Cache.html#c:get_dynamic_cache/0"},{"type":"callback","title":"Nebulex.Cache.has_key?/1","doc":"Returns whether the given `key` exists in the Cache.","ref":"Nebulex.Cache.html#c:has_key?/1"},{"type":"callback","title":"Examples - Nebulex.Cache.has_key?/1","doc":"iex> MyCache.put(:a, 1)\n    :ok\n\n    iex> MyCache.has_key?(:a)\n    true\n\n    iex> MyCache.has_key?(:b)\n    false","ref":"Nebulex.Cache.html#c:has_key?/1-examples"},{"type":"callback","title":"Nebulex.Cache.in_transaction?/0","doc":"Returns `true` if the current process is inside a transaction.","ref":"Nebulex.Cache.html#c:in_transaction?/0"},{"type":"callback","title":"Examples - Nebulex.Cache.in_transaction?/0","doc":"MyCache.in_transaction?\n    #=> false\n\n    MyCache.transaction(fn ->\n      MyCache.in_transaction? #=> true\n    end)","ref":"Nebulex.Cache.html#c:in_transaction?/0-examples"},{"type":"callback","title":"Nebulex.Cache.incr/3","doc":"Increments the counter stored at `key` by the given `amount`.\n\nIf `amount < 0` (negative), the value is decremented by that `amount`\ninstead.","ref":"Nebulex.Cache.html#c:incr/3"},{"type":"callback","title":"Options - Nebulex.Cache.incr/3","doc":"* `:ttl` - (positive integer or `:infinity`) Defines the time-to-live\n    (or expiry time) for the given key  in **milliseconds**. Defaults\n    to `:infinity`.\n\n  * `:default` - If `key` is not present in Cache, the default value is\n    inserted as initial value of key before the it is incremented.\n    Defaults to `0`.\n\nSee the configured adapter documentation for more runtime options.","ref":"Nebulex.Cache.html#c:incr/3-options"},{"type":"callback","title":"Examples - Nebulex.Cache.incr/3","doc":"iex> MyCache.incr(:a)\n    1\n\n    iex> MyCache.incr(:a, 2)\n    3\n\n    iex> MyCache.incr(:a, -1)\n    2\n\n    iex> MyCache.incr(:missing_key, 2, default: 10)\n    12","ref":"Nebulex.Cache.html#c:incr/3-examples"},{"type":"callback","title":"Nebulex.Cache.init/1","doc":"A callback executed when the cache starts or when configuration is read.","ref":"Nebulex.Cache.html#c:init/1"},{"type":"callback","title":"Nebulex.Cache.load/2","doc":"Loads a dumped cache from the given `path`.\n\nReturns `:ok` if successful, or `{:error, reason}` if an error occurs.","ref":"Nebulex.Cache.html#c:load/2"},{"type":"callback","title":"Options - Nebulex.Cache.load/2","doc":"Similar to `c:dump/2`, this operation relies entirely on the adapter\nimplementation, therefore, it is recommended to review the documentation\nof the adapter to be used. Similarly, the built-in adapters inherit the\ndefault implementation from `Nebulex.Adapter.Persistence`, hence, review\nthe available options there.","ref":"Nebulex.Cache.html#c:load/2-options"},{"type":"callback","title":"Examples - Nebulex.Cache.load/2","doc":"Populate the cache with some entries:\n\n    iex> entries = for x <- 1..10, into: %{}, do: {x, x}\n    iex> MyCache.set_many(entries)\n    :ok\n\nDump cache to a file:\n\n    iex> MyCache.dump(\"my_cache\")\n    :ok\n\nLoad the cache from a file:\n\n    iex> MyCache.load(\"my_cache\")\n    :ok","ref":"Nebulex.Cache.html#c:load/2-examples"},{"type":"callback","title":"Nebulex.Cache.put/3","doc":"Puts the given `value` under `key` into the Cache.\n\nIf `key` already holds an entry, it is overwritten. Any previous\ntime to live associated with the key is discarded on successful\n`put` operation.\n\nBy default, `nil` values are skipped, which means they are not stored;\nthe call to the adapter is bypassed.","ref":"Nebulex.Cache.html#c:put/3"},{"type":"callback","title":"Options - Nebulex.Cache.put/3","doc":"* `:ttl` - (positive integer or `:infinity`) Defines the time-to-live\n    (or expiry time) for the given key  in **milliseconds**. Defaults\n    to `:infinity`.\n\nSee the configured adapter documentation for more runtime options.","ref":"Nebulex.Cache.html#c:put/3-options"},{"type":"callback","title":"Example - Nebulex.Cache.put/3","doc":"iex> MyCache.put(\"foo\", \"bar\")\n    :ok\n\nIf the value is nil, then it is not stored (operation is skipped):\n\n    iex> MyCache.put(\"foo\", nil)\n    :ok\n\nPut key with time-to-live:\n\n    iex> MyCache.put(\"foo\", \"bar\", ttl: 10_000)\n    :ok\n\nUsing Nebulex.Time for TTL:\n\n    iex> MyCache.put(\"foo\", \"bar\", ttl: :timer.hours(1))\n    :ok\n\n    iex> MyCache.put(\"foo\", \"bar\", ttl: :timer.minutes(1))\n    :ok\n\n    iex> MyCache.put(\"foo\", \"bar\", ttl: :timer.seconds(1))\n    :ok","ref":"Nebulex.Cache.html#c:put/3-example"},{"type":"callback","title":"Nebulex.Cache.put_all/2","doc":"Puts the given `entries` (key/value pairs) into the Cache. It replaces\nexisting values with new values (just as regular `put`).","ref":"Nebulex.Cache.html#c:put_all/2"},{"type":"callback","title":"Options - Nebulex.Cache.put_all/2","doc":"* `:ttl` - (positive integer or `:infinity`) Defines the time-to-live\n    (or expiry time) for the given key  in **milliseconds**. Defaults\n    to `:infinity`.\n\nSee the configured adapter documentation for more runtime options.","ref":"Nebulex.Cache.html#c:put_all/2-options"},{"type":"callback","title":"Example - Nebulex.Cache.put_all/2","doc":"iex> MyCache.put_all(apples: 3, bananas: 1)\n    :ok\n\n    iex> MyCache.put_all(%{apples: 2, oranges: 1}, ttl: 10_000)\n    :ok\n\nIdeally, this operation should be atomic, so all given keys are put at once.\nBut it depends purely on the adapter's implementation and the backend used\ninternally by the adapter. Hence, it is recommended to review the adapter's\ndocumentation.","ref":"Nebulex.Cache.html#c:put_all/2-example"},{"type":"callback","title":"Nebulex.Cache.put_dynamic_cache/1","doc":"Sets the dynamic cache to be used in further commands\n(based on Ecto dynamic repo).\n\nThere might be cases where we want to have different cache instances but\naccessing them through the same cache module. By default, when you call\n`MyApp.Cache.start_link/1`, it will start a cache with the name\n`MyApp.Cache`. But it is also possible to start multiple caches by using\na different name for each of them:\n\n    MyApp.Cache.start_link(name: :cache1)\n    MyApp.Cache.start_link(name: :cache2, backend: :shards)\n\nYou can also start caches without names by explicitly setting the name\nto `nil`:\n\n    MyApp.Cache.start_link(name: nil, backend: :shards)\n\n> **NOTE:** There may be adapters requiring the `:name` option anyway,\n  therefore, it is highly recommended to see the adapter's documentation\n  you want to use.\n\nHowever, once the cache is started, it is not possible to interact directly\nwith it, since all operations through `MyApp.Cache` are sent by default to\nthe cache named `MyApp.Cache`. But you can change the default cache at\ncompile-time:\n\n    use Nebulex.Cache, default_dynamic_cache: :cache_name\n\nOr anytime at runtime by calling `put_dynamic_cache/1`:\n\n    MyApp.Cache.put_dynamic_cache(:another_cache_name)\n\nFrom this moment on, all future commands performed by the current process\nwill run on `:another_cache_name`.","ref":"Nebulex.Cache.html#c:put_dynamic_cache/1"},{"type":"callback","title":"Nebulex.Cache.put_new/3","doc":"Puts the given `value` under `key` into the cache, only if it does not\nalready exist.\n\nReturns `true` if a value was set, otherwise, `false` is returned.\n\nBy default, `nil` values are skipped, which means they are not stored;\nthe call to the adapter is bypassed.","ref":"Nebulex.Cache.html#c:put_new/3"},{"type":"callback","title":"Options - Nebulex.Cache.put_new/3","doc":"* `:ttl` - (positive integer or `:infinity`) Defines the time-to-live\n    (or expiry time) for the given key  in **milliseconds**. Defaults\n    to `:infinity`.\n\nSee the configured adapter documentation for more runtime options.","ref":"Nebulex.Cache.html#c:put_new/3-options"},{"type":"callback","title":"Example - Nebulex.Cache.put_new/3","doc":"iex> MyCache.put_new(\"foo\", \"bar\")\n    true\n\n    iex> MyCache.put_new(\"foo\", \"bar\")\n    false\n\nIf the value is nil, it is not stored (operation is skipped):\n\n    iex> MyCache.put_new(\"other\", nil)\n    true","ref":"Nebulex.Cache.html#c:put_new/3-example"},{"type":"callback","title":"Nebulex.Cache.put_new!/3","doc":"Similar to `c:put_new/3` but raises `Nebulex.KeyAlreadyExistsError` if the\nkey already exists.\n\nSee `c:put_new/3` for general considerations and options.","ref":"Nebulex.Cache.html#c:put_new!/3"},{"type":"callback","title":"Example - Nebulex.Cache.put_new!/3","doc":"iex> MyCache.put_new!(\"foo\", \"bar\")\n    true","ref":"Nebulex.Cache.html#c:put_new!/3-example"},{"type":"callback","title":"Nebulex.Cache.put_new_all/2","doc":"Puts the given `entries` (key/value pairs) into the `cache`. It will not\nperform any operation at all even if just a single key already exists.\n\nReturns `true` if all entries were successfully set. It returns `false`\nif no key was set (at least one key already existed).","ref":"Nebulex.Cache.html#c:put_new_all/2"},{"type":"callback","title":"Options - Nebulex.Cache.put_new_all/2","doc":"* `:ttl` - (positive integer or `:infinity`) Defines the time-to-live\n    (or expiry time) for the given key  in **milliseconds**. Defaults\n    to `:infinity`.\n\nSee the configured adapter documentation for more runtime options.","ref":"Nebulex.Cache.html#c:put_new_all/2-options"},{"type":"callback","title":"Example - Nebulex.Cache.put_new_all/2","doc":"iex> MyCache.put_new_all(apples: 3, bananas: 1)\n    true\n\n    iex> MyCache.put_new_all(%{apples: 3, oranges: 1}, ttl: 10_000)\n    false\n\nIdeally, this operation should be atomic, so all given keys are put at once.\nBut it depends purely on the adapter's implementation and the backend used\ninternally by the adapter. Hence, it is recommended to review the adapter's\ndocumentation.","ref":"Nebulex.Cache.html#c:put_new_all/2-example"},{"type":"callback","title":"Nebulex.Cache.replace/3","doc":"Alters the entry stored under `key`, but only if the entry already exists\ninto the Cache.\n\nReturns `true` if a value was set, otherwise, `false` is returned.\n\nBy default, `nil` values are skipped, which means they are not stored;\nthe call to the adapter is bypassed.","ref":"Nebulex.Cache.html#c:replace/3"},{"type":"callback","title":"Options - Nebulex.Cache.replace/3","doc":"* `:ttl` - (positive integer or `:infinity`) Defines the time-to-live\n    (or expiry time) for the given key  in **milliseconds**. Defaults\n    to `:infinity`.\n\nSee the configured adapter documentation for more runtime options.","ref":"Nebulex.Cache.html#c:replace/3-options"},{"type":"callback","title":"Example - Nebulex.Cache.replace/3","doc":"iex> MyCache.replace(\"foo\", \"bar\")\n    false\n\n    iex> MyCache.put_new(\"foo\", \"bar\")\n    true\n\n    iex> MyCache.replace(\"foo\", \"bar2\")\n    true\n\nUpdate current value and TTL:\n\n    iex> MyCache.replace(\"foo\", \"bar3\", ttl: 10_000)\n    true","ref":"Nebulex.Cache.html#c:replace/3-example"},{"type":"callback","title":"Nebulex.Cache.replace!/3","doc":"Similar to `c:replace/3` but raises `KeyError` if `key` is not found.\n\nSee `c:replace/3` for general considerations and options.","ref":"Nebulex.Cache.html#c:replace!/3"},{"type":"callback","title":"Example - Nebulex.Cache.replace!/3","doc":"iex> MyCache.replace!(\"foo\", \"bar\")\n    true","ref":"Nebulex.Cache.html#c:replace!/3-example"},{"type":"callback","title":"Nebulex.Cache.size/0","doc":"Returns the total number of cached entries.","ref":"Nebulex.Cache.html#c:size/0"},{"type":"callback","title":"Examples - Nebulex.Cache.size/0","doc":"iex> :ok = Enum.each(1..10, &MyCache.put(&1, &1))\n    iex> MyCache.size()\n    10\n\n    iex> :ok = Enum.each(1..5, &MyCache.delete(&1))\n    iex> MyCache.size()\n    5","ref":"Nebulex.Cache.html#c:size/0-examples"},{"type":"callback","title":"Nebulex.Cache.start_link/1","doc":"Starts a supervision and return `{:ok, pid}` or just `:ok` if nothing\nneeds to be done.\n\nReturns `{:error, {:already_started, pid}}` if the cache is already\nstarted or `{:error, term}` in case anything else goes wrong.","ref":"Nebulex.Cache.html#c:start_link/1"},{"type":"callback","title":"Options - Nebulex.Cache.start_link/1","doc":"See the configuration in the moduledoc for options shared between adapters,\nfor adapter-specific configuration see the adapter's documentation.","ref":"Nebulex.Cache.html#c:start_link/1-options"},{"type":"callback","title":"Nebulex.Cache.stats/0","doc":"Returns `Nebulex.Stats.t()` with the current stats values.\n\nIf the stats are disabled for the cache, then `nil` is returned.","ref":"Nebulex.Cache.html#c:stats/0"},{"type":"callback","title":"Example - Nebulex.Cache.stats/0","doc":"iex> MyCache.stats()\n    %Nebulex.Stats{\n      measurements: {\n        evictions: 0,\n        expirations: 0,\n        hits: 0,\n        misses: 0,\n        updates: 0,\n        writes: 0\n      },\n      metadata: %{}\n    }","ref":"Nebulex.Cache.html#c:stats/0-example"},{"type":"callback","title":"Nebulex.Cache.stop/1","doc":"Shuts down the cache.","ref":"Nebulex.Cache.html#c:stop/1"},{"type":"callback","title":"Nebulex.Cache.stream/2","doc":"Similar to `c:all/2` but returns a lazy enumerable that emits all entries\nfrom the cache matching the given `query`.\n\nIf `query` is `nil`, then all entries in cache match and are returned\nwhen the stream is evaluated; based on the `:return` option.\n\nMay raise `Nebulex.QueryError` if query validation fails.","ref":"Nebulex.Cache.html#c:stream/2"},{"type":"callback","title":"Query values - Nebulex.Cache.stream/2","doc":"See `c:all/2` callback for more information about the query values.","ref":"Nebulex.Cache.html#c:stream/2-query-values"},{"type":"callback","title":"Options - Nebulex.Cache.stream/2","doc":"* `:return` - Tells the query what to return from the matched entries.\n    See the possible values in the \"Query return option\" section below.\n    The default depends on the adapter, for example, the default for the\n    built-in adapters is `:key`. This option is supported by the built-in\n    adapters, but it is recommended to see the adapter's documentation\n    to confirm its compatibility with this option.\n\n  * `:page_size` - Positive integer (>= 1) that defines the page size\n    internally used by the adapter for paginating the results coming\n    back from the cache's backend. Defaults to `20`; it's unlikely\n    this will ever need changing.\n\nSee the configured adapter documentation for more runtime options.","ref":"Nebulex.Cache.html#c:stream/2-options"},{"type":"callback","title":"Query return option - Nebulex.Cache.stream/2","doc":"The following are the possible values for the `:return` option:\n\n  * `:key` - Returns a list only with the keys.\n  * `:value` - Returns a list only with the values.\n  * `:entry` - Returns a list of `t:Nebulex.Entry.t/0`.\n  * `{:key, :value}` - Returns a list of tuples in the form `{key, value}`.\n\nSee adapters documentation to confirm what of these options are supported\nand what other added.","ref":"Nebulex.Cache.html#c:stream/2-query-return-option"},{"type":"callback","title":"Examples - Nebulex.Cache.stream/2","doc":"Populate the cache with some entries:\n\n    iex> :ok = Enum.each(1..5, &MyCache.put(&1, &1 * 2))\n\nStream all (with default params):\n\n    iex> MyCache.stream() |> Enum.to_list()\n    [1, 2, 3, 4, 5]\n\nStream all entries and return values:\n\n    iex> nil |> MyCache.stream(return: :value, page_size: 3) |> Enum.to_list()\n    [2, 4, 6, 8, 10]\n\nStream all entries and return them as key/value pairs:\n\n    iex> nil |> MyCache.stream(return: {:key, :value}) |> Enum.to_list()\n    [{1, 2}, {2, 4}, {3, 6}, {4, 8}, {5, 10}]\n\nAdditional built-in queries for `Nebulex.Adapters.Local` adapter:\n\n    iex> unexpired_stream = MyCache.stream(:unexpired)\n    iex> expired_stream = MyCache.stream(:expired)\n\nIf we are using Nebulex.Adapters.Local adapter, the stored entry tuple\n`{:entry, key, value, touched, ttl}`, then the match spec could be\nsomething like:\n\n    iex> spec = [\n    ...>   {{:entry, :\"$1\", :\"$2\", :_, :_},\n    ...>   [{:>, :\"$2\", 5}], [{{:\"$1\", :\"$2\"}}]}\n    ...> ]\n    iex> MyCache.stream(spec, page_size: 100) |> Enum.to_list()\n    [{3, 6}, {4, 8}, {5, 10}]\n\nThe same previous query but using `Ex2ms`:\n\n    iex> import Ex2ms\n    Ex2ms\n\n    iex> spec =\n    ...>   fun do\n    ...>     {_, key, value, _, _} when value > 5 -> {key, value}\n    ...>   end\n\n    iex> spec |> MyCache.stream(page_size: 100) |> Enum.to_list()\n    [{3, 6}, {4, 8}, {5, 10}]","ref":"Nebulex.Cache.html#c:stream/2-examples"},{"type":"callback","title":"Nebulex.Cache.take/2","doc":"Returns and removes the value associated with `key` in the Cache.\nIf the `key` does not exist, then `nil` is returned.\n\nIf `key` is `nil`, the call to the adapter is bypassed, and `nil` is returned.\n\nSee the configured adapter documentation for runtime options.","ref":"Nebulex.Cache.html#c:take/2"},{"type":"callback","title":"Examples - Nebulex.Cache.take/2","doc":"iex> MyCache.put(:a, 1)\n    :ok\n\n    iex> MyCache.take(:a)\n    1\n\n    iex> MyCache.take(:a)\n    nil","ref":"Nebulex.Cache.html#c:take/2-examples"},{"type":"callback","title":"Nebulex.Cache.take!/2","doc":"Similar to `c:take/2` but raises `KeyError` if `key` is not found.\n\nSee `c:take/2` for general considerations and options.","ref":"Nebulex.Cache.html#c:take!/2"},{"type":"callback","title":"Example - Nebulex.Cache.take!/2","doc":"MyCache.take!(:a)","ref":"Nebulex.Cache.html#c:take!/2-example"},{"type":"callback","title":"Nebulex.Cache.touch/1","doc":"Returns `true` if the given `key` exists and the last access time was\nsuccessfully updated, otherwise, `false` is returned.","ref":"Nebulex.Cache.html#c:touch/1"},{"type":"callback","title":"Examples - Nebulex.Cache.touch/1","doc":"iex> MyCache.put(:a, 1)\n    :ok\n\n    iex> MyCache.touch(:a)\n    true\n\n    iex> MyCache.ttl(:b)\n    false","ref":"Nebulex.Cache.html#c:touch/1-examples"},{"type":"callback","title":"Nebulex.Cache.transaction/2","doc":"Runs the given function inside a transaction.\n\nA successful transaction returns the value returned by the function.\n\nSee the configured adapter documentation for runtime options.","ref":"Nebulex.Cache.html#c:transaction/2"},{"type":"callback","title":"Examples - Nebulex.Cache.transaction/2","doc":"MyCache.transaction fn ->\n      alice = MyCache.get(:alice)\n      bob = MyCache.get(:bob)\n      MyCache.put(:alice, %{alice | balance: alice.balance + 100})\n      MyCache.put(:bob, %{bob | balance: bob.balance + 100})\n    end\n\nLocking only the involved key (recommended):\n\n    MyCache.transaction [keys: [:alice, :bob]], fn ->\n      alice = MyCache.get(:alice)\n      bob = MyCache.get(:bob)\n      MyCache.put(:alice, %{alice | balance: alice.balance + 100})\n      MyCache.put(:bob, %{bob | balance: bob.balance + 100})\n    end","ref":"Nebulex.Cache.html#c:transaction/2-examples"},{"type":"callback","title":"Nebulex.Cache.ttl/1","doc":"Returns the remaining time-to-live for the given `key`. If the `key` does not\nexist, then `nil` is returned.","ref":"Nebulex.Cache.html#c:ttl/1"},{"type":"callback","title":"Examples - Nebulex.Cache.ttl/1","doc":"iex> MyCache.put(:a, 1, ttl: 5000)\n    :ok\n\n    iex> MyCache.put(:b, 2)\n    :ok\n\n    iex> MyCache.ttl(:a)\n    _remaining_ttl\n\n    iex> MyCache.ttl(:b)\n    :infinity\n\n    iex> MyCache.ttl(:c)\n    nil","ref":"Nebulex.Cache.html#c:ttl/1-examples"},{"type":"callback","title":"Nebulex.Cache.update/4","doc":"Updates the cached `key` with the given function.\n\nIf `key` is present in Cache with value `value`, `fun` is invoked with\nargument `value` and its result is used as the new value of `key`.\n\nIf `key` is not present in Cache, `initial` is inserted as the value of `key`.\nThe initial value will not be passed through the update function.","ref":"Nebulex.Cache.html#c:update/4"},{"type":"callback","title":"Options - Nebulex.Cache.update/4","doc":"* `:ttl` - (positive integer or `:infinity`) Defines the time-to-live\n    (or expiry time) for the given key  in **milliseconds**. Defaults\n    to `:infinity`.\n\nSee the configured adapter documentation for more runtime options.","ref":"Nebulex.Cache.html#c:update/4-options"},{"type":"callback","title":"Examples - Nebulex.Cache.update/4","doc":"iex> MyCache.update(:a, 1, &(&1 * 2))\n    1\n\n    iex> MyCache.update(:a, 1, &(&1 * 2))\n    2","ref":"Nebulex.Cache.html#c:update/4-examples"},{"type":"callback","title":"Nebulex.Cache.with_dynamic_cache/2","doc":"Invokes the given function `fun` for the dynamic cache `name_or_pid`.","ref":"Nebulex.Cache.html#c:with_dynamic_cache/2"},{"type":"callback","title":"Example - Nebulex.Cache.with_dynamic_cache/2","doc":"MyCache.with_dynamic_cache(:my_cache, fn ->\n      MyCache.put(\"foo\", \"var\")\n    end)\n\nSee `c:get_dynamic_cache/0` and `c:put_dynamic_cache/1`.","ref":"Nebulex.Cache.html#c:with_dynamic_cache/2-example"},{"type":"callback","title":"Nebulex.Cache.with_dynamic_cache/4","doc":"For the dynamic cache `name_or_pid`, invokes the given function name `fun`\nfrom `module` with the list of arguments `args`.","ref":"Nebulex.Cache.html#c:with_dynamic_cache/4"},{"type":"callback","title":"Example - Nebulex.Cache.with_dynamic_cache/4","doc":"MyCache.with_dynamic_cache(:my_cache, Module, :some_fun, [\"foo\", \"bar\"])\n\nSee `c:get_dynamic_cache/0` and `c:put_dynamic_cache/1`.","ref":"Nebulex.Cache.html#c:with_dynamic_cache/4-example"},{"type":"type","title":"Nebulex.Cache.entries/0","doc":"Cache entries","ref":"Nebulex.Cache.html#t:entries/0"},{"type":"type","title":"Nebulex.Cache.key/0","doc":"Cache entry key","ref":"Nebulex.Cache.html#t:key/0"},{"type":"type","title":"Nebulex.Cache.opts/0","doc":"Cache action options","ref":"Nebulex.Cache.html#t:opts/0"},{"type":"type","title":"Nebulex.Cache.t/0","doc":"","ref":"Nebulex.Cache.html#t:t/0"},{"type":"type","title":"Nebulex.Cache.value/0","doc":"Cache entry value","ref":"Nebulex.Cache.html#t:value/0"},{"type":"module","title":"Nebulex.Caching","doc":"At its core, the abstraction applies caching to Elixir functions, reducing\nthus the number of executions based on the information available in the\ncache. That is, each time a targeted function is invoked, the abstraction\nwill apply a caching behavior checking whether the function has been already\nexecuted and its result cached for the given arguments. If it has, then the\ncached result is returned without having to execute the actual function;\nif it has not, then function is executed, the result cached and returned\nto the user so that, the next time the method is invoked, the cached result\nis returned. This way, expensive functions (whether CPU or IO bound) can be\nexecuted only once for a given set of parameters and the result reused\nwithout having to actually execute the function again. The caching logic\nis applied transparently without any interference to the invoker.\n\nSee **`Nebulex.Caching.Decorators`** for more information about\n**\"Declarative annotation-based caching\"**.","ref":"Nebulex.Caching.html"},{"type":"macro","title":"Nebulex.Caching.keyref/2","doc":"A wrapper macro for `Nebulex.Caching.Decorators.build_keyref/2`.\n\nThis macro is imported automatically with `use Nebulex.Caching`,\nwhich means you don't need to do any additional `alias` or `import`.\n\nSee `cacheable/3` decorator for more information about its usage.","ref":"Nebulex.Caching.html#keyref/2"},{"type":"module","title":"Nebulex.Caching.Decorators","doc":"Declarative annotation-based caching via function\n[decorators](https://github.com/arjan/decorator).\n\nFor caching declaration, the abstraction provides three Elixir function\ndecorators: `cacheable `, `cache_evict`, and `cache_put`, which allow\nfunctions to trigger cache population or cache eviction.\nLet us take a closer look at each annotation.\n\n> Inspired by [Spring Cache Abstraction](https://docs.spring.io/spring/docs/3.2.x/spring-framework-reference/html/cache.html).\n\n## `cacheable` decorator\n\nAs the name implies, `cacheable` is used to demarcate functions that are\ncacheable - that is, functions for whom the result is stored into the cache\nso, on subsequent invocations (with the same arguments), the value in the\ncache is returned without having to actually execute the function. In its\nsimplest form, the decorator/annotation declaration requires the name of\nthe cache associated with the annotated function:\n\n    @decorate cacheable(cache: Cache)\n    def get_account(id) do\n      # the logic for retrieving the account ...\n    end\n\nIn the snippet above, the function `get_account/1` is associated with the\ncache named `Cache`. Each time the function is called, the cache is checked\nto see whether the invocation has been already executed and does not have\nto be repeated.","ref":"Nebulex.Caching.Decorators.html"},{"type":"module","title":"Default Key Generation - Nebulex.Caching.Decorators","doc":"Since caches are essentially key-value stores, each invocation of a cached\nfunction needs to be translated into a suitable key for cache access.\nOut of the box, the caching abstraction uses a simple key-generator\nbased on the following algorithm:\n\n  * If no params are given, return `0`.\n  * If only one param is given, return that param as key.\n  * If more than one param is given, return a key computed from the hashes\n    of all parameters (`:erlang.phash2(args)`).\n\n> **IMPORTANT:** Since Nebulex v2.1.0, the default key generation implements\n  the algorithm described above, breaking backward compatibility with older\n  versions. Therefore, you may need to change your code in case of using the\n  default key generation.\n\nThe default key generator is provided by the cache via the callback\n`c:Nebulex.Cache.__default_key_generator__/0` and it is applied only\nif the option `key:` or `keys:` is not configured. Defaults to\n`Nebulex.Caching.SimpleKeyGenerator`. You can change the default\nkey generator at compile time with the option `:default_key_generator`.\nFor example, one can define a cache with a default key generator as:\n\n    defmodule MyApp.Cache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local,\n        default_key_generator: __MODULE__\n\n      @behaviour Nebulex.Caching.KeyGenerator\n\n      @impl true\n      def generate(mod, fun, args), do: :erlang.phash2({mod, fun, args})\n    end\n\nThe key generator module must implement the `Nebulex.Caching.KeyGenerator`\nbehaviour.\n\n> **IMPORTANT:** There are some caveats to keep in mind when using\n  the key generator, therefore, it is highly recommended to review\n  `Nebulex.Caching.KeyGenerator` behaviour documentation before.\n\nAlso, you can provide a different key generator at any time\n(overriding the default one) when using any caching annotation\nthrough the option `:key_generator`. For example:\n\n    # With a module implementing the key-generator behaviour\n    @decorate cache_put(cache: Cache, key_generator: CustomKeyGenerator)\n    def update_account(account) do\n      # the logic for updating the given entity ...\n    end\n\n    # With the shorthand tuple {module, args}\n    @decorate cache_put(\n                cache: Cache,\n                key_generator: {CustomKeyGenerator, [account.name]}\n              )\n    def update_account2(account) do\n      # the logic for updating the given entity ...\n    end\n\n    # With a MFA tuple\n    @decorate cache_put(\n                cache: Cache,\n                key_generator: {AnotherModule, :genkey, [account.id]}\n              )\n    def update_account3(account) do\n      # the logic for updating the given entity ...\n    end\n\n> The `:key_generator` option is available for all caching annotations.","ref":"Nebulex.Caching.Decorators.html#module-default-key-generation"},{"type":"module","title":"Custom Key Generation Declaration - Nebulex.Caching.Decorators","doc":"Since caching is generic, it is quite likely the target functions have\nvarious signatures that cannot be simply mapped on top of the cache\nstructure. This tends to become obvious when the target function has\nmultiple arguments out of which only some are suitable for caching\n(while the rest are used only by the function logic). For example:\n\n    @decorate cacheable(cache: Cache)\n    def get_account(email, include_users?) do\n      # the logic for retrieving the account ...\n    end\n\nAt first glance, while the boolean argument influences the way the account\nis found, it is no use for the cache.\n\nFor such cases, the `cacheable` decorator allows the user to specify the\nkey explicitly based on the function attributes.\n\n    @decorate cacheable(cache: Cache, key: {Account, email})\n    def get_account(email, include_users?) do\n      # the logic for retrieving the account ...\n    end\n\n    @decorate cacheable(cache: Cache, key: {Account, user.account_id})\n    def get_user_account(%User{} = user) do\n      # the logic for retrieving the account ...\n    end\n\nIt is also possible passing options to the cache, like so:\n\n    @decorate cacheable(\n                cache: Cache,\n                key: {Account, email},\n                opts: [ttl: 300_000]\n              )\n    def get_account(email, include_users?) do\n      # the logic for retrieving the account ...\n    end\n\nSee the **\"Shared Options\"** section below.","ref":"Nebulex.Caching.Decorators.html#module-custom-key-generation-declaration"},{"type":"module","title":"Functions with multiple clauses - Nebulex.Caching.Decorators","doc":"Since [decorator lib](https://github.com/arjan/decorator#functions-with-multiple-clauses)\nis used, it is important to be aware of its recommendations, warns,\nlimitations, and so on. In this case, for functions with multiple clauses\nthe general advice is to create an empty function head, and call the\ndecorator on that head, like so:\n\n    @decorate cacheable(cache: Cache, key: email)\n    def get_account(email \\\\ nil)\n\n    def get_account(nil), do: nil\n\n    def get_account(email) do\n      # the logic for retrieving the account ...\n    end\n\n## `cache_put` decorator\n\nFor cases where the cache needs to be updated without interfering with the\nfunction execution, one can use the `cache_put` decorator. That is, the\nmethod will always be executed and its result placed into the cache\n(according to the `cache_put` options). It supports the same options as\n`cacheable`.\n\n    @decorate cache_put(cache: Cache, key: {Account, acct.email})\n    def update_account(%Account{} = acct, attrs) do\n      # the logic for updating the account ...\n    end\n\nNote that using `cache_put` and `cacheable` annotations on the same function\nis generally discouraged because they have different behaviors. While the\nlatter causes the method execution to be skipped by using the cache, the\nformer forces the execution in order to execute a cache update. This leads\nto unexpected behavior and with the exception of specific corner-cases\n(such as decorators having conditions that exclude them from each other),\nsuch declarations should be avoided.\n\n## `cache_evict` decorator\n\nThe cache abstraction allows not just the population of a cache store but\nalso eviction. This process is useful for removing stale or unused data from\nthe cache. Opposed to `cacheable`, the decorator `cache_evict` demarcates\nfunctions that perform cache eviction, which are functions that act as\ntriggers for removing data from the cache. The `cache_evict` decorator not\nonly allows a key to be specified, but also a set of keys. Besides, extra\noptions like`all_entries` which indicates whether a cache-wide eviction\nneeds to be performed rather than just an entry one (based on the key or\nkeys):\n\n    @decorate cache_evict(cache: Cache, key: {Account, email})\n    def delete_account_by_email(email) do\n      # the logic for deleting the account ...\n    end\n\n    @decorate cacheable(\n                cache: Cache,\n                keys: [{Account, acct.id}, {Account, acct.email}]\n              )\n    def delete_account(%Account{} = acct) do\n      # the logic for deleting the account ...\n    end\n\n    @decorate cacheable(cache: Cache, all_entries: true)\n    def delete_all_accounts do\n      # the logic for deleting all the accounts ...\n    end\n\nThe option `all_entries:` comes in handy when an entire cache region needs\nto be cleared out - rather than evicting each entry (which would take a\nlong time since it is inefficient), all the entries are removed in one\noperation as shown above.","ref":"Nebulex.Caching.Decorators.html#module-functions-with-multiple-clauses"},{"type":"module","title":"Shared Options - Nebulex.Caching.Decorators","doc":"All three cache annotations explained previously accept the following\noptions:\n\n  * `:cache` - Defines what cache to use (required). Raises `ArgumentError`\n    if the option is not present. It can be also a MFA tuple to resolve the\n    cache dynamically in runtime by calling it. See \"The :cache option\"\n    section below for more information.\n\n  * `:key` - Defines the cache access key (optional). It overrides the\n    `:key_generator` option. If this option is not present, a default\n    key is generated by the configured or default key generator.\n\n  * `:opts` - Defines the cache options that will be passed as argument\n    to the invoked cache function (optional).\n\n  * `:match` - Match function `t:match_fun/0`. This function is for matching\n    and deciding whether the code-block evaluation result (which is received\n    as an argument) is cached or not. The function should return:\n\n      * `true` - the code-block evaluation result is cached as it is\n        (the default).\n      * `{true, value}` - `value` is cached. This is useful to set what\n        exactly must be cached.\n      * `{true, value, opts}` - `value` is cached with the options given by\n        `opts`. This return allows us to set the value to be cached, as well\n        as the runtime options for storing it (e.g.: the `ttl`).\n      * `false` - Nothing is cached.\n\n    The default match function looks like this:\n\n    ```elixir\n    fn\n      {:error, _} -> false\n      :error -> false\n      nil -> false\n      _ -> true\n    end\n    ```\n\n    By default, if the code-block evaluation returns any of the following\n    terms/values `nil`, `:error`, `{:error, term}`, the default match\n    function returns `false` (the returned result is not cached),\n    otherwise, `true` is returned (the returned result is cached).\n\n  * `:key_generator` - The custom key-generator to be used (optional).\n    If present, this option overrides the default key generator provided\n    by the cache, and it is applied only if the option `key:` or `keys:`\n    is not present. In other words, the option `key:` or `keys:` overrides\n    the `:key_generator` option. See \"The `:key_generator` option\" section\n    below for more information about the possible values.\n\n  * `:on_error` - It may be one of `:raise` (the default) or `:nothing`.\n    The decorators/annotations call the cache under the hood, hence,\n    by default, any error or exception at executing a cache command\n    is propagated. When this option is set to `:nothing`, any error\n    or exception executing a cache command is ignored and the annotated\n    function is executed normally.","ref":"Nebulex.Caching.Decorators.html#module-shared-options"},{"type":"module","title":"The `:cache` option - Nebulex.Caching.Decorators","doc":"The cache option can be the de defined cache module or an MFA tuple to\nresolve the cache dynamically in runtime. When it is an MFA tuple, the\nMFA is invoked passing the calling module, function name, and arguments\nby default, and the MFA arguments are passed as extra arguments.\nFor example:\n\n    @decorate cacheable(cache: {MyApp.Cache, :cache, []}, key: var)\n    def some_function(var) do\n      # Some logic ...\n    end\n\nThe annotated function above will call `MyApp.Cache.cache(mod, fun, args)`\nto resolve the cache in runtime, where `mod` is the calling module, `fun`\nthe calling function name, and `args` the calling arguments.\n\nAlso, we can define the function passing some extra arguments, like so:\n\n    @decorate cacheable(cache: {MyApp.Cache, :cache, [\"extra\"]}, key: var)\n    def some_function(var) do\n      # Some logic ...\n    end\n\nIn this case, the MFA will be invoked by adding the extra arguments, like:\n`MyApp.Cache.cache(mod, fun, args, \"extra\")`.","ref":"Nebulex.Caching.Decorators.html#module-the-cache-option"},{"type":"module","title":"The `:key_generator` option - Nebulex.Caching.Decorators","doc":"The possible values for the `:key_generator` are:\n\n  * A module implementing the `Nebulex.Caching.KeyGenerator` behaviour.\n\n  * A MFA tuple `{module, function, args}` for a function to call to\n    generate the key before the cache is invoked. A shorthand value of\n    `{module, args}` is equivalent to\n    `{module, :generate, [calling_module, calling_function_name, args]}`.","ref":"Nebulex.Caching.Decorators.html#module-the-key_generator-option"},{"type":"module","title":"Putting all together - Nebulex.Caching.Decorators","doc":"Supposing we are using `Ecto` and we want to define some cacheable functions\nwithin the context `MyApp.Accounts`:\n\n    # The config\n    config :my_app, MyApp.Cache,\n      gc_interval: 86_400_000, #=> 1 day\n      backend: :shards\n\n    # The Cache\n    defmodule MyApp.Cache do\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local\n    end\n\n    # Some Ecto schema\n    defmodule MyApp.Accounts.User do\n      use Ecto.Schema\n\n      schema \"users\" do\n        field(:username, :string)\n        field(:password, :string)\n        field(:role, :string)\n      end\n\n      def changeset(user, attrs) do\n        user\n        |> cast(attrs, [:username, :password, :role])\n        |> validate_required([:username, :password, :role])\n      end\n    end\n\n    # Accounts context\n    defmodule MyApp.Accounts do\n      use Nebulex.Caching\n\n      alias MyApp.Accounts.User\n      alias MyApp.{Cache, Repo}\n\n      @ttl :timer.hours(1)\n\n      @decorate cacheable(cache: Cache, key: {User, id}, opts: [ttl: @ttl])\n      def get_user!(id) do\n        Repo.get!(User, id)\n      end\n\n      @decorate cacheable(\n                  cache: Cache,\n                  key: {User, username},\n                  opts: [ttl: @ttl]\n                )\n      def get_user_by_username(username) do\n        Repo.get_by(User, [username: username])\n      end\n\n      @decorate cache_put(\n                  cache: Cache,\n                  keys: [{User, usr.id}, {User, usr.username}],\n                  match: &match_update/1\n                )\n      def update_user(%User{} = usr, attrs) do\n        usr\n        |> User.changeset(attrs)\n        |> Repo.update()\n      end\n\n      defp match_update({:ok, usr}), do: {true, usr}\n      defp match_update({:error, _}), do: false\n\n      @decorate cache_evict(\n                  cache: Cache,\n                  keys: [{User, usr.id}, {User, usr.username}]\n                )\n      def delete_user(%User{} = usr) do\n        Repo.delete(usr)\n      end\n\n      def create_user(attrs \\\\ %{}) do\n        %User{}\n        |> User.changeset(attrs)\n        |> Repo.insert()\n      end\n    end\n\nSee [Cache Usage Patterns Guide](http://hexdocs.pm/nebulex/cache-usage-patterns.html).","ref":"Nebulex.Caching.Decorators.html#module-putting-all-together"},{"type":"function","title":"Nebulex.Caching.Decorators.build_keyref/2","doc":"A convenience function for building a cache key reference when using the\n`cacheable` decorator. If you want to build an external reference, which is,\nreferencing a `key` stored in an external cache, you have to provide the\n`cache` where the `key` is located to. The `cache` argument is optional,\nand by default is `nil`, which means, the referenced `key` is in the same\ncache provided via `:key` or `:key_generator` options (internal reference).\n\n**NOTE:** In case you need to build a reference, consider using the macro\n`Nebulex.Caching.keyref/2` instead.\n\nSee `cacheable/3` decorator for more information about external references.","ref":"Nebulex.Caching.Decorators.html#build_keyref/2"},{"type":"function","title":"Examples - Nebulex.Caching.Decorators.build_keyref/2","doc":"iex> Nebulex.Caching.Decorators.build_keyref(\"my-key\")\n    {:\"$nbx_cache_keyref\", nil, \"my-key\"}\n    iex> Nebulex.Caching.Decorators.build_keyref(MyCache, \"my-key\")\n    {:\"$nbx_cache_keyref\", MyCache, \"my-key\"}","ref":"Nebulex.Caching.Decorators.html#build_keyref/2-examples"},{"type":"macro","title":"Nebulex.Caching.Decorators.cache_evict/1","doc":"","ref":"Nebulex.Caching.Decorators.html#cache_evict/1"},{"type":"function","title":"Nebulex.Caching.Decorators.cache_evict/3","doc":"Provides a way of annotating functions to be evicted (eviction aspect).\n\nOn function's completion, the given key or keys (depends on the `:key` and\n`:keys` options) are deleted from the cache.","ref":"Nebulex.Caching.Decorators.html#cache_evict/3"},{"type":"function","title":"Options - Nebulex.Caching.Decorators.cache_evict/3","doc":"* `:keys` - Defines the set of keys to be evicted from cache on function\n    completion. It overrides `:key` and `:key_generator` options.\n\n  * `:all_entries` - Defines if all entries must be removed on function\n    completion. Defaults to `false`.\n\n  * `:before_invocation` - Boolean to indicate whether the eviction should\n    occur after (the default) or before the function executes. The former\n    provides the same semantics as the rest of the annotations; once the\n    function completes successfully, an action (in this case eviction)\n    on the cache is executed. If the function does not execute (as it might\n    be cached) or an exception is raised, the eviction does not occur.\n    The latter (`before_invocation: true`) causes the eviction to occur\n    always, before the function is invoked; this is useful in cases where\n    the eviction does not need to be tied to the function outcome.\n\nSee the \"Shared options\" section at the module documentation.","ref":"Nebulex.Caching.Decorators.html#cache_evict/3-options"},{"type":"function","title":"Examples - Nebulex.Caching.Decorators.cache_evict/3","doc":"defmodule MyApp.Example do\n      use Nebulex.Caching\n\n      alias MyApp.Cache\n\n      @decorate cache_evict(cache: Cache, key: id)\n      def delete(id) do\n        # your logic (maybe write/delete data to the SoR)\n      end\n\n      @decorate cache_evict(cache: Cache, keys: [object.name, object.id])\n      def delete_object(object) do\n        # your logic (maybe write/delete data to the SoR)\n      end\n\n      @decorate cache_evict(cache: Cache, all_entries: true)\n      def delete_all do\n        # your logic (maybe write/delete data to the SoR)\n      end\n    end\n\nThe **Write-through** pattern is supported by this decorator. Your function\nprovides the logic to write data to the system-of-record (SoR) and the rest\nis provided by the decorator under-the-hood. But in contrast with `update`\ndecorator, when the data is written to the SoR, the key for that value is\ndeleted from cache instead of updated.","ref":"Nebulex.Caching.Decorators.html#cache_evict/3-examples"},{"type":"macro","title":"Nebulex.Caching.Decorators.cache_put/1","doc":"","ref":"Nebulex.Caching.Decorators.html#cache_put/1"},{"type":"function","title":"Nebulex.Caching.Decorators.cache_put/3","doc":"Provides a way of annotating functions to be evicted; but updating the\ncached key instead of deleting it.\n\nThe content of the cache is updated without interfering with the function\nexecution. That is, the method would always be executed and the result\ncached.\n\nThe difference between `cacheable/3` and `cache_put/3` is that `cacheable/3`\nwill skip running the function if the key exists in the cache, whereas\n`cache_put/3` will actually run the function and then put the result in\nthe cache.","ref":"Nebulex.Caching.Decorators.html#cache_put/3"},{"type":"function","title":"Options - Nebulex.Caching.Decorators.cache_put/3","doc":"* `:keys` - The set of cached keys to be updated with the returned value\n    on function completion. It overrides `:key` and `:key_generator`\n    options.\n\nSee the \"Shared options\" section at the module documentation.","ref":"Nebulex.Caching.Decorators.html#cache_put/3-options"},{"type":"function","title":"Examples - Nebulex.Caching.Decorators.cache_put/3","doc":"defmodule MyApp.Example do\n      use Nebulex.Caching\n\n      alias MyApp.Cache\n\n      @ttl :timer.hours(1)\n\n      @decorate cache_put(cache: Cache, key: id, opts: [ttl: @ttl])\n      def update!(id, attrs \\\\ %{}) do\n        # your logic (maybe write data to the SoR)\n      end\n\n      @decorate cache_put(\n                  cache: Cache,\n                  key: id,\n                  match: &match_fun/1,\n                  opts: [ttl: @ttl]\n                )\n      def update(id, attrs \\\\ %{}) do\n        # your logic (maybe write data to the SoR)\n      end\n\n      @decorate cache_put(\n                  cache: Cache,\n                  keys: [object.name, object.id],\n                  match: &match_fun/1,\n                  opts: [ttl: @ttl]\n                )\n      def update_object(object) do\n        # your logic (maybe write data to the SoR)\n      end\n\n      defp match_fun({:ok, updated}), do: {true, updated}\n      defp match_fun({:error, _}), do: false\n    end\n\nThe **Write-through** pattern is supported by this decorator. Your function\nprovides the logic to write data to the system-of-record (SoR) and the rest\nis provided by the decorator under-the-hood.","ref":"Nebulex.Caching.Decorators.html#cache_put/3-examples"},{"type":"function","title":"Nebulex.Caching.Decorators.cache_put/4","doc":"Convenience function for cache_put annotation.\n\n**NOTE:** For internal purposes only.","ref":"Nebulex.Caching.Decorators.html#cache_put/4"},{"type":"macro","title":"Nebulex.Caching.Decorators.cacheable/1","doc":"","ref":"Nebulex.Caching.Decorators.html#cacheable/1"},{"type":"function","title":"Nebulex.Caching.Decorators.cacheable/3","doc":"Provides a way of annotating functions to be cached (cacheable aspect).\n\nThe returned value by the code block is cached if it doesn't exist already\nin cache, otherwise, it is returned directly from cache and the code block\nis not executed.","ref":"Nebulex.Caching.Decorators.html#cacheable/3"},{"type":"function","title":"Options - Nebulex.Caching.Decorators.cacheable/3","doc":"* `:references` - (Optional) (`t:references/0`) Indicates the key given\n    by the option `:key` references another key given by the option\n    `:references`. In other words, when it is present, this option tells\n    the `cacheable` decorator to store the function's block result under\n    the referenced key given by the option `:references`, and the referenced\n    key under the key given by the option `:key`. The value could be:\n\n    * `nil` - (Default) It is ignored (no key references).\n    * `(term -> keyref | term)` - An anonymous function receiving the\n      result of the function's code block evaluation and must return the\n      referenced key. There is also a special type of return in case you\n      want to reference a key located in an external/different cache than\n      the one defined with the options `:key` or `:key_generator`. In this\n      scenario, you must return a special type `t:keyref/0`, which can be\n      build with the macro [`keyref/2`](`Nebulex.Caching.keyref/2`).\n      See the \"External referenced keys\" section below.\n    * `any` - It could be an explicit term or value, for example, a fixed\n      value or a function argument.\n\n    See the \"Referenced keys\" section for more information.\n\nSee the \"Shared options\" section at the module documentation.","ref":"Nebulex.Caching.Decorators.html#cacheable/3-options"},{"type":"function","title":"Examples - Nebulex.Caching.Decorators.cacheable/3","doc":"defmodule MyApp.Example do\n      use Nebulex.Caching\n\n      alias MyApp.Cache\n\n      @ttl :timer.hours(1)\n\n      @decorate cacheable(cache: Cache, key: id, opts: [ttl: @ttl])\n      def get_by_id(id) do\n        # your logic (maybe the loader to retrieve the value from the SoR)\n      end\n\n      @decorate cacheable(cache: Cache, key: email, references: & &1.id)\n      def get_by_email(email) do\n        # your logic (maybe the loader to retrieve the value from the SoR)\n      end\n\n      @decorate cacheable(cache: Cache, key: clauses, match: &match_fun/1)\n      def all(clauses) do\n        # your logic (maybe the loader to retrieve the value from the SoR)\n      end\n\n      defp match_fun([]), do: false\n      defp match_fun(_), do: true\n    end\n\nThe **Read-through** pattern is supported by this decorator. The loader to\nretrieve the value from the system-of-record (SoR) is your function's logic\nand the rest is provided by the macro under-the-hood.","ref":"Nebulex.Caching.Decorators.html#cacheable/3-examples"},{"type":"function","title":"Referenced keys - Nebulex.Caching.Decorators.cacheable/3","doc":"Referenced keys are particularly useful when you have multiple different\nkeys keeping the same value. For example, let's imagine we have an schema\n`User` with more than one unique field, like `:id`, `:email`, and `:token`.\nWe may have a module with functions retrieving the user account by any of\nthose fields, like so:\n\n    defmodule MyApp.UserAccounts do\n      use Nebulex.Caching\n\n      alias MyApp.Cache\n\n      @decorate cacheable(cache: Cache, key: id)\n      def get_user_account(id) do\n        # your logic ...\n      end\n\n      @decorate cacheable(cache: Cache, key: email)\n      def get_user_account_by_email(email) do\n        # your logic ...\n      end\n\n      @decorate cacheable(cache: Cache, key: token)\n      def get_user_account_by_token(token) do\n        # your logic ...\n      end\n\n      @decorate cache_evict(\n                  cache: Cache,\n                  keys: [user.id, user.email, user.token]\n                )\n      def update_user_account(user) do\n        # your logic ...\n      end\n    end\n\nAs you notice, all the three functions will end up storing the same user\nrecord under a different key. This is not very efficient in terms of\nmemory space, is it? Besides, when the user record is updated, we have\nto invalidate the previously cached entries, which means, we have to\nspecify in the `cache_evict` decorator all the different keys the user\naccount has ben cached under.\n\nBy means of the referenced keys, we can address it in a better and simpler\nway. The module will look like this:\n\n    defmodule MyApp.UserAccounts do\n      use Nebulex.Caching\n\n      alias MyApp.Cache\n\n      @decorate cacheable(cache: Cache, key: id)\n      def get_user_account(id) do\n        # your logic ...\n      end\n\n      @decorate cacheable(cache: Cache, key: email, references: & &1.id)\n      def get_user_account_by_email(email) do\n        # your logic ...\n      end\n\n      @decorate cacheable(cache: Cache, key: token, references: & &1.id)\n      def get_user_account_by_token(token) do\n        # your logic ...\n      end\n\n      @decorate cache_evict(cache: Cache, key: user.id)\n      def update_user_account(user) do\n        # your logic ...\n      end\n    end\n\nWith the option `:references` we are indicating to the `cacheable` decorator\nto store the user id (`& &1.id` - assuming the function returns an user\nrecord) under the key `email` and the key `token`, and the user record\nitself under the user id, which is the referenced key. This time, instead of\nstoring the same object three times, it will be stored only once under the\nuser id, and the other entries will just keep a reference to it. When the\nfunctions `get_user_account_by_email/1` or `get_user_account_by_token/1`\nare executed, the decorator will automatically handle it; under-the-hood,\nit will fetch the referenced key given by `email` or `token` first, and\nthen get the user record under the referenced key.\n\nOn the other hand, in the eviction function `update_user_account/1`, since\nthe user record is stored only once under the user's ID, we could set the\noption `:key` to the user's ID, without specifying multiple keys like in the\nprevious case. However, there is a caveat: _\"the `cache_evict` decorator\ndoesn't evict the references automatically\"_. See the\n[\"CAVEATS\"](#cacheable/3-caveats) section below.","ref":"Nebulex.Caching.Decorators.html#cacheable/3-referenced-keys"},{"type":"function","title":"External referenced keys - Nebulex.Caching.Decorators.cacheable/3","doc":"Previously, we saw how to work with referenced keys but on the same cache,\nlike \"internal references.\" Despite this being the typical case scenario,\nthere could be situations where you may want to reference a key stored in a\ndifferent or external cache. Why would I want to reference a key located in\na separate cache? There may be multiple reasons, but let's give a few\nexamples.\n\n  * One example is when you have a Redis cache; in such case, you likely\n    want to optimize the calls to Redis as much as possible. Therefore, you\n    should store the referenced keys in a local cache and the values in\n    Redis. This way, we only hit Redis to access the keys with the actual\n    values, and the decorator resolves the referenced keys locally.\n\n  * Another example is for keeping the cache key references isolated,\n    preferably locally. Then, apply a different eviction (or garbage\n    collection) policy for the references; one may want to expire the\n    references more often to avoid having dangling keys since the\n    `cache_evict` decorator doesn't remove the references automatically,\n    just the defined key (or keys). See the\n    [\"CAVEATS\"](#cacheable/3-caveats) section below.\n\nLet us modify the previous _\"user accounts\"_ example based on the Redis\nscenario:\n\n    defmodule MyApp.UserAccounts do\n      use Nebulex.Caching\n\n      alias MyApp.{LocalCache, RedisCache}\n\n      @decorate cacheable(cache: RedisCache, key: id)\n      def get_user_account(id) do\n        # your logic ...\n      end\n\n      @decorate cacheable(\n                  cache: LocalCache,\n                  key: email,\n                  references: &keyref(RedisCache, &1.id)\n                )\n      def get_user_account_by_email(email) do\n        # your logic ...\n      end\n\n      @decorate cacheable(\n                  cache: LocalCache,\n                  key: token,\n                  references: &keyref(RedisCache, &1.id)\n                )\n      def get_user_account_by_token(token) do\n        # your logic ...\n      end\n\n      @decorate cache_evict(cache: RedisCache, key: user.id)\n      def update_user_account(user) do\n        # your logic ...\n      end\n    end\n\nThe functions `get_user_account/1` and `update_user_account/2` use\n`RedisCache` to store the real value in Redis while\n`get_user_account_by_email/1` and `get_user_account_by_token/1` use\n`LocalCache` to store the referenced keys. Then, with the option\n`references: &keyref(RedisCache, &1.id)` we are telling the `cacheable`\ndecorator the referenced key given by `&1.id` is located in the cache\n`RedisCache`; underneath, the macro [`keyref/2`](`Nebulex.Caching.keyref/2`)\nbuilds the special return type for the external cache reference.","ref":"Nebulex.Caching.Decorators.html#cacheable/3-external-referenced-keys"},{"type":"function","title":"CAVEATS - Nebulex.Caching.Decorators.cacheable/3","doc":"* When the `cache_evict` decorator annotates a key (or keys) to evict, the\n  decorator removes only the entry associated with that key. Therefore, if\n  the key has references, those are not automatically removed, which means\n  dangling keys. However, there are multiple ways to address dangling keys\n  (or references):\n\n  * The first (and the simplest) sets a TTL to the reference. For example:\n    `cacheable(key: name, references: & &1.id, opts: [ttl: @ttl])`. You can\n    also specify a different TTL for the referenced key:\n    `references: &keyref(&1.id, ttl: @another_ttl)`.\n\n  * The second alternative, perhaps the most recommended, is having a\n    separate cache to keep the references (e.g., a cache using the local\n    adapter). This way, you could provide a different eviction or GC\n    configuration to run the GC more often and keep the references cache\n    clean. See\n    [\"External referenced keys\"](#cacheable/3-external-referenced-keys).\n\n  * The third alternative uses the `:keys` option for specifying a key and\n    its references. For example, if you have\n    `@decorate cacheable(key: email, references: & &1.id)`, the eviction\n    may look like this `@decorate cache_evict(keys: [user.id, user.email])`.\n    This one is perhaps the least ideal option because it is cumbersome;\n    you have to know and specify the key and all its references, and at the\n    same time, you will need to have access to the key and references in the\n    arguments, which sometimes is not possible because you may receive only\n    the ID, but not the email.","ref":"Nebulex.Caching.Decorators.html#cacheable/3-caveats"},{"type":"function","title":"Nebulex.Caching.Decorators.eval_cacheable/7","doc":"Convenience function for evaluating the `cacheable` decorator in runtime.\n\n**NOTE:** For internal purposes only.","ref":"Nebulex.Caching.Decorators.html#eval_cacheable/7"},{"type":"function","title":"Nebulex.Caching.Decorators.eval_match/5","doc":"Convenience function for evaluating the `:match` function in runtime.\n\n**NOTE:** For internal purposes only.\n\n**NOTE:** Workaround to avoid dialyzer warnings when using declarative\nannotation-based caching via decorators.","ref":"Nebulex.Caching.Decorators.html#eval_match/5"},{"type":"function","title":"Nebulex.Caching.Decorators.run_cmd/5","doc":"Convenience function for ignoring cache errors when `:on_error` option\nis set to `:nothing`\n\n**NOTE:** For internal purposes only.","ref":"Nebulex.Caching.Decorators.html#run_cmd/5"},{"type":"type","title":"Nebulex.Caching.Decorators.keyref/0","doc":"Type spec for a key reference","ref":"Nebulex.Caching.Decorators.html#t:keyref/0"},{"type":"type","title":"Nebulex.Caching.Decorators.match_fun/0","doc":"Match function type","ref":"Nebulex.Caching.Decorators.html#t:match_fun/0"},{"type":"type","title":"Nebulex.Caching.Decorators.on_error_opt/0","doc":"Type for :on_error option","ref":"Nebulex.Caching.Decorators.html#t:on_error_opt/0"},{"type":"type","title":"Nebulex.Caching.Decorators.references/0","doc":"Type spec for the option :references","ref":"Nebulex.Caching.Decorators.html#t:references/0"},{"type":"behaviour","title":"Nebulex.Caching.KeyGenerator","doc":"Cache key generator. Used for creating a key based on the given module,\nfunction name and its arguments (the module and function name are used\nas context).\n\nSee the default implementation `Nebulex.Caching.SimpleKeyGenerator`.","ref":"Nebulex.Caching.KeyGenerator.html"},{"type":"behaviour","title":"Caveats when using the key generator - Nebulex.Caching.KeyGenerator","doc":"Since the callback `c:generate/3` is invoked passing the calling module where\nthe annotated function is defined, the name of the annotated function, and the\narguments given to that annotated function, there are some caveats to keep in\nmind:\n\n  * Only arguments explicitly assigned to a variable will be included when\n    calling the callback `c:generate/3`.\n  * Ignored or underscored arguments will be ignored.\n  * Pattern-matching expressions without a variable assignment will be\n    ignored. If there is a pattern-matching, it has to be explicitly\n    assigned to a variable so it can be included when calling the\n    callback `c:generate/3`.\n\nFor example, suppose you have a module with an annotated function:\n\n    defmodule MyApp.SomeModule do\n      use Nebulex.Caching\n\n      alias MyApp.{Cache, CustomKeyGenerator}\n\n      @decorate cacheable(cache: Cache, key_generator: CustomKeyGenerator)\n      def get_something(x, _ignored, _, {_, _}, [_, _], %{a: a}, %{} = y) do\n        # Function's logic\n      end\n    end\n\nThe generator will be invoked like so:\n\n    MyKeyGenerator.generate(MyApp.SomeModule, :get_something, [x, y])\n\nBased on the caveats described above, only the arguments `x` and `y` are\nincluded when calling the callback `c:generate/3`.","ref":"Nebulex.Caching.KeyGenerator.html#module-caveats-when-using-the-key-generator"},{"type":"callback","title":"Nebulex.Caching.KeyGenerator.generate/3","doc":"Generates a key for the given `module`, `function_name`, and its `args`.","ref":"Nebulex.Caching.KeyGenerator.html#c:generate/3"},{"type":"type","title":"Nebulex.Caching.KeyGenerator.t/0","doc":"Key generator type","ref":"Nebulex.Caching.KeyGenerator.html#t:t/0"},{"type":"module","title":"Nebulex.Caching.SimpleKeyGenerator","doc":"Default key generator implementation.\n\nIt implementats a simple algorithm:\n\n  * If no params are given, return `0`.\n  * If only one param is given, return that param as key.\n  * If more than one param is given, return a key computed from the hashes\n    of all parameters (`:erlang.phash2(args)`).\n\n> Based on the [default key generation in Spring Cache Abstraction](https://docs.spring.io/spring-framework/docs/3.2.x/spring-framework-reference/html/cache.html#cache-annotations-cacheable-default-key).\n\nThis implementation aims to cover those simple/generic scenarios where the\nkey generated based on the arguments only, fulfill the needs. For example:\n\n    defmodule MyApp.Users do\n      use Nebulex.Caching\n\n      alias MayApp.Cache\n\n      @decorate cacheable(cache: Cache)\n      def get_user(id) do\n        # logic for retrieving a user...\n      end\n\n      @decorate cache_evict(cache: Cache)\n      def delete_user(id) do\n        # logic for deleting a user...\n      end\n    end\n\nThe key generator will generate the same key for both, cacheable and\nevict functions; since it is generated based on the arguments only.","ref":"Nebulex.Caching.SimpleKeyGenerator.html"},{"type":"module","title":"Nebulex.Entry","doc":"Defines a Cache Entry.\n\nThis is the structure used by the caches for representing cache entries.","ref":"Nebulex.Entry.html"},{"type":"function","title":"Nebulex.Entry.decode/2","doc":"Decodes a previously encoded entry.","ref":"Nebulex.Entry.html#decode/2"},{"type":"function","title":"Example - Nebulex.Entry.decode/2","doc":"iex> \"hello\"\n    ...> |> Nebulex.Entry.encode()\n    ...> |> Nebulex.Entry.decode()\n    \"hello\"","ref":"Nebulex.Entry.html#decode/2-example"},{"type":"function","title":"Nebulex.Entry.encode/2","doc":"Encodes a cache entry.","ref":"Nebulex.Entry.html#encode/2"},{"type":"function","title":"Example - Nebulex.Entry.encode/2","doc":"iex> \"hello\"\n    ...> |> Nebulex.Entry.encode()\n    ...> |> Nebulex.Entry.decode()\n    \"hello\"","ref":"Nebulex.Entry.html#encode/2-example"},{"type":"function","title":"Nebulex.Entry.expired?/1","doc":"Returns whether the given `entry` has expired or not.","ref":"Nebulex.Entry.html#expired?/1"},{"type":"function","title":"Example - Nebulex.Entry.expired?/1","doc":"iex> Nebulex.Entry.expired?(%Nebulex.Entry{})\n    false\n\n    iex> Nebulex.Entry.expired?(\n    ...>   %Nebulex.Entry{touched: Nebulex.Time.now() - 10, ttl: 1}\n    ...> )\n    true","ref":"Nebulex.Entry.html#expired?/1-example"},{"type":"function","title":"Nebulex.Entry.ttl/1","doc":"Returns the remaining time-to-live.","ref":"Nebulex.Entry.html#ttl/1"},{"type":"function","title":"Example - Nebulex.Entry.ttl/1","doc":"iex> Nebulex.Entry.ttl(%Nebulex.Entry{})\n    :infinity\n\n    iex> ttl =\n    ...>   Nebulex.Entry.ttl(\n    ...>     %Nebulex.Entry{touched: Nebulex.Time.now(), ttl: 100}\n    ...>   )\n    iex> ttl > 0\n    true","ref":"Nebulex.Entry.html#ttl/1-example"},{"type":"type","title":"Nebulex.Entry.t/0","doc":"Defines a generic struct for a cache entry.\n\nThe entry depends on the adapter completely, this struct/type aims to define\nthe common fields.","ref":"Nebulex.Entry.html#t:t/0"},{"type":"module","title":"Nebulex.Hook","doc":"Pre/Post Hooks\n\nSince `v2.0.0`, pre/post hooks are not supported and/or handled by `Nebulex`\nitself. Hooks feature is not a common use-case and also it is something that\ncan be be easily implemented on top of the Cache at the application level.\n\nNevertheless, to keep backward compatibility somehow, `Nebulex` provides the\nnext decorators for implementing pre/post hooks very easily.\n\n## `before` decorator\n\nThe `before` decorator is declared for performing a hook action or callback\nbefore the annotated function is executed.\n\n    @decorate before(fn %Nebulex.Hook{} = hook -> inspect(hook) end)\n    def some_fun(var) do\n      # logic ...\n    end\n\n## `after_return` decorator\n\nThe `after_return` decorator is declared for performing a hook action or\ncallback after the annotated function is executed and its return is passed\nthrough the `return:` attribute.\n\n    @decorate after_return(&inspect(&1.return))\n    def some_fun(var) do\n      # logic ...\n    end\n\n## `around` decorator\n\nThe final kind of hook is `around` decorator. The `around` decorator runs\n\"around\" the annotated function execution. It has the opportunity to do\nwork both **before** and **after** the function executes. This means the\ngiven hook function is invoked twice, before and after the code-block is\nevaluated.\n\n    @decorate around(&inspect(&1.step))\n    def some_fun(var) do\n      # logic ...\n    end","ref":"Nebulex.Hook.html"},{"type":"module","title":"Putting all together - Nebulex.Hook","doc":"Suppose we want to track all cache calls (before and after they are called)\nby logging them (including the execution time). In this case, we need to\nprovide a pre/post hook to log these calls.\n\nFirst of all, we have to create a module implementing the hook function:\n\n    defmodule MyApp.Tracker do\n      use GenServer\n\n      alias Nebulex.Hook\n\n      require Logger\n\n      @actions [:get, :put]","ref":"Nebulex.Hook.html#module-putting-all-together"},{"type":"module","title":"API - Nebulex.Hook","doc":"def start_link(opts \\\\ []) do\n        GenServer.start_link(__MODULE__, opts, name: __MODULE__)\n      end\n\n      def track(%Hook{step: :before, name: name}) when name in @actions do\n        System.system_time(:microsecond)\n      end\n\n      def track(%Hook{step: :after_return, name: name} = event) when name in @actions do\n        GenServer.cast(__MODULE__, {:track, event})\n      end\n\n      def track(hook), do: hook","ref":"Nebulex.Hook.html#module-api"},{"type":"module","title":"GenServer Callbacks - Nebulex.Hook","doc":"@impl true\n      def init(_opts) do\n        {:ok, %{}}\n      end\n\n      @impl true\n      def handle_cast({:track, %Hook{acc: start} = hook}, state) do\n        diff = System.system_time(:microsecond) - start\n        Logger.info(\"#=> #{hook.module}.#{hook.name}/#{hook.arity}, Duration: #{diff}\")\n        {:noreply, state}\n      end\n    end\n\nAnd then, in the Cache:\n\n    defmodule MyApp.Cache do\n      use Nebulex.Hook\n      @decorate_all around(&MyApp.Tracker.track/1)\n\n      use Nebulex.Cache,\n        otp_app: :my_app,\n        adapter: Nebulex.Adapters.Local\n    end\n\nTry it out:\n\n    iex> MyApp.Cache.put 1, 1\n    10:19:47.736 [info] Elixir.MyApp.Cache.put/3, Duration: 27\n    iex> MyApp.Cache.get 1\n    10:20:14.941 [info] Elixir.MyApp.Cache.get/2, Duration: 11","ref":"Nebulex.Hook.html#module-genserver-callbacks"},{"type":"macro","title":"Nebulex.Hook.after_return/1","doc":"","ref":"Nebulex.Hook.html#after_return/1"},{"type":"function","title":"Nebulex.Hook.after_return/3","doc":"After-return decorator.\n\nIntercepts any call to the annotated function and calls the given `fun`\nafter the logic is executed, and the returned result is passed through\nthe `return:` attribute.","ref":"Nebulex.Hook.html#after_return/3"},{"type":"function","title":"Example - Nebulex.Hook.after_return/3","doc":"defmodule MyApp.Example do\n      use Nebulex.Hook\n\n      @decorate after_return(&inspect(&1))\n      def some_fun(var) do\n        # logic ...\n      end\n    end","ref":"Nebulex.Hook.html#after_return/3-example"},{"type":"macro","title":"Nebulex.Hook.around/1","doc":"","ref":"Nebulex.Hook.html#around/1"},{"type":"function","title":"Nebulex.Hook.around/3","doc":"Around decorator.\n\nIntercepts any call to the annotated function and calls the given `fun`\nbefore and after the logic is executed. The result of the first call to\nthe hook function is passed through the `acc:` attribute, so it can be\nused in the next call (after return). Finally, as the `after_return`\ndecorator, the returned code-block evaluation is passed through the\n`return:` attribute.","ref":"Nebulex.Hook.html#around/3"},{"type":"function","title":"Example - Nebulex.Hook.around/3","doc":"defmodule MyApp.Profiling do\n      alias Nebulex.Hook\n\n      def prof(%Hook{step: :before}) do\n        System.system_time(:microsecond)\n      end\n\n      def prof(%Hook{step: :after_return, acc: start} = hook) do\n        :telemetry.execute(\n          [:my_app, :profiling],\n          %{duration: System.system_time(:microsecond) - start},\n          %{module: hook.module, name: hook.name}\n        )\n      end\n    end\n\n    defmodule MyApp.Example do\n      use Nebulex.Hook\n\n      @decorate around(&MyApp.Profiling.prof/1)\n      def some_fun(var) do\n        # logic ...\n      end\n    end","ref":"Nebulex.Hook.html#around/3-example"},{"type":"macro","title":"Nebulex.Hook.before/1","doc":"","ref":"Nebulex.Hook.html#before/1"},{"type":"function","title":"Nebulex.Hook.before/3","doc":"Before decorator.\n\nIntercepts any call to the annotated function and calls the given `fun`\nbefore the logic is executed.","ref":"Nebulex.Hook.html#before/3"},{"type":"function","title":"Example - Nebulex.Hook.before/3","doc":"defmodule MyApp.Example do\n      use Nebulex.Hook\n\n      @decorate before(&inspect(&1))\n      def some_fun(var) do\n        # logic ...\n      end\n    end","ref":"Nebulex.Hook.html#before/3-example"},{"type":"function","title":"Nebulex.Hook.eval_hook/3","doc":"This function is for internal purposes.","ref":"Nebulex.Hook.html#eval_hook/3"},{"type":"type","title":"Nebulex.Hook.hook_fun/0","doc":"","ref":"Nebulex.Hook.html#t:hook_fun/0"},{"type":"type","title":"Nebulex.Hook.t/0","doc":"","ref":"Nebulex.Hook.html#t:t/0"},{"type":"module","title":"Nebulex.RPC","doc":"RPC utilities for distributed task execution.\n\nThis module uses supervised tasks underneath `Task.Supervisor`.\n\n> **NOTE:** The approach by using distributed tasks will be deprecated\n  in the future in favor of `:erpc`.","ref":"Nebulex.RPC.html"},{"type":"function","title":"Nebulex.RPC.call/6","doc":"Evaluates `apply(mod, fun, args)` on node `node` and returns the corresponding\nevaluation result, or `{:badrpc, reason}` if the call fails.\n\nA timeout, in milliseconds or `:infinity`, can be given with a default value\nof `5000`. It uses `Task.await/2` internally.","ref":"Nebulex.RPC.html#call/6"},{"type":"function","title":"Example - Nebulex.RPC.call/6","doc":"iex> Nebulex.RPC.call(:my_task_sup, :node1, Kernel, :to_string, [1])\n    \"1\"","ref":"Nebulex.RPC.html#call/6-example"},{"type":"function","title":"Nebulex.RPC.multi_call/3","doc":"In contrast to a regular single-node RPC, a multicall is an RPC that is sent\nconcurrently from one client to multiple servers. The function evaluates\n`apply(mod, fun, args)` on each `node_group` entry and collects the answers.\nThen, evaluates the `reducer` function (set in the `opts`) on each answer.\n\nThis function is similar to `:rpc.multicall/5`.","ref":"Nebulex.RPC.html#multi_call/3"},{"type":"function","title":"Options - Nebulex.RPC.multi_call/3","doc":"* `:timeout` - A timeout, in milliseconds or `:infinity`, can be given with\n    a default value of `5000`. It uses `Task.yield_many/2` internally.\n\n  * `:reducer` - Reducer function to be executed on each collected result.\n    (check out `reducer` type).","ref":"Nebulex.RPC.html#multi_call/3-options"},{"type":"function","title":"Example - Nebulex.RPC.multi_call/3","doc":"iex> Nebulex.RPC.multi_call(\n    ...>   :my_task_sup,\n    ...>   %{\n    ...>     node1: {Kernel, :to_string, [1]},\n    ...>     node2: {Kernel, :to_string, [2]}\n    ...>   },\n    ...>   timeout: 10_000,\n    ...>   reducer: {\n    ...>     [],\n    ...>     fn\n    ...>       {:ok, res}, _node_callback, acc ->\n    ...>         [res | acc]\n    ...>\n    ...>       {:error, _}, _node_callback, acc ->\n    ...>         acc\n    ...>     end\n    ...>   }\n    ...> )\n    [\"1\", \"2\"]","ref":"Nebulex.RPC.html#multi_call/3-example"},{"type":"function","title":"Nebulex.RPC.multi_call/6","doc":"Similar to `multi_call/3` but the same `node_callback` (given by `module`,\n`fun`, `args`) is executed on all `nodes`; Internally it creates a\n`node_group` with the same `node_callback` for each node.","ref":"Nebulex.RPC.html#multi_call/6"},{"type":"function","title":"Options - Nebulex.RPC.multi_call/6","doc":"Same options as `multi_call/3`.","ref":"Nebulex.RPC.html#multi_call/6-options"},{"type":"function","title":"Example - Nebulex.RPC.multi_call/6","doc":"iex> Nebulex.RPC.multi_call(\n    ...>   :my_task_sup,\n    ...>   [:node1, :node2],\n    ...>   Kernel,\n    ...>   :to_string,\n    ...>   [1],\n    ...>   timeout: 5000,\n    ...>   reducer: {\n    ...>     [],\n    ...>     fn\n    ...>       {:ok, res}, _node_callback, acc ->\n    ...>         [res | acc]\n    ...>\n    ...>       {:error, _}, _node_callback, acc ->\n    ...>         acc\n    ...>     end\n    ...>   }\n    ...> )\n    [\"1\", \"1\"]","ref":"Nebulex.RPC.html#multi_call/6-example"},{"type":"function","title":"Nebulex.RPC.rpc_multi_call/3","doc":"","ref":"Nebulex.RPC.html#rpc_multi_call/3"},{"type":"function","title":"Nebulex.RPC.rpc_multi_call/6","doc":"","ref":"Nebulex.RPC.html#rpc_multi_call/6"},{"type":"type","title":"Nebulex.RPC.callback/0","doc":"Task callback","ref":"Nebulex.RPC.html#t:callback/0"},{"type":"type","title":"Nebulex.RPC.node_callback/0","doc":"Group entry: node -> callback","ref":"Nebulex.RPC.html#t:node_callback/0"},{"type":"type","title":"Nebulex.RPC.node_group/0","doc":"Node group","ref":"Nebulex.RPC.html#t:node_group/0"},{"type":"type","title":"Nebulex.RPC.reducer/0","doc":"Reducer spec","ref":"Nebulex.RPC.html#t:reducer/0"},{"type":"type","title":"Nebulex.RPC.reducer_fun/0","doc":"Reducer function spec","ref":"Nebulex.RPC.html#t:reducer_fun/0"},{"type":"type","title":"Nebulex.RPC.task_sup/0","doc":"Task supervisor","ref":"Nebulex.RPC.html#t:task_sup/0"},{"type":"module","title":"Nebulex.Stats","doc":"Stats data type.\n\nStats struct defines two main keys:\n\n* `:measurements` - A map with the measurements provided by the underlying\n  adapter.\n* `:metadata` - A map for including additional information; also provided\n  by the underlying adapter.","ref":"Nebulex.Stats.html"},{"type":"module","title":"Measurements - Nebulex.Stats","doc":"The following measurements are expected to be present and fed by the\nunderlying adapter:\n\n  * `:evictions` - When a cache entry is removed.\n  * `:expirations` - When a cache entry is expired.\n  * `:hits` - When a key is looked up in cache and found.\n  * `:misses` - When a key is looked up in cache but not found.\n  * `:updates` - When an existing cache entry is or updated.\n  * `:writes` - When a cache entry is inserted or overwritten.","ref":"Nebulex.Stats.html#module-measurements"},{"type":"module","title":"Metadata - Nebulex.Stats","doc":"Despite the adapters can include any additional or custom metadata, It is\nrecommended they include the following keys:\n\n  * `:cache` - The cache module, or the name (if an explicit name has been\n    given to the cache).\n\n**IMPORTANT:** Since the adapter may include any additional or custom\nmeasurements, as well as metadata, it is recommended to check out the\nadapter's documentation.","ref":"Nebulex.Stats.html#module-metadata"},{"type":"type","title":"Nebulex.Stats.t/0","doc":"Nebulex.Stats data type","ref":"Nebulex.Stats.html#t:t/0"},{"type":"module","title":"Nebulex.Telemetry","doc":"Telemetry wrapper.","ref":"Nebulex.Telemetry.html"},{"type":"function","title":"Nebulex.Telemetry.attach_many/4","doc":"","ref":"Nebulex.Telemetry.html#attach_many/4"},{"type":"function","title":"Nebulex.Telemetry.detach/1","doc":"","ref":"Nebulex.Telemetry.html#detach/1"},{"type":"function","title":"Nebulex.Telemetry.execute/3","doc":"","ref":"Nebulex.Telemetry.html#execute/3"},{"type":"function","title":"Nebulex.Telemetry.span/3","doc":"","ref":"Nebulex.Telemetry.html#span/3"},{"type":"module","title":"Nebulex.Telemetry.StatsHandler","doc":"Telemetry handler for aggregating cache stats; it relies on the default stats\nimplementation based on Erlang counters. See `Nebulex.Adapter.Stats`.\n\nThis handler is used by the built-in local adapter when the option `:stats`\nis set to `true`.","ref":"Nebulex.Telemetry.StatsHandler.html"},{"type":"module","title":"Nebulex.Time","doc":"Time utilities.","ref":"Nebulex.Time.html"},{"type":"function","title":"Nebulex.Time.now/1","doc":"Returns the current system time in the given time unit.\n\nThe `unit` is set to `:millisecond` by default.","ref":"Nebulex.Time.html#now/1"},{"type":"function","title":"Examples - Nebulex.Time.now/1","doc":"iex> now = Nebulex.Time.now()\n    iex> is_integer(now) and now > 0\n    true","ref":"Nebulex.Time.html#now/1-examples"},{"type":"function","title":"Nebulex.Time.timeout?/1","doc":"Returns `true` if the given `timeout` is a valid timeout; otherwise, `false`\nis returned.\n\nValid timeout: `:infinity | non_neg_integer()`.","ref":"Nebulex.Time.html#timeout?/1"},{"type":"function","title":"Examples - Nebulex.Time.timeout?/1","doc":"iex> Nebulex.Time.timeout?(1)\n    true\n\n    iex> Nebulex.Time.timeout?(:infinity)\n    true\n\n    iex> Nebulex.Time.timeout?(-1)\n    false\n\n    iex> Nebulex.Time.timeout?(1.1)\n    false","ref":"Nebulex.Time.html#timeout?/1-examples"},{"type":"exception","title":"Nebulex.KeyAlreadyExistsError","doc":"Raised at runtime when a key already exists in cache.","ref":"Nebulex.KeyAlreadyExistsError.html"},{"type":"type","title":"Nebulex.KeyAlreadyExistsError.t/0","doc":"","ref":"Nebulex.KeyAlreadyExistsError.html#t:t/0"},{"type":"exception","title":"Nebulex.QueryError","doc":"Raised at runtime when the query is invalid.","ref":"Nebulex.QueryError.html"},{"type":"type","title":"Nebulex.QueryError.t/0","doc":"","ref":"Nebulex.QueryError.html#t:t/0"},{"type":"exception","title":"Nebulex.RPCError","doc":"Raised at runtime when a RPC error occurs.","ref":"Nebulex.RPCError.html"},{"type":"type","title":"Nebulex.RPCError.t/0","doc":"","ref":"Nebulex.RPCError.html#t:t/0"},{"type":"exception","title":"Nebulex.RPCMultiCallError","doc":"Raised at runtime when a RPC multi_call error occurs.","ref":"Nebulex.RPCMultiCallError.html"},{"type":"type","title":"Nebulex.RPCMultiCallError.t/0","doc":"","ref":"Nebulex.RPCMultiCallError.html#t:t/0"},{"type":"exception","title":"Nebulex.RegistryLookupError","doc":"Raised at runtime when the cache was not started or it does not exist.","ref":"Nebulex.RegistryLookupError.html"},{"type":"type","title":"Nebulex.RegistryLookupError.t/0","doc":"","ref":"Nebulex.RegistryLookupError.html#t:t/0"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nThis guide is an introduction to [Nebulex](https://github.com/cabol/nebulex),\na local and distributed caching toolkit for Elixir. Nebulex API is pretty much\ninspired by [Ecto](https://github.com/elixir-ecto/ecto), taking advantage of\nits simplicity, flexibility and pluggable architecture. In the same way\nas Ecto, developers can provide their own cache (adapter) implementations.\n\nIn this guide, we're going to learn some basics about Nebulex, such as insert,\nretrieve and destroy cache entries.","ref":"getting-started.html"},{"type":"extras","title":"Adding Nebulex to an application - Getting Started","doc":"Let's start creating a new Elixir application by running this command:\n\n```\nmix new blog --sup\n```\n\nThe `--sup` option ensures that this application has\n[a supervision tree](http://elixir-lang.org/getting-started/mix-otp/supervisor-and-application.html),\nwhich will be needed by Nebulex later on.\n\nTo add Nebulex to this application, there are a few steps that we need to take.\n\nThe first step will be adding Nebulex to our `mix.exs` file, which we'll do by\nchanging the `deps` definition in that file to this:\n\n```elixir\ndefp deps do\n  [\n    {:nebulex, \"~> 2.6\"},\n    {:shards, \"~> 1.0\"},      #=> When using :shards as backend\n    {:decorator, \"~> 1.4\"},   #=> When using Caching Annotations\n    {:telemetry, \"~> 1.0\"}    #=> When using the Telemetry events (Nebulex stats)\n  ]\nend\n```\n\nIn order to give more flexibility and loading only needed dependencies, Nebulex\nmakes all its dependencies as optional. For example:\n\n  * For intensive workloads, you may want to use `:shards` as the backend for\n    the local adapter and having partitioned tables. In such a case, you have\n    to add `:shards` to the dependency list.\n\n  * For enabling the usage of\n    [declarative annotation-based caching via decorators][nbx_caching],\n    you have to add `:decorator` to the dependency list.\n\n  * For enabling Telemetry events to be dispatched when using Nebulex,\n    you have to add `:telemetry` to the dependency list.\n    See [telemetry guide][telemetry].\n\n  * If you want to use an external adapter (e.g: Cachex or Redis adapter), you\n    have to add the adapter dependency too.\n\n[nbx_caching]: http://hexdocs.pm/nebulex/Nebulex.Caching.html\n[telemetry]: http://hexdocs.pm/nebulex/telemetry.html\n\nTo install these dependencies, we will run this command:\n\n```\nmix deps.get\n```\n\nWe now need to define a Cache and setup some configuration for Nebulex so that\nwe can perform actions on a cache from within the application's code.\n\nWe can set up this configuration by running this command:\n\n```\nmix nbx.gen.cache -c Blog.Cache\n```\n\nThis command will generate the configuration required to use the cache. The\nfirst bit of configuration is in `config/config.exs`:\n\n```elixir\nconfig :blog, Blog.Cache,\n  # When using :shards as backend\n  # backend: :shards,\n  # GC interval for pushing new generation: 12 hrs\n  gc_interval: :timer.hours(12),\n  # Max 1 million entries in cache\n  max_size: 1_000_000,\n  # Max 2 GB of memory\n  allocated_memory: 2_000_000_000,\n  # GC min timeout: 10 sec\n  gc_cleanup_min_timeout: :timer.seconds(10),\n  # GC max timeout: 10 min\n  gc_cleanup_max_timeout: :timer.minutes(10)\n```\n\nAssuming we will use `:shards` as backend, can add uncomment the first line in\nthe config\n\n```elixir\nconfig :blog, Blog.Cache,\n  # When using :shards as backend\n  backend: :shards,\n  # GC interval for pushing new generation: 12 hrs\n  gc_interval: :timer.hours(12),\n  # Max 1 million entries in cache\n  max_size: 1_000_000,\n  # Max 2 GB of memory\n  allocated_memory: 2_000_000_000,\n  # GC min timeout: 10 sec\n  gc_cleanup_min_timeout: :timer.seconds(10),\n  # GC max timeout: 10 min\n  gc_cleanup_max_timeout: :timer.minutes(10)\n```\n\n> By default, `partitions:` option is set to `System.schedulers_online()`.\n\n**NOTE:** For more information about the provided options, see the adapter's\ndocumentation.\n\nAnd the `Blog.Cache` module is defined in `lib/blog/cache.ex` by our\n`mix nbx.gen.cache` command:\n\n```elixir\ndefmodule Blog.Cache do\n  use Nebulex.Cache,\n    otp_app: :blog,\n    adapter: Nebulex.Adapters.Local\nend\n```\n\nThis module is what we'll be using to interact with the cache. It uses the\n`Nebulex.Cache` module and it expects the `:otp_app` as option. The `otp_app`\ntells Nebulex which Elixir application it can look for cache configuration in.\nIn this case, we've specified that it is the `:blog` application where Nebulex\ncan find that configuration and so Nebulex will use the configuration that was\nset up in `config/config.exs`.\n\nThe final piece of configuration is to setup the `Blog.Cache` as a\nsupervisor within the application's supervision tree, which we can do in\n`lib/blog/application.ex`, inside the `start/2` function:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    Blog.Cache\n  ]\n\n  ...\n```\n\nThis piece of configuration will start the Nebulex process which receives and\nexecutes our application's commands. Without it, we wouldn't be able to use\nthe cache at all!\n\nWe've now configured our application so that it's able to execute commands\nagainst our cache.\n\n**IMPORTANT:** Make sure the cache is put in first place within the children\nlist, or at least before the process or processes using it. Otherwise, there\ncould be race conditions causing `Nebulex.RegistryLookupError` errors;\nprocesses attempting to use the cache and this one hasn't been even\nstarted.","ref":"getting-started.html#adding-nebulex-to-an-application"},{"type":"extras","title":"Inserting entries - Getting Started","doc":"We can insert a new entries into our blog cache with this code:\n\n```elixir\niex> user = %{id: 1, first_name: \"Galileo\", last_name: \"Galilei\"}\niex> Blog.Cache.put(user[:id], user, ttl: :timer.hours(1))\n:ok\n```\n\nTo insert the data into our cache, we call `put` on `Blog.Cache`. This function\ntells Nebulex that we want to insert a new key/value entry into the cache\ncorresponding `Blog.Cache`.\n\nIt is also possible to insert multiple entries at once:\n\n```elixir\niex> users = %{\n...>   1 => %{id: 1, first_name: \"Galileo\", last_name: \"Galilei\"},\n...>   2 => %{id: 2, first_name: \"Charles\", last_name: \"Darwin\"},\n...>   3 => %{id: 3, first_name: \"Albert\", last_name: \"Einstein\"}\n...> }\niex> Blog.Cache.put_all(users)\n:ok\n```\n\n> The given entries can be a `map` or a Key/Value tuple list.","ref":"getting-started.html#inserting-entries"},{"type":"extras","title":"Inserting new entries and replacing existing ones - Getting Started","doc":"As we saw previously, `put` creates a new entry in cache if it doesn't exist,\nor overrides it if it does exist (including the `:ttl`). However, there might\nbe circumstances where we want to set the entry only if it doesn't exit or the\nother way around, this is where `put_new` and `replace` functions come in.\n\nLet's try `put_new` and `put_new!` functions:\n\n```elixir\niex> new_user = %{id: 4, first_name: \"John\", last_name: \"Doe\"}\niex> Blog.Cache.put_new(new_user.id, new_user, ttl: 900)\ntrue\n\niex> Blog.Cache.put_new(new_user.id, new_user)\nfalse\n\n# same as previous one but raises `Nebulex.KeyAlreadyExistsError`\niex> Blog.Cache.put_new!(new_user.id, new_user)\n```\n\nNow `replace` and `replace!` functions:\n\n```elixir\niex> existing_user = %{id: 5, first_name: \"John\", last_name: \"Doe2\"}\niex> Blog.Cache.replace(existing_user.id, existing_user)\nfalse\n\niex> Blog.Cache.put_new(existing_user.id, existing_user)\ntrue\n\niex> Blog.Cache.replace(existing_user.id, existing_user, ttl: 900)\ntrue\n\n# same as previous one but raises `KeyError`\niex> Blog.Cache.replace!(100, existing_user)\n```\n\nIt is also possible to insert multiple new entries at once:\n\n```elixir\niex> new_users = %{\n...>   6 => %{id: 6, first_name: \"Isaac\", last_name: \"Newton\"},\n...>   7 => %{id: 7, first_name: \"Marie\", last_name: \"Curie\"}\n...> }\niex> Blog.Cache.put_new_all(new_users)\ntrue\n\n# none of the entries is inserted if at least one key already exists\niex> Blog.Cache.put_new_all(new_users)\nfalse\n```","ref":"getting-started.html#inserting-new-entries-and-replacing-existing-ones"},{"type":"extras","title":"Retrieving entries - Getting Started","doc":"Let’s start off with fetching data by the key, which is the most basic and\ncommon operation to retrieve data from a cache.\n\n```elixir\niex> Blog.Cache.get(1)\n_user_1\n\niex> for key <- 1..3 do\n...>   user = Blog.Cache.get(key)\n...>   user.first_name\n...> end\n[\"Galileo\", \"Charles\", \"Albert\"]\n```\n\nThere is a function `has_key?` to check if a key exist in cache:\n\n```elixir\niex> Blog.Cache.has_key?(1)\ntrue\n\niex> Blog.Cache.has_key?(10)\nfalse\n```\n\nRetrieving multiple entries\n\n```elixir\niex> Blog.Cache.get_all([1, 2, 3])\n_users\n```","ref":"getting-started.html#retrieving-entries"},{"type":"extras","title":"Updating entries - Getting Started","doc":"Nebulex provides `update` and `get_and_update` functions to update an\nentry value based on current one, for example:\n\n```elixir\niex> initial = %{id: 1, first_name: \"\", last_name: \"\"}\n\n# using `get_and_update`\niex> Blog.Cache.get_and_update(1, fn v ->\n...>   if v, do: {v, %{v | first_name: \"X\"}}, else: {v, initial}\n...> iex> end)\n{_old, _updated}\n\n# using `update`\niex> Blog.Cache.update(1, initial, &(%{&1 | first_name: \"Y\"}))\n_updated\n```","ref":"getting-started.html#updating-entries"},{"type":"extras","title":"Counters - Getting Started","doc":"The function `incr` is provided to increment or decrement a counter; by default,\na counter is initialized to `0`. Let's see how counters works:\n\n```elixir\n# by default, the counter is incremented by 1\niex> Blog.Cache.incr(:my_counter)\n1\n\n# but we can also provide a custom increment value\niex> Blog.Cache.incr(:my_counter, 5)\n6\n\n# to decrement the counter, just pass a negative value\niex> Blog.Cache.incr(:my_counter, -5)\n1\n```","ref":"getting-started.html#counters"},{"type":"extras","title":"Deleting entries - Getting Started","doc":"We’ve now covered inserting, reading and updating entries. Now let's see how to\ndelete an entry using Nebulex.\n\n```elixir\niex> Blog.Cache.delete(1)\n:ok\n```","ref":"getting-started.html#deleting-entries"},{"type":"extras","title":"Take - Getting Started","doc":"This is another way not only for deleting an entry but also for retrieving it\nbefore its delete it:\n\n```elixir\niex> Blog.Cache.take(1)\n_entry\n\n# returns `nil` if `key` doesn't exist\niex> Blog.Cache.take(\"nonexistent\")\nnil\n\n# same as previous one but raises `KeyError`\niex> Blog.Cache.take!(\"nonexistent\")\n```","ref":"getting-started.html#take"},{"type":"extras","title":"Info - Getting Started","doc":"The last thing we’ll cover in this guide is how to retrieve information about\ncached objects or the cache itself.","ref":"getting-started.html#info"},{"type":"extras","title":"Remaining TTL - Getting Started","doc":"```elixir\niex> Blog.Cache.ttl(1)\n_remaining_ttl\n\niex> Blog.Cache.ttl(\"nonexistent\")\nnil\n```","ref":"getting-started.html#remaining-ttl"},{"type":"extras","title":"Query and/or Stream entries - Getting Started","doc":"Nebulex provides functions to fetch, count, delete, or stream all entries from\ncache matching the given query.","ref":"getting-started.html#query-and-or-stream-entries"},{"type":"extras","title":"Fetch all entries from cache matching the given query - Getting Started","doc":"```elixir\n# by default, returns all keys\niex> Blog.Cache.all()\n_all_entries\n\n# fetch all entries and return the keys\niex> Blog.Cache.all(nil, return: :key)\n_keys\n\n# built-in queries in `Nebulex.Adapters.Local` adapter\niex> Blog.Cache.all(nil)\niex> Blog.Cache.all(:unexpired)\niex> Blog.Cache.all(:expired)\n\n# if we are using `Nebulex.Adapters.Local` adapter, the stored entry\n# is a tuple `{:entry, key, value, touched, ttl}`, then the match spec\n# could be something like:\niex> spec = [{{:_, :\"$1\", :\"$2\", :_, :_}, [{:>, :\"$2\", 10}], [{{:\"$1\", :\"$2\"}}]}]\niex> Blog.Cache.all(spec)\n_all_matched\n\n# using Ex2ms\niex> import Ex2ms\niex> spec =\n...>   fun do\n...>     {_, key, value, _, _} when value > 10 -> {key, value}\n...>   end\niex> Blog.Cache.all(spec)\n_all_matched\n```","ref":"getting-started.html#fetch-all-entries-from-cache-matching-the-given-query"},{"type":"extras","title":"Count all entries from cache matching the given query - Getting Started","doc":"For example, to get the total number of cached objects (cache size):\n\n```elixir\niex> Blog.Cache.count_all()\n_num_cached_entries\n```\n\n> By default, since none query is given to `count_all/2`, all entries\n  in cache match.\n\nIn the same way as `all/2`, you can pass a query to count only the matched\nentries:\n\n```elixir\n# using Ex2ms\niex> import Ex2ms\niex> spec =\n...>   fun do\n...>     {_, value, _, _} when rem(value, 2) == 0 -> true\n...>   end\niex> Blog.Cache.count_all(spec)\n_num_of_matched_entries\n```\n\n> The previous example assumes you are using the built-in local adapter.\n\nAlso, if you are using the built-in local adapter, you can use the queries\n`:expired` and `:unexpired` too, like so:\n\n```elixir\niex> expired_entries = Blog.Cache.count_all(:expired)\niex> unexpired_entries = Blog.Cache.count_all(:unexpired)\n```","ref":"getting-started.html#count-all-entries-from-cache-matching-the-given-query"},{"type":"extras","title":"Delete all entries from cache matching the given query - Getting Started","doc":"Similar to `count_all/2`, Nebulex provides `delete_all/2` to not only count\nthe matched entries but also remove them from the cache at once, in one single\nexecution.\n\nThe first example is flushing the cache, delete all cached entries (which is\nthe default behavior when none query is provided):\n\n```elixir\niex> Blog.Cache.delete_all()\n_num_of_removed_entries\n```\n\nAnd just like `count_all/2`, you can also provide a custom query to delete only\nthe matched entries, or if you are using the built-in local adapter you can also\nuse the queries `:expired` and `:unexpired`. For example:\n\n```elixir\niex> expired_entries = Blog.Cache.delete_all(:expired)\niex> unexpired_entries = Blog.Cache.delete_all(:unexpired)\n\n# using Ex2ms\niex> import Ex2ms\niex> spec =\n...>   fun do\n...>     {_, value, _, _} when rem(value, 2) == 0 -> true\n...>   end\niex> Blog.Cache.delete_all(spec)\n_num_of_matched_entries\n```\n\n> These examples assumes you are using the built-in local adapter.","ref":"getting-started.html#delete-all-entries-from-cache-matching-the-given-query"},{"type":"extras","title":"Stream all entries from cache matching the given query - Getting Started","doc":"Similar to `all/2` but returns a lazy enumerable that emits all entries from the\ncache matching the provided query.\n\nIf the query is `nil`, then all entries in cache match and are returned when the\nstream is evaluated; based on the `:return` option.\n\n```elixir\niex> Blog.Cache.stream()\niex> Blog.Cache.stream(nil, page_size: 100, return: :value)\niex> Blog.Cache.stream(nil, page_size: 100, return: :entry)\n\n# using `Nebulex.Adapters.Local` adapter\niex> spec = [{{:\"$1\", :\"$2\", :_, :_}, [{:>, :\"$2\", 10}], [{{:\"$1\", :\"$2\"}}]}]\niex> Blog.Cache.stream(spec)\n_all_matched\n\n# using Ex2ms\niex> import Ex2ms\niex> spec =\n...>   fun do\n...>     {key, value, _, _} when value > 10 -> {key, value}\n...>   end\niex> Blog.Cache.stream(spec)\n_all_matched\n```","ref":"getting-started.html#stream-all-entries-from-cache-matching-the-given-query"},{"type":"extras","title":"Partitioned Cache - Getting Started","doc":"Nebulex provides the adapter `Nebulex.Adapters.Partitioned`, which allows to\nset up a partitioned cache topology.\n\nLet's set up the partitioned cache by using the `mix` task `mix nbx.gen.cache`:\n\n```\nmix nbx.gen.cache -c Blog.PartitionedCache -a Nebulex.Adapters.Partitioned\n```\n\nAs we saw previously, this command will generate the cache in\n`lib/bolg/partitioned_cache.ex` (in this case using the partitioned adapter)\nmodule along with the initial configuration in `config/config.exs`.\n\nThe cache:\n\n```elixir\ndefmodule Blog.PartitionedCache do\n  use Nebulex.Cache,\n    otp_app: :blog,\n    adapter: Nebulex.Adapters.Partitioned,\n    primary_storage_adapter: Nebulex.Adapters.Local\nend\n```\n\nAnd the config:\n\n```elixir\nconfig :blog, Blog.PartitionedCache,\n  primary: [\n    # When using :shards as backend\n    backend: :shards,\n    # GC interval for pushing new generation: 12 hrs\n    gc_interval: :timer.hours(12),\n    # Max 1 million entries in cache\n    max_size: 1_000_000,\n    # Max 2 GB of memory\n    allocated_memory: 2_000_000_000,\n    # GC min timeout: 10 sec\n    gc_cleanup_min_timeout: :timer.seconds(10),\n    # GC max timeout: 10 min\n    gc_cleanup_max_timeout: :timer.minutes(10)\n  ]\n```\n\nAnd remember to add the new cache `Blog.PartitionedCache` to your application's\nsupervision tree (such as we did it previously):\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    Blog.Cache,\n    Blog.PartitionedCache\n  ]\n\n  ...\n```\n\nNow we are ready to start using our partitioned cache!","ref":"getting-started.html#partitioned-cache"},{"type":"extras","title":"Timeout option - Getting Started","doc":"The `Nebulex.Adapters.Partitioned` supports `:timeout` option, it is a value in\nmilliseconds for the command that will be executed.\n\n```elixir\niex> Blog.PartitionedCache.get(\"foo\", timeout: 10)\n_value\n\n# if the timeout is exceeded, then the current process will exit\niex> Blog.PartitionedCache.put(\"foo\", \"bar\", timeout: 10)\n# ** (EXIT) time out\n```\n\nTo learn more about how partitioned cache works, please check\n`Nebulex.Adapters.Partitioned` documentation, and also it is recommended see the\n[partitioned cache example](https://github.com/cabol/nebulex_examples/tree/master/partitioned_cache)","ref":"getting-started.html#timeout-option"},{"type":"extras","title":"Multilevel Cache - Getting Started","doc":"Nebulex also provides the adapter `Nebulex.Adapters.Multilevel`, which allows to\nsetup a multi-level caching hierarchy.\n\nFirst, let's set up the multi-level cache by using the `mix` task\n`mix nbx.gen.cache`:\n\n```\nmix nbx.gen.cache -c Blog.NearCache -a Nebulex.Adapters.Multilevel\n```\n\nBy default, the command generates a 2-level near-cache topology. The first\nlevel or `L1` using the built-in local adapter, and the second one or `L2`\nusing the built-in partitioned adapter.\n\nThe generated cache module `lib/blog/near_cache.ex`:\n\n```elixir\ndefmodule Blog.NearCache do\n  use Nebulex.Cache,\n    otp_app: :blog,\n    adapter: Nebulex.Adapters.Multilevel","ref":"getting-started.html#multilevel-cache"},{"type":"extras","title":"Cache Levels - Getting Started","doc":"# Default auto-generated L1 cache (local)\n  defmodule L1 do\n    use Nebulex.Cache,\n      otp_app: :blog,\n      adapter: Nebulex.Adapters.Local\n  end\n\n  # Default auto-generated L2 cache (partitioned cache)\n  defmodule L2 do\n    use Nebulex.Cache,\n      otp_app: :blog,\n      adapter: Nebulex.Adapters.Partitioned\n  end","ref":"getting-started.html#cache-levels"},{"type":"extras","title":"TODO: Add, remove or modify the auto-generated cache levels above - Getting Started","doc":"end\n```\n\nAnd the configuration (`config/config.exs`):\n\n```elixir\nconfig :blog, Blog.NearCache,\n  model: :inclusive,\n  levels: [\n    # Default auto-generated L1 cache (local)\n    {\n      Blog.NearCache.L1,\n      # GC interval for pushing new generation: 12 hrs\n      gc_interval: :timer.hours(12),\n      # Max 1 million entries in cache\n      max_size: 1_000_000\n    },\n    # Default auto-generated L2 cache (partitioned cache)\n    {\n      Blog.NearCache.L2,\n      primary: [\n        # GC interval for pushing new generation: 12 hrs\n        gc_interval: :timer.hours(12),\n        # Max 1 million entries in cache\n        max_size: 1_000_000\n      ]\n    }\n  ]\n```\n\n> Remember you can add `backend: :shards` to use Shards as backend.\n\nFinally, add the new cache `Blog.NearCache` to your application's supervision\ntree (such as we did it previously):\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    Blog.Cache,\n    Blog.PartitionedCache,\n    Blog.NearCache\n  ]\n\n  ...\n```\n\nLet's try it out!\n\n```elixir\niex> Blog.NearCache.put(\"foo\", \"bar\", ttl: :timer.hours(1))\n\"bar\"\n\niex> Blog.NearCache.get(\"foo\")\n\"bar\"\n```\n\nTo learn more about how multilevel-cache works, please check\n`Nebulex.Adapters.Multilevel` documentation, and also it is recommended see the\n[near cache example](https://github.com/cabol/nebulex_examples/tree/master/near_cache)","ref":"getting-started.html#todo-add-remove-or-modify-the-auto-generated-cache-levels-above"},{"type":"extras","title":"Next - Getting Started","doc":"* [Cache Usage Patterns via Nebulex.Caching](http://hexdocs.pm/nebulex/cache-usage-patterns.html) -\n   Annotations-based DSL to implement different cache usage patterns.","ref":"getting-started.html#next"},{"type":"extras","title":"Cache Usage Patterns via Nebulex.Caching","doc":"# Cache Usage Patterns via Nebulex.Caching\n\nThere are several common access patterns when using a cache. **Nebulex**\nsupports most of these patterns by means of [Nebulex.Caching][nbx_caching].\n\n[nbx_caching]: http://hexdocs.pm/nebulex/Nebulex.Caching.html\n\n> Most of the following documentation about caching patterns it based on\n  [EHCache Docs][EHCache]\n\n[EHCache]: https://github.com/ehcache/ehcache3/blob/master/docs/src/docs/asciidoc/user/caching-patterns.adoc","ref":"cache-usage-patterns.html"},{"type":"extras","title":"Cache-aside - Cache Usage Patterns via Nebulex.Caching","doc":"With the cache-aside pattern, application code uses the cache directly.\n\nThis means that application code which accesses the system-of-record (SoR)\nshould consult the cache first, and if the cache contains the data, then return\nthe data directly from the cache, bypassing the SoR. Otherwise, the application\ncode must fetch the data from the system-of-record, store the data in the cache,\nand then return it. When data is written, the cache must be updated along with\nthe system-of-record.","ref":"cache-usage-patterns.html#cache-aside"},{"type":"extras","title":"Reading values - Cache Usage Patterns via Nebulex.Caching","doc":"```elixir\nif value = MyCache.get(key) do\n  value\nelse\n  value = SoR.get(key) # maybe Ecto?\n  :ok = MyCache.put(key, value)\n  value\nend\n```","ref":"cache-usage-patterns.html#reading-values"},{"type":"extras","title":"Writing values - Cache Usage Patterns via Nebulex.Caching","doc":"```elixir\n:ok = MyCache.put(key, value)\nSoR.insert(key, value) # maybe Ecto?\n```\n\nAs you may have noticed, this is the default behavior for most of the caches,\nwe have to interact directly with the cache as well as the SoR (most likely the\nDatabase).","ref":"cache-usage-patterns.html#writing-values"},{"type":"extras","title":"Cache-as-SoR - Cache Usage Patterns via Nebulex.Caching","doc":"The cache-as-SoR pattern implies using the cache as though it were the\nprimary system-of-record (SoR).\n\nThe pattern delegates SoR reading and writing activities to the cache, so that\napplication code is (at least directly) absolved of this responsibility.\nTo implement the cache-as-SoR pattern, use a combination of the following\nread and write patterns:\n\n * **Read-through**\n\n * **Write-through**\n\nAdvantages of using the cache-as-SoR pattern are:\n\n * Less cluttered application code (improved maintainability through centralized\n   SoR read/write operations)\n\n * Choice of write-through or write-behind strategies on a per-cache basis\n\n * Allows the cache to solve the thundering-herd problem\n\nA disadvantage of using the cache-as-SoR pattern is:\n\n * Less directly visible code-path\n\nBut how to get all this out-of-box? This is where declarative annotation-based\ncaching comes in. Nebulex provides a set of annotation to abstract most of the\nlogic behind **Read-through** and **Write-through** patterns and make the\nimplementation extremely easy. But let's go over these patterns more in detail\nand how to implement them by using [Nebulex annotations][nbx_caching].","ref":"cache-usage-patterns.html#cache-as-sor"},{"type":"extras","title":"Read-through - Cache Usage Patterns via Nebulex.Caching","doc":"Under the read-through pattern, the cache is configured with a loader component\nthat knows how to load data from the system-of-record (SoR).\n\nWhen the cache is asked for the value associated with a given key and such an\nentry does not exist within the cache, the cache invokes the loader to retrieve\nthe value from the SoR, then caches the value, then returns it to the caller.\n\nThe next time the cache is asked for the value for the same key it can be\nreturned from the cache without using the loader (unless the entry has been\nevicted or expired).\n\nThis pattern can be easily implemented using `cache` decorator as follows:\n\n```elixir\ndefmodule MyApp.Example do\n  use Nebulex.Caching\n\n  alias MyApp.Cache\n\n  @ttl :timer.hours(1)\n\n  @decorate cacheable(cache: Cache, key: name)\n  def get_by_name(name) do\n    # your logic (the loader to retrieve the value from the SoR)\n  end\n\n  @decorate cacheable(cache: Cache, key: age, opts: [ttl: @ttl])\n  def get_by_age(age) do\n    # your logic (the loader to retrieve the value from the SoR)\n  end\n\n  @decorate cacheable(cache: Cache)\n  def all(query) do\n    # your logic (the loader to retrieve the value from the SoR)\n  end\nend\n```\n\nAs you can see, the loader to retrieve the value from the system-of-record (SoR)\nis the function logic itself.","ref":"cache-usage-patterns.html#read-through"},{"type":"extras","title":"Write-through - Cache Usage Patterns via Nebulex.Caching","doc":"Under the write-through pattern, the cache is configured with a writer component\nthat knows how to write data to the system-of-record (SoR).\n\nWhen the cache is asked to store a value for a key, the cache invokes the writer\nto store the value in the SoR, as well as updating (or deleting) the cache.\n\nThis pattern can be implemented using `defevict` or `defupdatable`. When the\ndata is written to the system-of-record (SoR), you can update the cached value\nassociated with the given key using `defupdatable`, or just delete it using\n`defevict`.\n\n```elixir\ndefmodule MyApp.Example do\n  use Nebulex.Caching\n\n  alias MyApp.Cache\n\n  # When the data is written to the SoR, it is updated in the cache\n  @decorate cache_put(cache: Cache, key: something)\n  def update(something) do\n    # Write data to the SoR (most likely the Database)\n  end\n\n  # When the data is written to the SoR, it is deleted (evicted) from the cache\n  @decorate cache_evict(cache: Cache, key: something)\n  def update_something(something) do\n    # Write data to the SoR (most likely the Database)\n  end\nend\n```\n\nAs you can see, the logic to write data to the system-of-record (SoR) is the\nfunction logic itself.","ref":"cache-usage-patterns.html#write-through"},{"type":"extras","title":"Telemetry","doc":"# Telemetry\n\nThis guide is not focused on explaining `:telemetry` itself, how it works, how\nto configure it, and so on. Instead, we will show you how to instrument and\nreport on Cache Telemetry events in your application when using Nebulex.\n\nFor more information about `:telemetry`, you can check the\n[documentation][telemetry], or the [Phoenix Telemetry][phx_telemetry]\nguide is also recommended.\n\n[telemetry]: https://github.com/beam-telemetry/telemetry\n[phx_telemetry]: https://hexdocs.pm/phoenix/telemetry.html","ref":"telemetry.html"},{"type":"extras","title":"Telemetry Events - Telemetry","doc":"Many Elixir libraries (including Nebulex) are already using the `:telemetry`\npackage as a way to give users more insight into the behavior of their\napplications, by emitting events at key moments in the application lifecycle.","ref":"telemetry.html#telemetry-events"},{"type":"extras","title":"Nebulex built-in events - Telemetry","doc":"The following events are emitted by all Nebulex caches:\n\n  * `[:nebulex, :cache, :init]` - it is dispatched whenever a cache starts.\n\n    * Measurement: `%{system_time: System.monotonic_time()}`\n    * Metadata: `%{cache: atom, opts: [term]}`","ref":"telemetry.html#nebulex-built-in-events"},{"type":"extras","title":"Adapter-specific events - Telemetry","doc":"Nebulex currently suggests the adapters to dispatch the following Telemetry\nevents:\n\n  * `[:my_app, :cache, :command, :start]` - Dispatched by the underlying cache\n    adapter before an adapter callback is executed.\n\n    * Measurement: `%{system_time: System.monotonic_time()}`\n    * Metadata: `%{adapter_meta: map, function_name: atom, args: [term]}`\n\n  * `[:my_app, :cache, :command, :stop]` - Dispatched by the underlying cache\n    adapter after an adapter callback has been successfully executed.\n\n    * Measurement: `%{duration: native_time}`\n    * Metadata:\n\n      ```elixir\n      %{\n        adapter_meta: map,\n        function_name: atom,\n        args: [term],\n        result: term\n      }\n      ```\n\n  * `[:my_app, :cache, :command, :exception]` - Dispatched by the underlying\n    cache adapter when an exception is raised while the adapter callback is\n    executed.\n\n    * Measurement: `%{duration: native_time}`\n    * Metadata:\n\n      ```elixir\n      %{\n        adapter_meta: map,\n        function_name: atom,\n        args: [term],\n        kind: :error | :exit | :throw,\n        reason: term,\n        stacktrace: term\n      }\n      ```","ref":"telemetry.html#adapter-specific-events"},{"type":"extras","title":"Nebulex Metrics - Telemetry","doc":"Assuming you have defined the cache `MyApp.Cache` with the default\n`:telemetry_prefix`, using `Telemetry.Metrics`, you can define a\ncounter metric, which counts how many cache commands were completed:\n\n```elixir\nTelemetry.Metrics.counter(\"my_app.cache.command.stop.duration\")\n```\n\nor you could use a distribution metric to see how many commands were completed\nin particular time buckets:\n\n```elixir\nTelemetry.Metrics.distribution(\n  \"my_app.cache.command.stop.duration\",\n  buckets: [100, 200, 300]\n)\n```\n\nSo far, these metrics are only helpful to be able to see just the total number\nof executed cache commands. What if you wanted to see the average command\nduration, minimum and maximum, or percentiles, but aggregated per command\nor callback name? In this case, one could define a summary metric like so:\n\n```elixir\nTelemetry.Metrics.summary(\n  \"my_app.cache.command.stop.duration\",\n  unit: {:native, :millisecond},\n  tags: [:function_name]\n)\n```\n\nAs it is described above in the **\"Adapter-specific events\"** section, the event\nincludes the invoked callback name into the metadata as `:function_name`, then\nwe can add it to the metric's tags.","ref":"telemetry.html#nebulex-metrics"},{"type":"extras","title":"Extracting tag values from adapter's metadata - Telemetry","doc":"Let's add another metric for the command event, this time to group by **cache**,\n**adapter**, and **function_name** (adapter's callback):\n\n```elixir\nTelemetry.Metrics.summary(\n  \"my_app.cache.command.stop.duration\",\n  unit: {:native, :millisecond},\n  tags: [:cache, :adapter, :function_name],\n  tag_values:\n    &Map.merge(&1, %{\n      cache: &1.adapter_meta.cache,\n      adapter: &1.adapter_meta.cache.__adapter__()\n    })\n)\n```\n\nWe've introduced the `:tag_values` option here, because we need to perform a\ntransformation on the event metadata in order to get to the values we need.","ref":"telemetry.html#extracting-tag-values-from-adapter-s-metadata"},{"type":"extras","title":"Cache Stats - Telemetry","doc":"Each adapter is responsible for providing stats by implementing\n`Nebulex.Adapter.Stats` behaviour. However, Nebulex provides a simple default\nimplementation using [Erlang counters][erl_counters], which is used by\nthe built-in local adapter. The local adapter uses\n`Nebulex.Telemetry.StatsHandler` to aggregate the stats and keep\nthem updated, therefore, it requires the Telemetry events are dispatched\nby the adapter, otherwise, it won't work properly.\n\n[erl_counters]: https://erlang.org/doc/man/counters.html\n\nFurthermore, when the `:stats` option is enabled, we can use Telemetry for\nemitting the current stat values.\n\nFirst of all, make sure you have added `:telemetry`, `:telemetry_metrics`, and\n`:telemetry_poller` packages as dependencies to your `mix.exs` file.\n\nLet's define out cache module:\n\n```elixir\ndefmodule MyApp.Cache do\n  use Nebulex.Cache,\n    otp_app: :my_app,\n    adapter: Nebulex.Adapters.Local\nend\n```\n\nMake sure the `:stats` option is set to `true`, for example in the\nconfiguration:\n\n```elixir\nconfig :my_app, MyApp.Cache,\n  stats: true,\n  backend: :shards,\n  gc_interval: :timer.hours(12),\n  max_size: 1_000_000,\n  gc_cleanup_min_timeout: :timer.seconds(10),\n  gc_cleanup_max_timeout: :timer.minutes(10)\n```\n\nCreate your Telemetry supervisor at `lib/my_app/telemetry.ex`:\n\n```elixir\n# lib/my_app/telemetry.ex\ndefmodule MyApp.Telemetry do\n  use Supervisor\n  import Telemetry.Metrics\n\n  def start_link(arg) do\n    Supervisor.start_link(__MODULE__, arg, name: __MODULE__)\n  end\n\n  def init(_arg) do\n    children = [\n      # Configure `:telemetry_poller` for reporting the cache stats\n      {:telemetry_poller, measurements: periodic_measurements(), period: 10_000},\n\n      # For example, we use the console reporter, but you can change it.\n      # See `:telemetry_metrics` for for information.\n      {Telemetry.Metrics.ConsoleReporter, metrics: metrics()}\n    ]\n\n    Supervisor.init(children, strategy: :one_for_one)\n  end\n\n  defp metrics do\n    [\n      # Nebulex Stats Metrics\n      last_value(\"my_app.cache.stats.hits\", tags: [:cache]),\n      last_value(\"my_app.cache.stats.misses\", tags: [:cache]),\n      last_value(\"my_app.cache.stats.writes\", tags: [:cache]),\n      last_value(\"my_app.cache.stats.updates\", tags: [:cache]),\n      last_value(\"my_app.cache.stats.evictions\", tags: [:cache]),\n      last_value(\"my_app.cache.stats.expirations\", tags: [:cache])\n    ]\n  end\n\n  defp periodic_measurements do\n    [\n      {MyApp.Cache, :dispatch_stats, []}\n    ]\n  end\nend\n```\n\nThen add to your main application's supervision tree\n(usually in `lib/my_app/application.ex`):\n\n```elixir\nchildren = [\n  MyApp.Cache,\n  MyApp.Telemetry,\n  ...\n]\n```\n\nNow start an IEx session and call the server:\n\n```\niex(1)> MyApp.Cache.get 1\nnil\niex(2)> MyApp.Cache.put 1, 1, ttl: 10\n:ok\niex(3)> MyApp.Cache.get 1\n1\niex(4)> MyApp.Cache.put 2, 2\n:ok\niex(5)> MyApp.Cache.delete 2\n:ok\niex(6)> Process.sleep(20)\n:ok\niex(7)> MyApp.Cache.get 1\nnil\niex(2)> MyApp.Cache.replace 1, 11\ntrue\n```\n\nand you should see something like the following output:\n\n```\n[Telemetry.Metrics.ConsoleReporter] Got new event!\nEvent name: my_app.cache.stats\nAll measurements: %{evictions: 2, expirations: 1, hits: 1, misses: 2, updates: 1, writes: 2}\nAll metadata: %{cache: MyApp.Cache}\n\nMetric measurement: :hits (last_value)\nWith value: 1\nTag values: %{cache: MyApp.Cache}\n\nMetric measurement: :misses (last_value)\nWith value: 2\nTag values: %{cache: MyApp.Cache}\n\nMetric measurement: :writes (last_value)\nWith value: 2\nTag values: %{cache: MyApp.Cache}\n\nMetric measurement: :updates (last_value)\nWith value: 1\nTag values: %{cache: MyApp.Cache}\n\nMetric measurement: :evictions (last_value)\nWith value: 2\nTag values: %{cache: MyApp.Cache}\n\nMetric measurement: :expirations (last_value)\nWith value: 1\nTag values: %{cache: MyApp.Cache}\n```","ref":"telemetry.html#cache-stats"},{"type":"extras","title":"Custom metrics - Telemetry","doc":"In the same way, for instance, you can add another periodic measurement for\nreporting the cache size:\n\n```elixir\ndefmodule MyApp.Cache do\n  use Nebulex.Cache,\n    otp_app: :my_app,\n    adapter: Nebulex.Adapters.Local\n\n  def dispatch_cache_size do\n    :telemetry.execute(\n      [:my_app, :cache, :size],\n      %{value: size()},\n      %{cache: __MODULE__, node: node()}\n    )\n  end\nend\n```\n\nNow let's add a new periodic measurement to invoke `dispatch_cache_size()`\nthrough `:telemetry_poller`:\n\n```elixir\ndefp periodic_measurements do\n  [\n    {MyApp.Cache, :dispatch_stats, [[metadata: %{node: node()}]]},\n    {MyApp.Cache, :dispatch_cache_size, []}\n  ]\nend\n```\n\n> Notice the node name was added to the metadata so we can use it in the\n  metric's tags.\n\nMetrics:\n\n```elixir\ndefp metrics do\n  [\n    # Nebulex Stats Metrics\n    last_value(\"my_app.cache.stats.hits\", tags: [:cache, :node]),\n    last_value(\"my_app.cache.stats.misses\", tags: [:cache, :node]),\n    last_value(\"my_app.cache.stats.writes\", tags: [:cache, :node]),\n    last_value(\"my_app.cache.stats.updates\", tags: [:cache, :node]),\n    last_value(\"my_app.cache.stats.evictions\", tags: [:cache, :node]),\n    last_value(\"my_app.cache.stats.expirations\", tags: [:cache, :node]),\n\n    # Nebulex custom Metrics\n    last_value(\"my_app.cache.size.value\", tags: [:cache, :node])\n  ]\nend\n```\n\nIf you start an IEx session like previously, you should see the new metric too:\n\n```\n[Telemetry.Metrics.ConsoleReporter] Got new event!\nEvent name: my_app.cache.stats\nAll measurements: %{evictions: 0, expirations: 0, hits: 0, misses: 0, updates: 0, writes: 0}\nAll metadata: %{cache: MyApp.Cache, node: :nonode@nohost}\n\nMetric measurement: :hits (last_value)\nWith value: 0\nTag values: %{cache: MyApp.Cache, node: :nonode@nohost}\n\nMetric measurement: :misses (last_value)\nWith value: 0\nTag values: %{cache: MyApp.Cache, node: :nonode@nohost}\n\nMetric measurement: :writes (last_value)\nWith value: 0\nTag values: %{cache: MyApp.Cache, node: :nonode@nohost}\n\nMetric measurement: :updates (last_value)\nWith value: 0\nTag values: %{cache: MyApp.Cache, node: :nonode@nohost}\n\nMetric measurement: :evictions (last_value)\nWith value: 0\nTag values: %{cache: MyApp.Cache, node: :nonode@nohost}\n\nMetric measurement: :expirations (last_value)\nWith value: 0\nTag values: %{cache: MyApp.Cache, node: :nonode@nohost}\n\n[Telemetry.Metrics.ConsoleReporter] Got new event!\nEvent name: my_app.cache.size\nAll measurements: %{value: 0}\nAll metadata: %{cache: MyApp.Cache, node: :nonode@nohost}\n\nMetric measurement: :value (last_value)\nWith value: 0\nTag values: %{cache: MyApp.Cache, node: :nonode@nohost}\n```","ref":"telemetry.html#custom-metrics"},{"type":"extras","title":"Multi-level Cache Stats - Telemetry","doc":"When using the multi-level adapter the returned stats measurements may look\na bit different, because the multi-level adapter works as a wrapper for the\nconfigured cache levels, so the returned measurements are grouped and\nconsolidated by level. For example, suppose you have the cache:\n\n```elixir\ndefmodule MyApp.Multilevel do\n  use Nebulex.Cache,\n    otp_app: :nebulex,\n    adapter: Nebulex.Adapters.Multilevel\n\n  defmodule L1 do\n    use Nebulex.Cache,\n      otp_app: :nebulex,\n      adapter: Nebulex.Adapters.Local\n  end\n\n  defmodule L2 do\n    use Nebulex.Cache,\n      otp_app: :nebulex,\n      adapter: Nebulex.Adapters.Partitioned\n  end\nend\n```\n\nThen, when you run `MyApp.Multilevel.stats()` you get something like:\n\n```elixir\n%Nebulex.Stats{\n  measurements: %{\n    l1: %{evictions: 0, expirations: 0, hits: 0, misses: 0, updates: 0, writes: 0},\n    l2: %{evictions: 0, expirations: 0, hits: 0, misses: 0, updates: 0, writes: 0}\n  },\n  metadata: %{\n    l1: %{\n      cache: NMyApp.Multilevel.L1,\n      started_at: ~U[2021-01-10 13:06:04.075084Z]\n    },\n    l2: %{\n      cache: MyApp.Multilevel.L2.Primary,\n      started_at: ~U[2021-01-10 13:06:04.089888Z]\n    },\n    cache: MyApp.Multilevel,\n    started_at: ~U[2021-01-10 13:06:04.066750Z]\n  }\n}\n```\n\nAs you can see, the measurements map has the stats grouped by level, every key\nis an atom specifying the level and the value is a map with the stats and/or\nmeasurements for that level. Based on that, you could define the Telemetry\nmetrics in this way:\n\n```elixir\n[\n  # L1 metrics\n  last_value(\"my_app.cache.stats.l1.hits\",\n    event_name: \"my_app.cache.stats\",\n    measurement: &get_in(&1, [:l1, :hits]),\n    tags: [:cache]\n  ),\n  last_value(\"my_app.cache.stats.l1.misses\",\n    event_name: \"my_app.cache.stats\",\n    measurement: &get_in(&1, [:l1, :misses]),\n    tags: [:cache]\n  ),\n  last_value(\"my_app.cache.stats.l1.writes\",\n    event_name: \"my_app.cache.stats\",\n    measurement: &get_in(&1, [:l1, :writes]),\n    tags: [:cache]\n  ),\n  last_value(\"my_app.cache.stats.l1.updates\",\n    event_name: \"my_app.cache.stats\",\n    measurement: &get_in(&1, [:l1, :updates]),\n    tags: [:cache]\n  ),\n  last_value(\"my_app.cache.stats.l1.evictions\",\n    event_name: \"my_app.cache.stats\",\n    measurement: &get_in(&1, [:l1, :evictions]),\n    tags: [:cache]\n  ),\n  last_value(\"my_app.cache.stats.l1.expirations\",\n    event_name: \"my_app.cache.stats\",\n    measurement: &get_in(&1, [:l1, :expirations]),\n    tags: [:cache]\n  ),\n\n  # L2 metrics\n  last_value(\"my_app.cache.stats.l2.hits\",\n    event_name: \"my_app.cache.stats\",\n    measurement: &get_in(&1, [:l2, :hits]),\n    tags: [:cache]\n  ),\n  ...\n]\n```\n\nIf what you need is the aggregated stats for all levels, you can always define\nyour own function to emit the Telemetry events. You just need to call\n`MyApp.Multilevel.stats()` and then you add the logic to process the results\nin the way you need. On the other hand, if you are using Datadog through the\nStatsD reporter, you could do the aggregation directly in Datadog.","ref":"telemetry.html#multi-level-cache-stats"},{"type":"extras","title":"Migrating to v2.x","doc":"# Migrating to v2.x\n\nFor the v2, Nebulex introduces several breaking changes, so this guide aims to\nhighlight most of these changes to make easier the transition to v2. Be aware\nthis guide won't focus on every change, just the most significant ones that can\naffect how your application code interacts with the cache. Also, it is not a\ndetailed guide about how to translate the current code from older versions to\nv2, just pointing out the areas the new documentation should be consulted on.","ref":"migrating-to-v2.html"},{"type":"extras","title":"Configuration - Migrating to v2.x","doc":"This is one of the biggest changes. Version 1.x, most of the configuration\noptions are resolved in compile-time, which has a lot of limitations.\nSince version 2.x, only few arguments are configured in compile-time when\ndefining a cache, e.g.: `otp_app:`, `adapter:`, and `primary_storage_adapter:`\n(for partitioned and replicated adapters). The rest of configuration parameters\nare given via config file or at startup time. For more information and examples,\nsee `Nebulex.Cache`, `Nebulex.Adapters.Local`, `Nebulex.Adapters.Partitioned`,\n`Nebulex.Adapters.Replicated`, `Nebulex.Adapters.Multilevel`.","ref":"migrating-to-v2.html#configuration"},{"type":"extras","title":"Cache API - Migrating to v2.x","doc":"There are several changes on the `Nebulex.Cache` API:\n\n  * The `:return` option is not available anymore, so it has to be removed.\n  * The `:version` option is not available anymore, so it has to be removed.\n  * Callback `set/3` was refactored to `put/3`.\n  * Callback `set_many/2` was refactored to `put_all/2`.\n  * Callback `get_many/2` was refactored to `get_all/2`.\n  * Callbacks `add/3` and `add!/3` were refactored to `put_new/3` and\n    `put_new!/3`.\n  * Callback `update_counter/3` was refactored to `incr/3` and `decr/3`.\n  * Callback `add_or_replace/3` was removed.\n  * Callback `object_info/2` was removed, and callbacks `ttl/1` and\n    `touch/1` were added instead.","ref":"migrating-to-v2.html#cache-api"},{"type":"extras","title":"Declarative annotation-based caching via decorators - Migrating to v2.x","doc":"* Module `Nebulex.Caching.Decorators` was refactored to `Nebulex.Caching` –\n    Keep in mind that since v1.2.x the caching decorators were included instead\n    of the previous macros or DSL (this applies for version lower than v1.1.x).\n  * Decorator `cache/3` was refactored to `cacheable/3`.\n  * Decorator `evict/3` was refactored to `cache_evict/3`.\n  * Decorator `update/3` was refactored to `cache_put/3`.\n  * Improved the `:match` option to return not only a boolean but return a\n    specific value to be cached `(term -> boolean | {true, term})` – If `true`\n    the code-block evaluation result is cached as it is (the default). If\n    `{true, value}` is returned, then the `value` is what is cached.","ref":"migrating-to-v2.html#declarative-annotation-based-caching-via-decorators"},{"type":"extras","title":"Hooks - Migrating to v2.x","doc":"Since v2.x, pre/post hooks are deprecated and won't be longer supported by\n`Nebulex`, at least not directly. Mainly, because the hooks feature is not a\ncommon use-case and also it is something that can be be easily implemented\non top of the Cache at the application level. However, to keep backward\ncompatibility somehow, `Nebulex` provides decorators for implementing\npre/post hooks very easily. For that reason, it is highly recommended\nto removed all pre/post hooks related code and adapt it to the new way.\nSee `Nebulex.Hook` for more information.","ref":"migrating-to-v2.html#hooks"},{"type":"extras","title":"Built-In Adapters - Migrating to v2.x","doc":"There have been several and significant improvements on the built-in adapters,\nso it is also highly recommended to take a look at them;\n`Nebulex.Adapters.Local`, `Nebulex.Adapters.Partitioned`,\n`Nebulex.Adapters.Replicated`, and `Nebulex.Adapters.Multilevel`.\n\nIn case of using a distributed adapter, the module `Nebulex.Adapter.HashSlot`\nwas refactored to `Nebulex.Adapter.Keyslot` and the callback `keyslot /2` to\n`hash_slot/2`.","ref":"migrating-to-v2.html#built-in-adapters"},{"type":"extras","title":"Statistics - Migrating to v2.x","doc":"For older versions (<= 1.x), the stats were implemented via a post-hook and the\nmeasurements were oriented for counting the number of times a cache function is\ncalled. But what is interesting and useful to see is, for instance, the number\nof writes, hits, misses, evictions, etc. Therefore, the whole stats'\nfunctionality was refactored entirely.\n\n  1. This feature is not longer using pre/post hooks. Besides, pre/post hooks\n     are deprecated in v2.x.\n  2. The stats support is optional by implementing the `Nebulex.Adapter.Stats`\n     behaviour from the adapter. However, Nebulex provides a default\n     implementation using [Erlang counters][https://erlang.org/doc/man/counters.html]\n     which is supported by the local built-in adapter.\n     See the [Telemetry guide](http://hexdocs.pm/nebulex/telemetry.html) for\n     more information.\n  3. Since Nebulex 2.x on-wards, enabling stats is a matter of setting the\n     option `:stats` to `true`. See `Nebulex.Cache` for more information.","ref":"migrating-to-v2.html#statistics"},{"type":"extras","title":"Mix Tasks - Migrating to v2.x","doc":"* `mix nebulex.gen.cache` was refactored to `mix nbx.gen.cache`.\n  * `mix nebulex` was refactored to `mix nbx`.","ref":"migrating-to-v2.html#mix-tasks"},{"type":"extras","title":"Creating New Adapter","doc":"# Creating New Adapter\n\nThis guide will walk you through creating a custom Nebulex adapter. We will\nstart by creating a new project, making tests pass, and then implementing a\nsimple in-memory adapter. It will be roughly based on\n[`NebulexRedisAdapter`](https://github.com/cabol/nebulex_redis_adapter/) so you\ncan consult this repo as an example.","ref":"creating-new-adapter.html"},{"type":"extras","title":"Mix Project - Creating New Adapter","doc":"Nebulex's main repo contains some very useful shared tests that we are going to\ntake advantage of. To do so we will need to checkout Nebulex from Github as the\nversion published to Hex does not contain test code. To accommodate this\nworkflow we will start by creating a new project.\n\n```console\nmix new nebulex_memory_adapter\n```\n\nNow let's modify `mix.exs` so that we could fetch Nebulex repository.\n\n```elixir\ndefmodule NebulexMemoryAdapter.MixProject do\n  use Mix.Project\n\n  @nbx_vsn \"2.5.2\"\n  @version \"0.1.0\"\n\n  def project do\n    [\n      app: :nebulex_memory_adapter,\n      version: @version,\n      elixir: \"~> 1.13\",\n      elixirc_paths: elixirc_paths(Mix.env()),\n      aliases: aliases(),\n      deps: deps(),\n    ]\n  end\n\n  # Run \"mix help compile.app\" to learn about applications.\n  def application do\n    [\n      extra_applications: [:logger]\n    ]\n  end\n\n  defp elixirc_paths(:test), do: [\"lib\", \"test/support\"]\n  defp elixirc_paths(_), do: [\"lib\"]\n\n  # Run \"mix help deps\" to learn about dependencies.\n  defp deps do\n    [\n      nebulex_dep(),\n      {:telemetry, \"~> 0.4 or ~> 1.0\", optional: true}\n    ]\n  end\n\n  defp nebulex_dep do\n    if path = System.get_env(\"NEBULEX_PATH\") do\n      {:nebulex, \"~> #{@nbx_vsn}\", path: path}\n    else\n      {:nebulex, \"~> #{@nbx_vsn}\"}\n    end\n  end\n\n  defp aliases do\n    [\n      \"nbx.setup\": [\n        \"cmd rm -rf nebulex\",\n        \"cmd git clone --depth 1 --branch v#{@nbx_vsn} https://github.com/cabol/nebulex\"\n      ]\n    ]\n  end\nend\n```\n\nAs you can see here we define a `mix nbx.setup` task that will fetch a Nebulex\nversion to a folder specified in `NEBULEX_PATH` environmental variable. Let's\nrun it.\n\n```console\nexport NEBULEX_PATH=nebulex\nmix nbx.setup\n```\n\nNow is the good time to fetch other dependencies\n\n```console\nmix deps.get\n```","ref":"creating-new-adapter.html#mix-project"},{"type":"extras","title":"Tests - Creating New Adapter","doc":"Before we start implementing our custom adapter, let's make our tests up and\nrunning.\n\nWe start by defining a cache that uses our adapter in\n`test/support/test_cache.ex`\n\n```elixir\ndefmodule NebulexMemoryAdapter.TestCache do\n  use Nebulex.Cache,\n    otp_app: :nebulex_memory_adapter,\n    adapter: NebulexMemoryAdapter\nend\n```\n\nWe won't be writing tests ourselves. Instead, we will use shared tests from the\nNebulex parent repo. To do so, we will create a helper module in\n`test/shared/cache_test.exs` that will `use` test suites for behaviour we are\ngoing to implement. The minimal set of behaviours is `Entry` and `Queryable` so\nwe'll go with them.\n\n```elixir\ndefmodule NebulexMemoryAdapter.CacheTest do\n  @moduledoc \"\"\"\n  Shared Tests\n  \"\"\"\n\n  defmacro __using__(_opts) do\n    quote do\n      use Nebulex.Cache.EntryTest\n      use Nebulex.Cache.QueryableTest\n    end\n  end\nend\n```\n\nLet's now edit `test/nebulex_memory_adapter_test.exs` and make it run those\nshared tests by calling `use NebulexMemoryAdapter.CacheTest`. We also need to\ndefine a setup that will start our cache process and put the `cache` and `name`\nkeys into the test context.\n\n```elixir\ndefmodule NebulexMemoryAdapterTest do\n  use ExUnit.Case, async: true\n  use NebulexMemoryAdapter.CacheTest\n\n  alias NebulexMemoryAdapter.TestCache, as: Cache\n\n  setup do\n    {:ok, pid} = Cache.start_link()\n    Cache.delete_all()\n    :ok\n\n    on_exit(fn ->\n      :ok = Process.sleep(100)\n      if Process.alive?(pid), do: Cache.stop(pid)\n    end)\n\n    {:ok, cache: Cache, name: Cache}\n  end\nend\n```\n\nNow it's time to grind through failing tests.\n\n```console\nmix test\n== Compilation error in file test/support/test_cache.ex ==\n** (ArgumentError) expected :adapter option given to Nebulex.Cache to list Nebulex.Adapter as a behaviour\n    (nebulex 2.4.2) lib/nebulex/cache/supervisor.ex:50: Nebulex.Cache.Supervisor.compile_config/1\n    test/support/test_cache.ex:2: (module)\n```\n\nLooks like our adapter needs to `Nebulex.Adapter` behaviour. Luckily, it's just\n2 callback that we can copy from `Nebulex.Adapters.Nil`\n\n```elixir\n# lib/nebulex_memory_adapter.ex\ndefmodule NebulexMemoryAdapter do\n  @behaviour Nebulex.Adapter\n\n  @impl Nebulex.Adapter\n  defmacro __before_compile__(_env), do: :ok\n\n  @impl Nebulex.Adapter\n  def init(_opts) do\n    child_spec = Supervisor.child_spec({Agent, fn -> :ok end}, id: {Agent, 1})\n    {:ok, child_spec, %{}}\n  end\nend\n```\n\nAnother try\n\n```console\nmix test\n== Compilation error in file test/nebulex_memory_adapter_test.exs ==\n** (CompileError) test/nebulex_memory_adapter_test.exs:3: module Nebulex.Cache.EntryTest is not loaded and could not be found\n    (elixir 1.13.2) expanding macro: Kernel.use/1\n    test/nebulex_memory_adapter_test.exs:3: NebulexMemoryAdapterTest (module)\n    expanding macro: NebulexMemoryAdapter.CacheTest.__using__/1\n    test/nebulex_memory_adapter_test.exs:3: NebulexMemoryAdapterTest (module)\n    (elixir 1.13.2) expanding macro: Kernel.use/1\n    test/nebulex_memory_adapter_test.exs:3: NebulexMemoryAdapterTest (module)\n```\n\nLooks like files from Nebulex parent repo aren't automatically compiled. Let's\naddress this in `test/test_helper.exs`\n\n```elixir\n# Nebulex dependency path\nnbx_dep_path = Mix.Project.deps_paths()[:nebulex]\n\nfor file <- File.ls!(\"#{nbx_dep_path}/test/support\"), file != \"test_cache.ex\" do\n  Code.require_file(\"#{nbx_dep_path}/test/support/\" <> file, __DIR__)\nend\n\nfor file <- File.ls!(\"#{nbx_dep_path}/test/shared/cache\") do\n  Code.require_file(\"#{nbx_dep_path}/test/shared/cache/\" <> file, __DIR__)\nend\n\nfor file <- File.ls!(\"#{nbx_dep_path}/test/shared\"), file != \"cache\" do\n  Code.require_file(\"#{nbx_dep_path}/test/shared/\" <> file, __DIR__)\nend\n\n# Load shared tests\nfor file <- File.ls!(\"test/shared\"), not File.dir?(\"test/shared/\" <> file) do\n  Code.require_file(\"./shared/\" <> file, __DIR__)\nend\n\nExUnit.start()\n```\n\nOne more attempt\n\n```console\nmix test\n< ... >\n 54) test put_all/2 puts the given entries using different data types at once (NebulexMemoryAdapterTest)\n     test/nebulex_memory_adapter_test.exs:128\n     ** (UndefinedFunctionError) function NebulexMemoryAdapter.TestCache.delete_all/0 is undefined or private. Did you mean:\n\n           * delete/1\n           * delete/2\n\n     stacktrace:\n       (nebulex_memory_adapter 0.1.0) NebulexMemoryAdapter.TestCache.delete_all()\n       test/nebulex_memory_adapter_test.exs:9: NebulexMemoryAdapterTest.__ex_unit_setup_0/1\n       test/nebulex_memory_adapter_test.exs:1: NebulexMemoryAdapterTest.__ex_unit__/2\n\n\n\nFinished in 0.2 seconds (0.2s async, 0.00s sync)\n54 tests, 54 failures\n```","ref":"creating-new-adapter.html#tests"},{"type":"extras","title":"Implementation - Creating New Adapter","doc":"Now that we have our failing tests we can write some implementation. We'll start\nby making `delete_all/0` work as it is called in the setup.\n\n```elixir\ndefmodule NebulexMemoryAdapter do\n  @behaviour Nebulex.Adapter\n  @behaviour Nebulex.Adapter.Queryable\n\n  @impl Nebulex.Adapter\n  defmacro __before_compile__(_env), do: :ok\n\n  @impl Nebulex.Adapter\n  def init(_opts) do\n    child_spec = Supervisor.child_spec({Agent, fn -> %{} end}, id: {Agent, 1})\n    {:ok, child_spec, %{}}\n  end\n\n  @impl Nebulex.Adapter.Queryable\n  def execute(adapter_meta, :delete_all, query, opts) do\n    deleted = Agent.get(adapter_meta.pid, &map_size/1)\n    Agent.update(adapter_meta.pid, fn _state -> %{} end)\n\n    deleted\n  end\nend\n```\n\nDid we make any progress?\n\n```console\nmix test\n< ... >\n\n 44) test decr/3 decrements a counter by the given amount with default (NebulexMemoryAdapterTest)\n     test/nebulex_memory_adapter_test.exs:355\n     ** (UndefinedFunctionError) function NebulexMemoryAdapter.update_counter/6 is undefined or private\n     stacktrace:\n       (nebulex_memory_adapter 0.1.0) NebulexMemoryAdapter.update_counter(%{cache: NebulexMemoryAdapter.TestCache, pid: #PID<0.549.0>}, :counter1, -1, :infinity, 10, [default: 10])\n       test/nebulex_memory_adapter_test.exs:356: (test)\n\n\n\nFinished in 5.7 seconds (5.7s async, 0.00s sync)\n54 tests, 44 failures\n```\n\nWe certainly did! From here you can continue to implement necessary callbacks\none-by-one or define them all in bulk. For posterity, we put a complete\n`NebulexMemoryAdapter` module here that passes all tests.\n\n```elixir\ndefmodule NebulexMemoryAdapter do\n  @behaviour Nebulex.Adapter\n  @behaviour Nebulex.Adapter.Entry\n  @behaviour Nebulex.Adapter.Queryable\n\n  @impl Nebulex.Adapter\n  defmacro __before_compile__(_env), do: :ok\n\n  @impl Nebulex.Adapter\n  def init(_opts) do\n    child_spec = Supervisor.child_spec({Agent, fn -> %{} end}, id: {Agent, 1})\n    {:ok, child_spec, %{}}\n  end\n\n  @impl Nebulex.Adapter.Entry\n  def get(adapter_meta, key, _opts) do\n    Agent.get(adapter_meta.pid, &Map.get(&1, key))\n  end\n\n  @impl Nebulex.Adapter.Entry\n  def get_all(adapter_meta, keys, _opts) do\n    Agent.get(adapter_meta.pid, &Map.take(&1, keys))\n  end\n\n  @impl Nebulex.Adapter.Entry\n  def put(adapter_meta, key, value, ttl, :put_new, opts) do\n    if get(adapter_meta, key, []) do\n      false\n    else\n      put(adapter_meta, key, value, ttl, :put, opts)\n      true\n    end\n  end\n\n  def put(adapter_meta, key, value, ttl, :replace, opts) do\n    if get(adapter_meta, key, []) do\n      put(adapter_meta, key, value, ttl, :put, opts)\n      true\n    else\n      false\n    end\n  end\n\n  def put(adapter_meta, key, value, _ttl, _on_write, _opts) do\n    Agent.update(adapter_meta.pid, &Map.put(&1, key, value))\n    true\n  end\n\n  @impl Nebulex.Adapter.Entry\n  def put_all(adapter_meta, entries, ttl, :put_new, opts) do\n    if get_all(adapter_meta, Map.keys(entries), []) == %{} do\n      put_all(adapter_meta, entries, ttl, :put, opts)\n      true\n    else\n      false\n    end\n  end\n\n  def put_all(adapter_meta, entries, _ttl, _on_write, _opts) do\n    entries = Map.new(entries)\n    Agent.update(adapter_meta.pid, &Map.merge(&1, entries))\n    true\n  end\n\n  @impl Nebulex.Adapter.Entry\n  def delete(adapter_meta, key, _opts) do\n    Agent.update(adapter_meta.pid, &Map.delete(&1, key))\n  end\n\n  @impl Nebulex.Adapter.Entry\n  def take(adapter_meta, key, _opts) do\n    value = get(adapter_meta, key, [])\n    delete(adapter_meta, key, [])\n    value\n  end\n\n  @impl Nebulex.Adapter.Entry\n  def update_counter(adapter_meta, key, amount, _ttl, default, _opts) do\n    Agent.update(adapter_meta.pid, fn state ->\n      Map.update(state, key, default + amount, fn v -> v + amount end)\n    end)\n\n    get(adapter_meta, key, [])\n  end\n\n  @impl Nebulex.Adapter.Entry\n  def has_key?(adapter_meta, key) do\n    Agent.get(adapter_meta.pid, &Map.has_key?(&1, key))\n  end\n\n  @impl Nebulex.Adapter.Entry\n  def ttl(_adapter_meta, _key) do\n    nil\n  end\n\n  @impl Nebulex.Adapter.Entry\n  def expire(_adapter_meta, _key, _ttl) do\n    true\n  end\n\n  @impl Nebulex.Adapter.Entry\n  def touch(_adapter_meta, _key) do\n    true\n  end\n\n  @impl Nebulex.Adapter.Queryable\n  def execute(adapter_meta, :delete_all, _query, _opts) do\n    deleted = execute(adapter_meta, :count_all, nil, [])\n    Agent.update(adapter_meta.pid, fn _state -> %{} end)\n\n    deleted\n  end\n\n  def execute(adapter_meta, :count_all, _query, _opts) do\n    Agent.get(adapter_meta.pid, &map_size/1)\n  end\n\n  def execute(adapter_meta, :all, _query, _opts) do\n    Agent.get(adapter_meta.pid, &Map.values/1)\n  end\n\n  @impl Nebulex.Adapter.Queryable\n  def stream(_adapter_meta, :invalid_query, _opts) do\n    raise Nebulex.QueryError, message: \"foo\", query: :invalid_query\n  end\n\n  def stream(adapter_meta, _query, opts) do\n    fun =\n      case Keyword.get(opts, :return) do\n        :value ->\n          &Map.values/1\n\n        {:key, :value} ->\n          &Map.to_list/1\n\n        _ ->\n          &Map.keys/1\n      end\n\n    Agent.get(adapter_meta.pid, fun)\n  end\nend\n```\n\nOf course, this isn't a useful adapter in any sense but it should be enough to\nget you started with your own.","ref":"creating-new-adapter.html#implementation"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}